Package: geometry
License: GPL (>= 3)
Title: Mesh Generation and Surface Tessellation
Authors@R: c(
    person("Jean-Romain", "Roussel" , role=c("cph", "ctb"), comment = "wrote tsearch function with QuadTrees"),
    person("C. B.", "Barber" , role="cph"),
    person("Kai", "Habel", role=c("cph","aut")),
    person("Raoul", "Grasman", role=c("cph","aut")),
    person("Robert B.", "Gramacy", role=c("cph","aut")),
    person("Pavlo", "Mozharovskyi", role=c("cph","aut")),
    person("David C.", "Sterratt", role=c("cph","aut","cre"), email="david.c.sterratt@ed.ac.uk", comment=c(ORCID="0000-0001-9092-9099")))
Description: Makes the 'Qhull' library <http://www.qhull.org>
    available in R, in a similar manner as in Octave and MATLAB. Qhull
    computes convex hulls, Delaunay triangulations, halfspace
    intersections about a point, Voronoi diagrams, furthest-site
    Delaunay triangulations, and furthest-site Voronoi diagrams. It
    runs in 2D, 3D, 4D, and higher dimensions. It implements the
    Quickhull algorithm for computing the convex hull. Qhull does not
    support constrained Delaunay triangulations, or mesh generation of
    non-convex objects, but the package does include some R functions
    that allow for this.
Version: 0.5.2
URL: https://davidcsterratt.github.io/geometry/
Date: 2025-02-08
BugReports: https://github.com/davidcsterratt/geometry/issues
Depends: R (>= 3.5.0)
Imports: magic, Rcpp, lpSolve, linprog
Suggests: spelling, testthat, rgl, R.matlab, interp
LinkingTo: Rcpp, RcppProgress
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-02-08 18:21:07 UTC; dcs
Author: Jean-Romain Roussel [cph, ctb] (wrote tsearch function with QuadTrees),
  C. B. Barber [cph],
  Kai Habel [cph, aut],
  Raoul Grasman [cph, aut],
  Robert B. Gramacy [cph, aut],
  Pavlo Mozharovskyi [cph, aut],
  David C. Sterratt [cph, aut, cre]
    (<https://orcid.org/0000-0001-9092-9099>)
Maintainer: David C. Sterratt <david.c.sterratt@ed.ac.uk>
Repository: CRAN
Date/Publication: 2025-02-08 22:10:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 13:52:51 UTC; unix
Archs: geometry.so.dSYM

Package: geometry
License: GPL (>= 3)
Title: Mesh Generation and Surface Tessellation
Authors@R: c(
    person("Jean-Romain", "Roussel" , role=c("cph", "ctb"), comment = "wrote tsearch function with QuadTrees"),
    person("C. B.", "Barber" , role="cph"),
    person("Kai", "Habel", role=c("cph","aut")),
    person("Raoul", "Grasman", role=c("cph","aut")),
    person("Robert B.", "Gramacy", role=c("cph","aut")),
    person("Pavlo", "Mozharovskyi", role=c("cph","aut")),
    person("David C.", "Sterratt", role=c("cph","aut","cre"), email="david.c.sterratt@ed.ac.uk", comment=c(ORCID="0000-0001-9092-9099")))
Description: Makes the 'Qhull' library <http://www.qhull.org>
    available in R, in a similar manner as in Octave and MATLAB. Qhull
    computes convex hulls, Delaunay triangulations, halfspace
    intersections about a point, Voronoi diagrams, furthest-site
    Delaunay triangulations, and furthest-site Voronoi diagrams. It
    runs in 2D, 3D, 4D, and higher dimensions. It implements the
    Quickhull algorithm for computing the convex hull. Qhull does not
    support constrained Delaunay triangulations, or mesh generation of
    non-convex objects, but the package does include some R functions
    that allow for this.
Version: 0.5.2
URL: https://davidcsterratt.github.io/geometry/
Date: 2025-02-08
BugReports: https://github.com/davidcsterratt/geometry/issues
Depends: R (>= 3.5.0)
Imports: magic, Rcpp, lpSolve, linprog
Suggests: spelling, testthat, rgl, R.matlab, interp
LinkingTo: Rcpp, RcppProgress
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-02-08 18:21:07 UTC; dcs
Author: Jean-Romain Roussel [cph, ctb] (wrote tsearch function with QuadTrees),
  C. B. Barber [cph],
  Kai Habel [cph, aut],
  Raoul Grasman [cph, aut],
  Robert B. Gramacy [cph, aut],
  Pavlo Mozharovskyi [cph, aut],
  David C. Sterratt [cph, aut, cre]
    (<https://orcid.org/0000-0001-9092-9099>)
Maintainer: David C. Sterratt <david.c.sterratt@ed.ac.uk>
Repository: CRAN
Date/Publication: 2025-02-08 22:10:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:24:36 UTC; unix
Archs: geometry.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(plot,convhulln)
S3method(plot,delaunayn)
S3method(plot,intersectn)
S3method(to.mesh3d,convhulln)
export("entry.value<-")
export(Unique)
export(bary2cart)
export(cart2bary)
export(cart2pol)
export(cart2sph)
export(convhulln)
export(delaunayn)
export(distmesh2d)
export(distmeshnd)
export(dot)
export(entry.value)
export(extprod3d)
export(feasible.point)
export(halfspacen)
export(inhulln)
export(intersectn)
export(matmax)
export(matmin)
export(matorder)
export(matsort)
export(mesh.dcircle)
export(mesh.diff)
export(mesh.drectangle)
export(mesh.dsphere)
export(mesh.hunif)
export(mesh.intersect)
export(mesh.union)
export(pol2cart)
export(polyarea)
export(rbox)
export(sph2cart)
export(surf.tri)
export(tetramesh)
export(to.mesh3d)
export(trimesh)
export(tsearch)
export(tsearchn)
importFrom(Rcpp,sourceCpp)
importFrom(graphics,box)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(graphics,segments)
importFrom(magic,ashift)
importFrom(magic,shift)
importFrom(utils,packageDescription)
useDynLib(geometry)

# Ellipse
A small demo library for calculating the area of an ellipse

### Installation
```
pip install geometry-543
```

### Get started
How to create an ellipse and calculate the area

```Python
from geometry_543 import Ellipse

# Instantiate an Ellipse object
e = Ellipse(10,13)

# Call the method for calculating the area
result = e.calc_area()
```

library(geometry)

## 2D example
ps1 <- rbind(c(0,   sqrt(3)),
             c(3/2, -sqrt(3)/2),
             c(-3/2, -sqrt(3)/2))
ps2 <- ps1
ps2[,2] <- -ps2[,2]

is <-  intersectn(ps1, ps2)
plot(is, asp=1)

## 3D example
ps1a <- rbox(2, C=0.5)
dt1a <- delaunayn(ps1a)
ps1b <- rbox(2, C=0.5) + 2
dt1b <- delaunayn(ps1b)
ps1 <- rbind(ps1a, ps1b)
dt1 <- rbind(dt1a, dt1b + nrow(ps1a))
tetramesh(dt1, ps1, alpha=0.5, col="yellow")

ps2 <-  rbox(2, C=0.5) + 0.5
dt2 <- delaunayn(ps2)

tetramesh(dt2, ps2, alpha=0.5, col="red", clear=FALSE)

vol <- 0
for (i in 1:nrow(dt1)) {
  for (j in 1:nrow(dt2)) {
    is <- intersectn(ps1[dt1[i,],], ps2[dt2[j,],])
    vol <- vol + is$ch$vol
  }
}
message(paste("Volume of overlap should be 0.125. It is:", vol))

##' If \code{x} and \code{y} are matrices, calculate the dot-product
##' along the first non-singleton dimension.  If the optional argument
##' \code{d} is given, calculate the dot-product along this
##' dimension. 
##' 
##' @title Compute the dot product of two vectors
##' @param x Matrix of vectors
##' @param y Matrix of vectors
##' @param d Dimension along which to calculate the dot product
##' @return Vector with length of \code{d}th dimension
##' @author David Sterratt
##' @keywords arith math array
##' @export
dot <- function(x, y, d=NULL) {
  if (is.vector(x)) 
    x <- matrix(x, ncol = 1)
  if (is.vector(y)) 
    y <- matrix(y, ncol = 1)
  ndim <- length(dim(x))
  
  ## Determine dimension along which to sum
  if (is.null(d)) {
    di <- which(dim(x) > 1)
    if (length(di == 0)) {
      d <- 1
    } else {
      d <- di[1]
    }
  }

  ## Check size of d
  if (d > ndim) {
    stop("d is larger than the number of dimensions of the data")
  }
  
  ## Use rowSums and colSums as they are more efficient than apply
  if (ndim == 2) {
    if (d == 2) {
      return(rowSums(x*y))
    }
    if (d == 1) {
      return(colSums(x*y))
    }
  }
  return(apply(x*y, d, sum))
}

##' Delaunay triangulation in N dimensions
##'
##' The Delaunay triangulation is a tessellation of the convex hull of
##' the points such that no \eqn{N}-sphere defined by the \eqn{N}-
##' triangles contains any other points from the set.
##'
##' @param p An \eqn{M}-by-\eqn{N} matrix whose rows represent \eqn{M}
##'   points in \eqn{N}-dimensional space.
##'
##' @param options String containing extra control options for the
##'   underlying Qhull command; see the Qhull documentation
##'   (\url{../doc/qhull/html/qdelaun.html}) for the available
##'   options.
##'
##'   The \code{Qbb} option is always passed to Qhull. The remaining
##'   default options are \code{Qcc Qc Qt Qz} for \eqn{N<4} and
##'   \code{Qcc Qc Qt Qx} for \eqn{N>=4}. If neither of the \code{QJ}
##'   or \code{Qt} options are supplied, the \code{Qt} option is
##'   passed to Qhull. The \code{Qt} option ensures all Delaunay
##'   regions are simplical (e.g., triangles in 2D). See
##'   \url{../doc/qhull/html/qdelaun.html} for more details. Contrary
##'   to the Qhull documentation, no degenerate (zero area) regions
##'   are returned with the \code{Qt} option since the R function
##'   removes them from the triangulation.
##'
##'   \emph{If \code{options} is specified, the default options are
##'   overridden.} It is recommended to use \code{output.options} for
##'   options controlling the outputs.
##'
##' @param output.options String containing Qhull options to control
##'   output. Currently \code{Fn} (neighbours) and \code{Fa} (areas)
##'   are supported. Causes an object of  return value for details. If
##'   \code{output.options} is \code{TRUE}, select all supported
##'   options.
##'
##' @param full Deprecated and will be removed in a future release.
##'   Adds options \code{Fa} and \code{Fn}.
##'
##' @return If \code{output.options} is \code{NULL} (the default),
##'   return the Delaunay triangulation as a matrix with \eqn{M} rows
##'   and \eqn{N+1} columns in which each row contains a set of
##'   indices to the input points \code{p}. Thus each row describes a
##'   simplex of dimension \eqn{N}, e.g. a triangle in 2D or a
##'   tetrahedron in 3D.
##'
##'   If the \code{output.options} argument is \code{TRUE} or is a
##'   string containing \code{Fn} or \code{Fa}, return a list with
##'   class \code{delaunayn} comprising the named elements:
##'   \describe{
##'     \item{\code{tri}}{The Delaunay triangulation described above}
##'     \item{\code{areas}}{If \code{TRUE} or if \code{Fa} is specified, an
##'       \eqn{M}-dimensional vector containing the generalised area of
##'       each simplex (e.g. in 2D the areas of triangles; in 3D the volumes
##'       of tetrahedra). See \url{../doc/qhull/html/qh-optf.html#Fa}.}
##'     \item{\code{neighbours}}{If \code{TRUE} or if \code{Fn} is specified,
##'       a list of neighbours of each simplex. Note that a negative number
##'       corresponds to "facet" (="edge" in 2D or "face" in 3D) that has no
##'       neighbour, as will be the case for some simplices on the boundary
##'       of the triangulation.
##'       See \url{../doc/qhull/html/qh-optf.html#Fn}}
##'   }
##'
##' @note This function interfaces the Qhull library and is a port
##'   from Octave (\url{https://octave.org/}) to R. Qhull computes
##'   convex hulls, Delaunay triangulations, halfspace intersections
##'   about a point, Voronoi diagrams, furthest-site Delaunay
##'   triangulations, and furthest-site Voronoi diagrams. It runs in
##'   2D, 3D, 4D, and higher dimensions. It implements the
##'   Quickhull algorithm for computing the convex hull. Qhull handles
##'   round-off errors from floating point arithmetic. It computes
##'   volumes, surface areas, and approximations to the convex
##'   hull. See the Qhull documentation included in this distribution
##'   (the doc directory \url{../doc/qhull/index.html}).
##'
##' Qhull does not support constrained Delaunay triangulations, triangulation
##' of non-convex surfaces, mesh generation of non-convex objects, or
##' medium-sized inputs in 9D and higher. A rudimentary algorithm for mesh
##' generation in non-convex regions using Delaunay triangulation is
##' implemented in \link{distmesh2d} (currently only 2D).
##' @author Raoul Grasman and Robert B. Gramacy; based on the
##' corresponding Octave sources of Kai Habel.
##' @seealso \code{\link[interp]{tri.mesh}}, \code{\link{convhulln}},
##' \code{\link{surf.tri}}, \code{\link{distmesh2d}}
##' @references \cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
##' \dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM Trans. on
##' Mathematical Software,} Dec 1996.}
##'
##' \url{http://www.qhull.org}
##' @keywords math dplot graphs
##' @examples
##'
##' # example delaunayn
##' d <- c(-1,1)
##' pc <- as.matrix(rbind(expand.grid(d,d,d),0))
##' tc <- delaunayn(pc)
##'
##' # example tetramesh
##' \dontrun{
##' rgl::view3d(60)
##' rgl::light3d(120,60)
##' tetramesh(tc,pc, alpha=0.9)
##' }
##'
##' tc1 <- delaunayn(pc, output.options="Fa")
##' ## sum of generalised areas is total volume of cube
##' sum(tc1$areas)
##'
##' @export
##' @useDynLib geometry
delaunayn <-
function(p, options=NULL, output.options=NULL, full=FALSE) {
  tmp_stdout <- tempfile("Rf")
  tmp_stderr <- tempfile("Rf")
  on.exit(unlink(c(tmp_stdout, tmp_stderr)))

  ## Coerce the input to be matrix
  if (is.data.frame(p)) {
    p <- as.matrix(p)
  }

  ## Make sure we have real-valued input
  storage.mode(p) <- "double"

  ## We need to check for NAs in the input, as these will crash the C
  ## code.
  if (any(is.na(p))) {
    stop("The first argument should not contain any NAs")
  }

  ## Default options
  if (is.null(options)) {
    if (ncol(p) < 4) {
      options <- "Qt Qc Qz"
    } else {
      options <- "Qt Qc Qx"
    }
  }

  ## Combine and check options
  options <- tryCatch(qhull.options(options, output.options, supported_output.options  <- c("Fa", "Fn"), full=full), error=function(e) {stop(e)})

  ## It is essential that delaunayn is called with either the QJ or Qt
  ## option. Otherwise it may return a non-triangulated structure, i.e
  ## one with more than dim+1 points per structure, where dim is the
  ## dimension in which the points p reside.
  if (!grepl("Qt", options) & !grepl("QJ", options)) {
    options <- paste(options, "Qt")
  }

  out <- .Call("C_delaunayn", p, as.character(options), tmp_stdout, tmp_stderr, PACKAGE="geometry")

  # Remove NULL elements
  out[which(sapply(out, is.null))] <- NULL
  if (is.null(out$areas) & is.null(out$neighbours)) {
    attr(out$tri, "delaunayn") <- attr(out$tri, "delaunayn")
    return(out$tri)
  }
  class(out) <- "delaunayn"
  out$p <- p
  return(out)
}

##' @importFrom graphics plot
##' @method plot delaunayn
##' @export
plot.delaunayn <- function(x, y, ...) {
  if (ncol(x$p) < 2 || ncol(x$p) > 3)
    stop("Only 2D and 3D convhullns can be plotted")
  args <- list(...)
  add <- FALSE
  if ("add" %in% names(args)) {
    add <- args$add
    args$add <- NULL
  }
  if (ncol(x$p) == 2) {
    if (!add) {
      plot(x$p[,1], x$p[,2], ...)
    }
    m <- Unique(rbind(x$tri[,1:2],
                      x$tri[,2:3],
                      x$tri[,c(1,3)]))
    p <- x$p
    do.call(segments, c(list(p[m[,1],1],p[m[,1],2],p[m[,2],1],p[m[,2],2]),
                        args))
  }
  if (ncol(x$p) == 3) {
    do.call(tetramesh, c(list(x$tri, x$p),
                         args))
  }
}

##  LocalWords:  param Qhull Fn delaunayn Qbb Qcc Qc Qz Qx QJ itemize
##  LocalWords:  tri Voronoi Quickhull distmesh Grasman Gramacy Kai
##  LocalWords:  Habel seealso interp convhulln Dobkin Huhdanpaa ACM
##  LocalWords:  dQuote emph dplot pc tc tetramesh dontrun useDynLib
# LocalWords:  eqn url math tmp stdout tempfile stderr unlink NAs na
# LocalWords:  tryCatch qhull grepl sapply attr importFrom convhullns
# LocalWords:  args rbind

context("Interaction with parallel package")
library(parallel)
test_that("delaunayn can be called with mc.apply", {
  ## mc.cores must be 1 on Windows. Otherwise use only 2 cores to comply
  ## with CRAN guidelines.
  mc.cores <- ifelse(Sys.info()[1] == "Windows", 1, 2)

  ## Set seed for replicability
  set.seed(1)

  ## Create points and try standard Delaunay Triangulation
  N <- 100000
  P <- matrix(runif(2*N), N, 2)
  T <- delaunayn(P)
  expect_identical(nrow(T), 199966L)

  ## Now try out the parallel version.
  Ts <- mclapply(list(P, P, P, P), delaunayn, mc.cores=mc.cores)
  expect_length(Ts, 4)
  expect_identical(nrow(Ts[[1]]), 199966L)
  expect_identical(Ts[[1]], T)
})

context("delaunayn")
test_that("delaunayn produces the correct output", {
  ## Create points that, when passed to Qhull with the Qt option,
  ## would give degenerate simplices - thanks to Bill Denney for
  ## example
  ps <- as.matrix(rbind(data.frame(a=0, b=0, d=0),
                        merge(merge(data.frame(a=c(-1, 1)),
                                    data.frame(b=c(-1, 1))),
                              data.frame(d=c(-1, 1)))))
  ts <- delaunayn(ps)
  expect_type(ts, "integer")
  expect_identical(dim(ts), c(12L, 4L))

  ## With output.options=TRUE, there should be a trinagulation, areas and
  ## neighbours and the sum of the ares should be 8
  ts.full <- delaunayn(ps, output.options=TRUE)
  expect_equal(ts, ts.full$tri, check.attributes=FALSE)
  expect_equal(length(ts.full$areas), nrow(ts.full$tri))
  expect_equal(length(ts.full$neighbours), nrow(ts.full$tri))
  expect_equal(sum(ts.full$area), 8)

  ## With full output, there should be a trinagulation, areas and
  ## neighbours and the sum of the ares should be 8
  ## full will be deprecated in a future version
  ts.full <- delaunayn(ps, full=TRUE)
  expect_equal(ts, ts.full$tri, check.attributes=FALSE)
  expect_equal(length(ts.full$areas), nrow(ts.full$tri))
  expect_equal(length(ts.full$neighbours), nrow(ts.full$tri))
  expect_equal(sum(ts.full$area), 8)

  ## tsearchn shouldn't return a "degnerate simplex" error.
  expect_silent(tsearchn(ps, ts, cbind(1, 2, 4)))

  ## If the input matrix contains NAs, delaunayn should return an error
  ps <- rbind(ps, NA)
  expect_error(delaunayn(ps))

})

test_that("In the case of just one triangle, delaunayn returns a matrix", {
  pc  <- rbind(c(0, 0), c(0, 1), c(1, 0))
  pct <- delaunayn(pc)
  expect_type(pct, "integer")
  expect_identical(dim(pct), c(1L, 3L))
  ## With no options it should also produce a triangulation. This
  ## mirrors the behaviour of octave and matlab
  pct <- delaunayn(pc, "")
  expect_type(pct, "integer")
  expect_identical(dim(pct), c(1L, 3L))

  pct.full <- delaunayn(pc, output.options=TRUE)
  expect_equal(pct.full$areas, 0.5)
})

test_that("In the case of a degenerate triangle, delaunayn returns a matrix with zero rows", {
  pc  <- rbind(c(0, 0), c(0, 1), c(0, 2))
  pct <- delaunayn(pc)
  expect_type(pct, "integer")
  expect_identical(dim(pct), c(0L, 3L))
  pct.full <- delaunayn(pc, output.options=TRUE)
  expect_equal(length(pct.full$areas), 0)
  expect_equal(length(pct.full$neighbours), 0)
})

test_that("In the case of just one tetrahaedron, delaunayn returns a matrix", {
  pc  <- rbind(c(0, 0, 0), c(0, 1, 0), c(1, 0, 0), c(0, 0, 1))
  pct <- delaunayn(pc)
  expect_type(pct, "integer")
  expect_identical(dim(pct), c(1L, 4L))
  pct.full <- delaunayn(pc, output.options=TRUE)
   expect_equal(pct.full$areas, 1/6)
})

test_that("Output to file works", {
  ps <-  matrix(rnorm(3000), ncol=3)
  ps <-  sqrt(3)*ps/drop(sqrt((ps^2) %*% rep(1, 3)))
  fname <- path.expand(file.path(tempdir(), "test1.txt"))
  pst <- delaunayn(ps, paste0("QJ TO '", fname, "'"))
  expect_true(file.exists(fname))
})

test_that("The QJ option can give degenerate simplices", {
  ## Create degenerate simplex - thanks to Bill Denney for example
  ps <- as.matrix(rbind(data.frame(a=0, b=0, d=0),
                        merge(merge(data.frame(a=c(-1, 1)),
                                    data.frame(b=c(-1, 1))),
                              data.frame(d=c(-1, 1)))))

  ## The QJ option leads to on simplex being very small
  ts <- delaunayn(ps, "QJ")
  expect_warning(tsearchn(ps, ts, cbind(1, 2, 4)))
})

test_that("A square is triangulated", {
  ## This doesn't work if the Qz option isn't supplied
  square <- rbind(c(0, 0), c(0, 1), c(1, 0), c(1, 1))
  expect_equal(delaunayn(square), rbind(c(4, 2, 1),
                                        c(4, 3, 1)),
               check.attributes=FALSE)
  expect_error(delaunayn(square, "", "QH6239 Qhull precision error: Initial simplex is cocircular or cospherical"))
})

##' Compute halfspace intersection about a point
##' 
##' @param p An \eqn{M}-by-\eqn{N+1} matrix. Each row of \code{p}
##'   represents a halfspace by a \eqn{N}-dimensional normal to a
##'   hyperplane and the offset of the hyperplane.
##' @param fp A \dQuote{feasible} point that is within the space
##'   contained within all the halfspaces.
##' @param options String containing extra options, separated by
##'   spaces, for the underlying Qhull command; see Qhull
##'   documentation at \url{../doc/qhull/html/qhalf.html}.
##' 
##' @return A \eqn{N}-column matrix containing the intersection
##'   points of the hyperplanes \url{../doc/qhull/html/qhalf.html}.
##' 
##' @author David Sterratt
##' @note \code{halfspacen} was introduced in geometry 0.4.0, and is
##'   still under development. It is worth checking results for
##'   unexpected behaviour.
##' @seealso \code{\link{convhulln}}
##' @references \cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
##'   \dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM
##'   Trans. on Mathematical Software,} Dec 1996.}
##' 
##' \url{http://www.qhull.org}
##' @examples
##' p <- rbox(0, C=0.5)  # Generate points on a unit cube centered around the origin
##' ch <- convhulln(p, "n") # Generate convex hull, including normals to facets, with "n" option
##' # Intersections of half planes
##' # These points should be the same as the orginal points
##' pn <- halfspacen(ch$normals, c(0, 0, 0)) 
##' 
##' @export
##' @useDynLib geometry
halfspacen <- function (p, fp, options = "Tv") {
  tmp_stdout <- tempfile("Rf")
  tmp_stderr <- tempfile("Rf")
  on.exit(c(tmp_stdout, tmp_stderr))
  
  ## Input sanitisation
  options <- paste(options, collapse=" ")
  
  ## Coerce the input to be matrix
  if (is.data.frame(p)) {
    p <- as.matrix(p)
  }

  ## Make sure we have real-valued input
  storage.mode(p) <- "double"

  ## We need to check for NAs in the input, as these will crash the C
  ## code.
  if (any(is.na(p))) {
    stop("The first argument should not contain any NAs")
  }

  ## Check dimensions
  if (ncol(p) - 1 != length(as.vector(fp))) {
    stop("Dimension of hyperspace is ", ncol(p) - 1, " but dimension of fixed point is ", length(as.vector(fp)))
  }
  

  ## In geometry 0.4.0, we tried to get around halspacen fails because
  ## of similar hyperplanes, by removing the most similar ones (i.e.
  ## those that had very acute angles to one another). However, this
  ## was ugly and turned out to unreliable , so it has been removed in
  ## geometry 0.4.1 and above. Users are recommended to supply the
  ## QJ option in 
  
  ## The fixed point is passed as an option
  out <- tryCatch(.Call("C_halfspacen", p,
                        as.character(paste(options, paste0("H",paste(fp, collapse=",")))),
                        tmp_stdout, tmp_stderr,
                        PACKAGE="geometry"),
                  error=function(e) {
                    if (grepl("^Received error code 2 from qhull.", e$message)) {
                      e$message <- paste(e$message, "\nTry calling halfspacen with options=\"Tv QJ\"")
                    }
                    return(e)
                  })
  if (inherits(out, "error")) {
    stop(out$message)
  }
  return(out)
}

## If there is an error, it could be because of two very similar halfspaces.
## n1 = ch1$normals[1,1:3]
## n2 = ch2$normals[1,1:3]
## d1 = ch1$normals[1,4]
## d2 = ch2$normals[1,4]
## solve(rbind(n1, n2, extprod3d(n1, n2)), c(d1, d2, 0))
## sqrt(sum(solve(rbind(n1, n2, extprod3d(n1, n2)), c(-d1, -d2, 0))^2))
## dot(n1+ n2, extprod3d(n1, n2))

context("halfspacen")
test_that("halfspacen works on a cube", {
  ## Cube with unit length edges, centred on the origin
  ps <- rbox(0, C=0.5)
  ## Convex hull. When "n" is specified normals should be returned
  ch <- convhulln(ps, "n")
  ## Intersections of half planes
  ## These points should be the same as the orginal points
  pn <- halfspacen(ch$normals, c(0, 0, 0))

  ## Convex hull of these points should have same characteristics as original cube
  ts <- convhulln(pn, "FA")
  expect_equal(ts$area, 6)
  expect_equal(ts$vol, 1)

  ## If the feasible point is outwith the normlas to the cube, an
  ## error should be thrown
  expect_error(halfspacen(ch$normals, c(1, 1, 1)))
  
})

test_that("halfspacen works on a cube with non triangulated facets", {
  ## Cube with unit length edges, centred on the origin
  ps <- rbox(0, C=0.5)
  ## Convex hull. When "n" is specified normals should be returned
  ch <- convhulln(ps, "n", return.non.triangulated.facets=TRUE)
  ## Intersections of half planes
  ## These points should be the same as the orginal points
  pn <- halfspacen(ch$normals, c(0, 0, 0))

  ## Convex hull of these points should have same characteristics as original cube
  ts <- convhulln(pn, "FA")
  expect_equal(ts$area, 6)
  expect_equal(ts$vol, 1)

  ## If the feasible point is outwith the normlas to the cube, an
  ## error should be thrown
  expect_error(halfspacen(ch$normals, c(1, 1, 1)))
})

test_that("halfspacen can compute volume of intersection of halfspaces", {
  ## Cube with unit length edges, centred on the origin
  ps1 <- rbox(0, C=0.5)

  ## Cube with unit length edges, centred on the (0.5, 0.5, 0.5)
  ps2 <- rbox(0, C=0.5) + 0.5

  ## Convex hulls with normals
  ch1 <- convhulln(ps1, "n", return.non.triangulated.facets=TRUE)
  ch2 <- convhulln(ps2, "n", return.non.triangulated.facets=TRUE)

  ## Intersection of merged halfspaces
  pn <- halfspacen(rbind(ch1$normals, ch2$normals), c(0.25, 0.25, 0.25))

  ## Convex hull of these points should be cube with vertices at
  ## intersection of cubes, i.e. a cube of length 0.5
  ts <- convhulln(pn, "FA")
  expect_equal(ts$area, 6*0.5^2)
  expect_equal(ts$vol, 1*0.5^3)
})

test_that("halfspacen can do the round trip on an example with 3000 points",
{
  set.seed(1)
  ps <- matrix(rnorm(3000), ncol=3)
  ps <- sqrt(3)*ps/drop(sqrt((ps^2) %*% rep(1,3)))
  ch <- convhulln(ps, "n FA")
  
  pn <- halfspacen(ch$normals, c(0, 0, 0))
  chn <- convhulln(pn, "n FA")
  
  expect_equal(ch$area, chn$area)
  expect_equal(ch$vol, chn$vol)
})

test_that("halfspacen throws an error when the feasible point is not clearly inside the halfspace",
{
  load(file.path(system.file(package="geometry"), "extdata", "halfspacen.RData"))
  expect_error(halfspacen(normals, fp), "QH6023 qhull input error")
})

##' Uniform desired edge length function of position to allow easy
##' definition of regions when passed as the \code{fh} argument of
##' \code{\link{distmesh2d}} or \code{\link{distmeshnd}}.
##'
##' @title Uniform desired edge length
##' @param p A \code{n}-by-\code{m} matrix, each row representing a
##' point in an \code{m}-dimensional space.
##' @param ... additional arguments (not used)
##' @return Vector of ones of length \code{n}.
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @seealso \code{\link{distmesh2d}} and \code{\link{distmeshnd}}.
##' @export
mesh.hunif <- function (p, ...) {
  if (!is.matrix(p)) 
    stop("Input `p' should be matrix.")
  return(rep(1, nrow(p)))
}

##' Find surface triangles from tetrahedral mesh
##'
##' Find surface triangles from tetrahedral mesh typically obtained
##' with \code{\link{delaunayn}}.
##'
##' \code{surf.tri} and \code{\link{convhulln}} serve a similar purpose in 3D,
##' but \code{surf.tri} also works for non-convex meshes obtained e.g. with
##' \code{\link{distmeshnd}}.  It also does not produce currently unavoidable
##' diagnostic output on the console as \code{convhulln} does at the Rterm
##' console--i.e., \code{surf.tri} is silent.
##'
##' @param p An \code{n}-by-\code{3} matrix. The rows of \code{p} represent
##' \code{n} points in \code{dim}-dimensional space.
##' @param t Matrix with 4 columns, interpreted as output of
##' \code{\link{delaunayn}}.
##' @return An \code{m}-by-\code{3} index matrix of which each row defines a
##' triangle. The indices refer to the rows in \code{p}.
##' @note \code{surf.tri} was based on Matlab code for mesh of Per-Olof Persson
##' (\url{http://persson.berkeley.edu/distmesh/}).
##' @author Raoul Grasman
##' @seealso \code{\link[interp]{tri.mesh}}, \code{\link{convhulln}},
##' \code{\link{surf.tri}}, \code{\link{distmesh2d}}
##' @keywords math optimize dplot
##' @examples
##'
##' \dontrun{
##' # more extensive example of surf.tri
##'
##' # url's of publically available data:
##' data1.url = "http://neuroimage.usc.edu/USCPhantom/mesh_data.bin"
##' data2.url = "http://neuroimage.usc.edu/USCPhantom/CT_PCS_trans.bin"
##'
##' meshdata = R.matlab::readMat(url(data1.url))
##' elec = R.matlab::readMat(url(data2.url))$eeg.ct2pcs/1000
##' brain = meshdata$mesh.brain[,c(1,3,2)]
##' scalp = meshdata$mesh.scalp[,c(1,3,2)]
##' skull = meshdata$mesh.skull[,c(1,3,2)]
##' tbr = t(surf.tri(brain, delaunayn(brain)))
##' tsk = t(surf.tri(skull, delaunayn(skull)))
##' tsc = t(surf.tri(scalp, delaunayn(scalp)))
##' rgl::triangles3d(brain[tbr,1], brain[tbr,2], brain[tbr,3],col="gray")
##' rgl::triangles3d(skull[tsk,1], skull[tsk,2], skull[tsk,3],col="white", alpha=0.3)
##' rgl::triangles3d(scalp[tsc,1], scalp[tsc,2], scalp[tsc,3],col="#a53900", alpha=0.6)
##' rgl::view3d(-40,30,.4,zoom=.03)
##' lx = c(-.025,.025); ly = -c(.02,.02);
##' rgl::spheres3d(elec[,1],elec[,3],elec[,2],radius=.0025,col='gray')
##' rgl::spheres3d( lx, ly,.11,radius=.015,col="white")
##' rgl::spheres3d( lx, ly,.116,radius=.015*.7,col="brown")
##' rgl::spheres3d( lx, ly,.124,radius=.015*.25,col="black")
##' }
##'
##' @export
"surf.tri" <-
function(p,t){
    # original by Per-Olof Persson (c) 2005 for MATLAB
    # ported to R and modified for efficiency by Raoul Grasman (c) 2005

    # construct all faces
    faces = rbind(t[,-4], t[,-3], t[,-2], t[,-1]);
    node4 = rbind(t[, 4], t[, 3], t[, 2], t[, 1]);

#    #original translated from MATLAB:
#    # select the faces that occur only once --> these are the surface boundary faces
#    faces = t(apply(faces,1,sort));                                         # sort each row
#    foo   = apply(faces,1,function(x) do.call("paste",as.list(x,sep=" "))); # makes a string from each row
#    vec   = table(foo);                                                     # tabulates the number of occurences of each string
#    ix    = sapply(names(vec[vec==1]),function(b) which(b==foo))            # obtain indices of faces with single occurence
#    tri   = faces[ix,];
#    node4 = node4[ix];


    # we wish to achieve
    #   > faces = t(apply(faces,1,sort));
    # but this is much too slow, we therefore use max.col and the fact
    # that there are only 3 columns in faces
    i.max = 3*(1:nrow(faces)-1) + max.col(faces)
    i.min = 3*(1:nrow(faces)-1) + max.col(-faces)
    faces = t(faces)
    faces = cbind(faces[i.min], faces[-c(i.max,i.min)], faces[i.max])
    ix = order(faces[,1], faces[,2], faces[,3])

    # Next, we wish to detect duplicated rows in faces, that is,
    #   > qx = duplicated(faces[ix,],MARGIN=1)              # logical indicating duplicates
    # but this is also much to slow, we therefore use the fact that
    # faces[ix,] has the duplicate rows ordered beneath each other
    # and the fact that each row occurs exactly once or twice
    fo = apply(faces[ix,],2,diff)
    dup = (abs(fo) %*% rep(1,3)) == 0        # a row of only zeros indicates duplicate
    dup = c(FALSE,dup)                       # first is never a duplicate
    qx = diff(dup)==0                        # only zero if two consecutive elems are not duplicates
    qx = c(qx, !dup[length(dup)])            # last row is either non-duplicate or should not be selected
    tri = faces[ix[qx],]                     # ix[qx] are indices of singly occuring faces
    node4 = node4[ix[qx]]

    # compute face orientations
    v1 = p[tri[,2],] - p[tri[,1],]; # edge vectors
    v2 = p[tri[,3],] - p[tri[,1],];
    v3 = p[node4,]   - p[tri[,1],];
    ix = which( apply(extprod3d(v1,v2) * v3, 1, sum) > 0 )
    tri[ix,c(2,3)] = tri[ix,c(3,2)]
    rownames(tri) = NULL
    tri
}

## Copyright (C) 2017 David Sterratt
## Copyright (C) 2000-2017 Kai Habel
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it){
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for (more details.){
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

##' Transform spherical coordinates to Cartesian coordinates
##'
##' The inputs \code{theta}, \code{phi}, and \code{r} must be the same
##' shape, or scalar.  If called with a single matrix argument then
##' each row of \code{S} represents the spherical coordinate
##' (\code{theta}, \code{phi}, \code{r}).
##'
##' @param theta describes the angle relative to the positive x-axis.
##' @param phi is the angle relative to the xy-plane.
##' @param r is the distance to the origin \code{(0, 0, 0)}.
##'
##' If only a single return argument is requested then return a matrix
##' \code{C} where each row represents one Cartesian coordinate
##' (\code{x}, \code{y}, \code{z}).
##' @seealso \code{\link{cart2sph}}, \code{\link{pol2cart}}, \code{\link{cart2pol}}
##' @author Kai Habel
##' @author David Sterratt
##' @export
sph2cart <- function(theta, phi=NULL, r=NULL) {
  if ((is.null(phi) & !is.null(r)) |
      (is.null(r) & !is.null(phi))) {
    stop("There should be 3 arguments (theta, phi, r) or one argument (theta)")
  }

  if (is.null(phi) & is.null(r)) {
    if (!(is.numeric(theta))) {
      stop("input must be matrix with 3 columns [theta, phi, r]")
    }
    if (!(is.matrix(theta) & (ncol(theta) == 3))) {
      stop("matrix input must have 3 columns [theta, phi, r]")
    }
    r <- theta[,3]
    phi <- theta[,2]
    theta <- theta[,1]
  } else {
    if (!is.numeric(theta) | !is.numeric(phi) | !is.numeric (r))
      stop("theta, phi, r must be numeric arrays of the same size, or scalar")

    if ( !(((length(theta) == length(phi)) | (length(theta) == 1) | (length(phi) == 1)) &
           ((length(theta) == length(r)) | (length(theta) == 1) | (length(r) == 1)) &
           ((length(phi) == length(r)) | (length(phi) == 1) | (length(r) == 1)))) {
      stop("theta, phi, r must be matrices of the same size, or scalar")
    }
  }

  x <- r*cos(phi)*cos(theta)
  y <- r*cos(phi)*sin(theta)
  z <- r*sin(phi)

  return(cbind(x, y, z))
}

context("sph2cart")
test_that("sph2cart works correctly", {
  t <- c(0, 0, 0)
  p <- c(0, 0, 0)
  r <- c(0, 1, 2)
  C <- sph2cart (t, p, r)
  expect_equal(C[,"x"], r)
  expect_equal(C[,"y"], c(0, 0, 0))
  expect_equal(C[,"z"], c(0, 0, 0))

  t <- 0
  p <- c(0, 0, 0)
  r <- c(0, 1, 2)
  C <- sph2cart(t, p, r)
  expect_equal(C[,"x"], r)
  expect_equal(C[,"y"], c(0, 0, 0))
  expect_equal(C[,"z"], c(0, 0, 0))

  t <- c(0, 0, 0)
  p <- 0
  r <- c(0, 1, 2)
  C <- sph2cart (t, p, r)
  expect_equal(C[,"x"], r)
  expect_equal(C[,"y"], c(0, 0, 0))
  expect_equal(C[,"z"], c(0, 0, 0))

  t <- c(0, 0.5, 1)*pi
  p <- c(0, 0, 0)
  r <- 1
  C <- sph2cart(t, p, r)
  expect_equal(C[,"x"], c(1, 0, -1))
  expect_equal(C[,"y"], c(0, 1, 0))
  expect_equal(C[,"z"], c(0, 0, 0))

  C <- sph2cart(c(0, 0, 0), 0, 1)
  expect_equal(C[,"x"], c(1, 1, 1))
  expect_equal(C[,"y"], c(0, 0, 0))
  expect_equal(C[,"z"], c(0, 0, 0))

  S <- rbind(c(0, 0, 1),
             c(0.5*pi, 0, 1),
             c(pi, 0, 1))
  C <- rbind(c(x=1, y=0, z=0),
             c(0, 1, 0),
             c(-1, 0, 0))
  expect_equal(sph2cart(S), C)
})
  
# FIXME: to implement
#! c(t, p, r) <- meshgrid (c(0, pi/2), c(0, pi/2), c(0, 1))
#! c(x, y, z) <- sph2cart (t, p, r)
#! X <- zeros(2, 2, 2)
#! X(1, 1, 2) <- 1
#! Y <- zeros(2, 2, 2)
#! Y(1, 2, 2) <- 1
#! Z <- zeros(2, 2, 2)
#! Z(2, :, 2) <- c(1 1)
#! expect_equal(x, X, eps)
#! expect_equal(y, Y, eps)
#! expect_equal(z, Z)
  
test_that("sph2cart error validation works correctly", {
  expect_error(sph2cart())
  expect_error(sph2cart(1,2))
  expect_error(sph2cart(1,2,3,4))
  expect_error(sph2cart(list(1, 2, 3)), regexp="input must be matrix with 3 columns")
  expect_error(sph2cart(array(1, c(3,3,2))), regexp="matrix input must have 3 columns")
  expect_error(sph2cart(cbind(1,2,3,4)), regexp=c("matrix input must have 3 columns"))
  expect_error(sph2cart(list(1,2,3), c(1,2,3), c(1,2,3)), regexp="numeric arrays of the same size")
  expect_error(sph2cart(c(1,2,3), list(1,2,3), c(1,2,3), regexp="numeric arrays of the same size"))
  expect_error(sph2cart(c(1,2,3), c(1,2,3), list(1,2,3)), regexp="numeric arrays of the same size")
  expect_error(sph2cart(array(1, c(3, 3, 3)), 1, array(1, c(3,3,2))), regexp="matrices of the same size")
  expect_error(sph2cart(array(1, c(3, 3, 3)), array(1, c(3,3,2)), 1), regexp="matrices of the same size")
})

##' Default is corners of a hypercube.
##' @title Generate various point distributions
##' @param n number of random points in hypercube
##' @param D number of dimensions of hypercube
##' @param B bounding box coordinate - faces will be \code{-B} and \code{B} from origin
##' @param C add a unit hypercube to the output - faces will be \code{-C} and \code{C} from origin
##' @return Matrix of points
##' @author David Sterratt
##' @export
rbox <- function(n=3000, D=3, B=0.5, C=NA) {
  P <- matrix(0, 0, D)
  if (!is.na(C)) {
    P <- rbind(P, 
               as.matrix(do.call(expand.grid, rep(list(c(-C, C)), D))))
  }
  if (n > 0) {
    P <- rbind(P, 
               matrix(stats::runif(n=n*D, min=-B, max=B), n, D))
  }
  return(P)
}

context("convhulln")
test_that("convhulln works on a cube", {
  ## Cube with unit length edges, centred on the origin
  ps <- rbox(0, C=0.5)
  ts <- convhulln(ps)
  ## Expect 12 facets, since faceted output is produced by default
  expect_equal(nrow(ts), 12)
  ## When "FA" is specified area and volume should be returned
  ts <- convhulln(ps, "FA")
  expect_equal(length(ts), 4)
  expect_equal(ts$area, 6)
  expect_equal(ts$vol, 1)
  ## When "n" is specified normals should be returned
  ts <- convhulln(ps, "n")
  expect_equal(length(ts), 3)
  ## There are 12 normals, one for each facet. There are 6 *unique*
  ## normals, since for each face of the cube there are two triangular
  ## facets with the same normal
  expect_equal(ts$normals,
               rbind(c(  0,   0,   -1, -0.5),
                     c(  0,   0,   -1, -0.5),
                     c(  0,  -1,    0, -0.5),
                     c(  0,  -1,    0, -0.5),
                     c(  1,   0,    0, -0.5),
                     c(  1,   0,    0, -0.5),
                     c( -1,   0,    0, -0.5),
                     c( -1,   0,    0, -0.5),
                     c(  0,   1,    0, -0.5),
                     c(  0,   1,    0, -0.5),
                     c(  0,   0,    1, -0.5),
                     c(  0,   0,    1, -0.5)))

})

test_that("convhulln works on a cube with output.options", {
  ## Cube with unit length edges, centred on the origin
  ps <- rbox(0, C=0.5)
  ts <- convhulln(ps)
  ## Expect 12 facets, since faceted output is produced by default
  expect_equal(nrow(ts), 12)
  ## When "FA" is specified area and volume should be returned
  ts <- convhulln(ps, output.options="FA")
  expect_equal(length(ts), 4)
  expect_equal(ts$area, 6)
  expect_equal(ts$vol, 1)
  ## When "n" is specified normals should be returned
  ts <- convhulln(ps, output.options="n")
  expect_equal(length(ts), 3)
  ## There are 12 normals, one for each facet. There are 6 *unique*
  ## normals, since for each face of the cube there are two triangular
  ## facets with the same normal
  expect_equal(ts$normals,
               rbind(c(  0,   0,   -1, -0.5),
                     c(  0,   0,   -1, -0.5),
                     c(  0,  -1,    0, -0.5),
                     c(  0,  -1,    0, -0.5),
                     c(  1,   0,    0, -0.5),
                     c(  1,   0,    0, -0.5),
                     c( -1,   0,    0, -0.5),
                     c( -1,   0,    0, -0.5),
                     c(  0,   1,    0, -0.5),
                     c(  0,   1,    0, -0.5),
                     c(  0,   0,    1, -0.5),
                     c(  0,   0,    1, -0.5)))


  ts <- convhulln(ps, output.options=TRUE)
  expect_equal(length(ts), 5)
})


test_that("convhulln can run on an example with 3000 points", {
  set.seed(1)
  ps <- matrix(rnorm(3000), ncol=3)
  ps <- sqrt(3)*ps/drop(sqrt((ps^2) %*% rep(1,3)))
  ts <- convhulln(ps)
  expect_identical(nrow(ts), 1996L)
  ts.full <- convhulln(ps, "FA")
  expect_equal(ts.full$area, 37.47065, tolerance=0.001)
  expect_equal(ts.full$vol, 21.50165, tolerance=0.001)
})

test_that("convhulln throws an error with duplicated points", {
  load(file.path(system.file(package="geometry"), "extdata", "ordination.Rdata"))
  expect_error(out <- convhulln(ordination), "QH6114 qhull precision error: initial simplex is not convex")

})

test_that("If the input matrix contains NAs, convhulln should return an error", {
  ps <- matrix(rnorm(999), ncol=3)
  ps <- sqrt(3)*ps/drop(sqrt((ps^2) %*% rep(1,3)))
  ps <- rbind(ps, NA)
  expect_error(convhulln(ps))
})

test_that("If there are not enough points to construct a simplex, an error is thrown", {
  expect_error(convhulln(diag(4)))
})

test_that("Output to file works", {
  ## To prevent regression in package betapart
  fname <- path.expand(file.path(tempdir(), "vert.txt"))
  unlink(fname)
  tr <- rbind(c(3,1),c(2,1),c(4,3),c(4,2))
  convhulln(tr, paste0("Fx TO '", fname, "'"))
  expect_true(file.exists(fname))
  vert <- scan(fname, quiet=TRUE)
  expect_equal(vert, c(4, 2, 1, 0, 3))
})

test_that("Output of non-triangulated facets works", {
  X1 <- matrix(c( 1,  1,  1,
                  1,  1, -1,
                  1, -1,  1,
                  1, -1, -1,
                 -1,  1,  1,
                 -1,  1, -1,
                 -1, -1,  1,
                 -1, -1, -1,
                  3,  0,  0), ncol=3, byrow = TRUE)
  ts1 <- convhulln(X1, return.non.triangulated.facets = TRUE)
  tbl1 <- table(rowSums(!is.na(ts1)))
  expect_equal(names(tbl1), c("3", "4"))
  expect_equal(as.numeric(tbl1), c(4, 5))
})

context("inhulln")
test_that("inhulln gives the expected output", {
  ## Basic test
  x <- c(-1, -1, 1)
  y <- c(-1, 1, -1)
  p <- cbind(x, y)
  ch <- convhulln(p)
  ## Should be in hull
  pin <- inhulln(ch, cbind(-0.5, -0.5))
  expect_true(pin)
  ## Should be outside hull
  pout <- inhulln(ch, cbind(1, 1))
  expect_false(pout)

  ## Erroneous input is caught safely
  expect_error(inhulln(1, 2), "Convex hull has no convhulln attribute")
  expect_error(inhulln(ch, rbind(1, 1)), "Number of columns in test points p (1) not equal to dimension of hull (2).", fixed=TRUE)
  expect_error(inhulln(ch, cbind(1, 1, 1)), "Number of columns in test points p (3) not equal to dimension of hull (2).", fixed=TRUE)

  ## Test cube
  p <- rbox(n=0, D=3, C=1)
  ch <- convhulln(p)
  tp <-  cbind(seq(-1.9, 1.9, by=0.2), 0, 0)
  pin <- inhulln(ch, tp)
  ## Points on x-axis should be in box only between -1 and 1
  expect_equal(pin, tp[,1] < 1 & tp[,1] > -1)

  ## Test hypercube
  p <- rbox(n=0, D=4, C=1)
  ch <- convhulln(p)
  tp <-  cbind(seq(-1.9, 1.9, by=0.2), 0, 0, 0)
  pin <- inhulln(ch, tp)
  ## Points on x-axis should be in box only between -1 and 1
  expect_equal(pin, tp[,1] < 1 & tp[,1] > -1)

})

##' A simple mesh generator for non-convex regions
##'
##' An unstructured simplex requires a choice of mesh points (vertex nodes) and
##' a triangulation.  This is a simple and short algorithm that improves the
##' quality of a mesh by relocating the mesh points according to a relaxation
##' scheme of forces in a truss structure. The topology of the truss is reset
##' using Delaunay triangulation. A (sufficiently smooth) user supplied signed
##' distance function (\code{fd}) indicates if a given node is inside or
##' outside the region. Points outside the region are projected back to the
##' boundary.
##'
##' This is an implementation of original Matlab software of Per-Olof Persson.
##'
##' Excerpt (modified) from the reference below:
##'
##' \sQuote{The algorithm is based on a mechanical analogy between a triangular
##' mesh and a 2D truss structure. In the physical model, the edges of the
##' Delaunay triangles of a set of points correspond to bars of a truss. Each
##' bar has a force-displacement relationship \eqn{f(\ell, \ell_{0})}{F(L,L0)}
##' depending on its current length \eqn{\ell}{L} and its unextended length
##' \eqn{\ell_{0}}{L0}.}
##'
##' \sQuote{External forces on the structure come at the boundaries, on which
##' external forces have normal orientations. These external forces are just
##' large enough to prevent nodes from moving outside the boundary. The
##' position of the nodes are the unknowns, and are found by solving for a
##' static force equilibrium. The hope is that (when \code{fh = function(p)
##' return(rep(1,nrow(p)))}), the lengths of all the bars at equilibrium will
##' be nearly equal, giving a well-shaped triangular mesh.}
##'
##' See the references below for all details. Also, see the comments in the
##' source file.
##'
##' @param fd Vectorized signed distance function, for example
##'   \code{\link{mesh.dcircle}} or \code{\link{mesh.diff}}, accepting
##'   an \code{n}-by-\code{2} matrix, where \code{n} is arbitrary, as
##'   the first argument.
##' @param fh Vectorized function, for example
##'   \code{\link{mesh.hunif}}, that returns desired edge length as a
##'   function of position.  Accepts an \code{n}-by-\code{2} matrix,
##'   where \code{n} is arbitrary, as its first argument.
##' @param h0 Initial distance between mesh nodes. (Ignored of
##'   \code{p} is supplied)
##' @param bbox Bounding box \code{cbind(c(xmin,xmax), c(ymin,ymax))}
##' @param p An \code{n}-by-\code{2} matrix. The rows of \code{p}
##'   represent locations of starting mesh nodes.
##' @param pfix \code{nfix}-by-2 matrix with fixed node positions.
##' @param \dots parameters to be passed to \code{fd} and/or \code{fh}
##' @param dptol Algorithm stops when all node movements are smaller
##'   than \code{dptol}
##' @param ttol Controls how far the points can move (relatively)
##'   before a retriangulation with \code{\link{delaunayn}}.
##' @param Fscale \dQuote{Internal pressure} in the edges.
##' @param deltat Size of the time step in Euler's method.
##' @param geps Tolerance in the geometry evaluations.
##' @param deps Stepsize \eqn{\Delta x} in numerical derivative
##'   computation for distance function.
##' @param maxiter Maximum iterations.
##' @param plot logical.  If \code{TRUE} (default), the mesh is
##'   plotted as it is generated.
##' @return \code{n}-by-\code{2} matrix with node positions.
##' @section Wishlist : \itemize{ \item Implement in C/Fortran
##'   \item Implement an \code{n}D version as provided in the Matlab
##'   package \item Translate other functions of the Matlab package }
##' @author Raoul Grasman
##' @seealso \code{\link[interp]{tri.mesh}}, \code{\link{delaunayn}},
##'   \code{\link{mesh.dcircle}}, \code{\link{mesh.drectangle}},
##'   \code{\link{mesh.diff}}, \code{\link{mesh.union}},
##'   \code{\link{mesh.intersect}}
##' @references \url{http://persson.berkeley.edu/distmesh/}
##'
##' \cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
##' Review, Volume 46 (2), pp. 329-345, June 2004}
##' @keywords math optimize dplot graphs
##' @examples
##'
##' # examples distmesh2d
##' fd <- function(p, ...) sqrt((p^2)%*%c(1,1)) - 1
##'      # also predefined as `mesh.dcircle'
##' fh <- function(p,...)  rep(1,nrow(p))
##' bbox <- matrix(c(-1,1,-1,1),2,2)
##' p <- distmesh2d(fd,fh,0.2,bbox, maxiter=100)
##'     # this may take a while:
##'     # press Esc to get result of current iteration
##'
##' # example with non-convex region
##' fd <- function(p, ...) mesh.diff(p , mesh.drectangle, mesh.dcircle, radius=.3)
##'      # fd defines difference of square and circle
##'
##' p <- distmesh2d(fd,fh,0.05,bbox,radius=0.3,maxiter=4)
##' p <- distmesh2d(fd,fh,0.05,bbox,radius=0.3, maxiter=10)
##'      # continue on previous mesh
##' @export

distmesh2d <- function(fd, fh, h0, bbox, p=NULL, pfix=array(0,dim=c(0,2)), ...,
                       dptol=0.001, ttol=0.1, Fscale=1.2, deltat=0.2,
                       geps=0.001*h0, deps=sqrt(.Machine$double.eps)*h0,
                       maxiter=1000, plot=TRUE)
{
  rownorm2 = function(x) drop(sqrt((x^2)%*%c(1, 1)))

  if(any(apply(bbox, 2, diff)==0))
     stop("Supplied bounding box has zero area.")

  if(is.null(p)) {
    #%1 generate initial grid
    y = seq(bbox[1, 2],bbox[2, 2], by=h0*sqrt(3)/2)
    x = seq(bbox[1, 1], bbox[2, 1], by=h0)
    x = matrix(x,length(y),length(x),byrow=TRUE)
    x[seq(2,length(y),by=2),] = x[seq(2,length(y),by=2),] + h0/2
    p = cbind(c(x),y)

    #%2 remove nodes outside boundary specified by fd (points evaluated negative are
    # considered to lie inside the boundary)
    p = p[fd(p, ...) < geps,]
  }

  ## For density control
  nfix <- nrow(pfix)
  count <- 0
  densityctrlfreq <- 30
  count2 <- 0

  r0 = 1 / fh(p, ...)^2                           # acceptance probability
  p = rbind(pfix, p[stats::runif(nrow(p))<r0/max(r0),]) # rejection sampling
  N = nrow(p)
  if (N <= 3)
    stop("Not enough starting points inside boundary (is h0 too large?).")

  on.exit(return(invisible(p)));                 # in case we need to stop earlier
  message("Press esc if the mesh seems fine but the algorithm hasn't converged.")
  utils::flush.console()

  #%3 main loop: iterative improvement of points
  pold = 1.0/.Machine$double.eps;
  iter = 0
  while(TRUE) {
    count <- count+1
    if (max(rownorm2(p - pold)/h0 ) > ttol) {
        pold = p

        T = delaunayn(p)                           # generate a Delaunay triangulation

        pmid = (p[T[, 1],] + p[T[, 2],] + p[T[, 3],])/3          # calculate average of node locations as centers
        T  = T[fd(pmid, ...) < (-geps), 1:3]                  # remove triangles with center outside region

        #%4 describe edges by uniqe pairs of nodes
        #bars = unique(rbind(T[, -1],T[, -2],T[, -3]), MARGIN=1);
        #bars = bars[order(bars[, 1],bars[, 2]),];

        bars = rbind(T[, -3], T[, -2], T[, -1])                    # select unique edges
        #too slow: bars = unique(matsort(bars), MARGIN=1)
        bars = Unique(matsort(bars))                          # order the edges according to the node indices

        #%5 Graphical display
        if (plot)
          trimesh(T, p, asp=1)         # a la Matlab
    }

    #%6 compute force F on the basis of edge lenghts
    barvec = p[bars[, 1],] - p[bars[, 2],]                      # bar vectors
    L = rownorm2(barvec)                                      # their lengths

    # calculate desired lengths L0 by use of fh
    hbars = fh((p[bars[, 1],] + p[bars[, 2],])/2, ...)
    L0 = hbars * Fscale * sqrt(sum(L^2)/sum(hbars^2))
    ######################################################
    # Original Matlab code
    #     % Density control - remove points that are too close
    #     if mod(count,densityctrlfreq)==0 & any(L0>2*L)
    #       p(setdiff(reshape(bars(L0>2*L,:),[],1),1:nfix),:)=[];
    #       N=size(p,1); pold=inf;
    #       continue;
    #     end
    ######################################################
    # R translation
    # Density control - remove points that are too close
    tmp1 <- (count %% densityctrlfreq)
    tmp2 <- any(L0 > (2*L))
    if (tmp1 == 0 & tmp2) {
      count2 <- count2 + 1
      # reshape(bars(L0>2*L,:),[],1)
      tmp3 <- c(bars[L0 > (2*L),])
      up <- which(duplicated(tmp3) == TRUE, arr.ind=TRUE)
      p <- p[-up,]
      N <- length(p[,1])
      pold <- 1/.Machine$double.eps
    }
    ######################################################
    F = drop(L0 - L)
    F[F < 0] = 0
    Fvec = barvec * (F/L)
    Ftot = matrix(0, N, 2)
    ii = bars[, c(1, 1, 2, 2)]
    jj = rep(1, length(F)) %o% c(1, 2, 1, 2)
    s = c(cbind(Fvec, -Fvec))
    ns = length(s)
    Ftot[1:(2*N)] = rowsum(s, ii[1:ns] + ns*(jj[1:ns] - 1))  # sum all forces on each node
    if (nrow(pfix) > 0) Ftot[1:nrow(pfix),] = 0                 # Force = 0 at fixed points
    p = p + deltat*Ftot;

    #%7 excercise normal force at boundary: move overshoot points to nearest boundary point
    d = fd(p);
    ix= d > 0;                                                  # find points outside
    dgradx= (fd(cbind(p[ix, 1]+deps, p[ix, 2]),...) - d[ix])/deps;  # Numerical
    dgrady= (fd(cbind(p[ix, 1], p[ix, 2]+deps),...) - d[ix])/deps;  #    gradient
    p[ix,] = p[ix,] - cbind(d[ix]*dgradx, d[ix]*dgrady);      # Project back to boundary

    #%8 test for convergence
    if(max(rownorm2(deltat*Ftot[d < (-geps),])/h0) < dptol | iter>=maxiter) break;
    iter = iter + 1
  }
  message(sprintf("Number of density control ops = %d",count2))
  message(sprintf("Number of iterations = %d",iter))
  if (iter >= maxiter)
    warning(" Maximum iterations reached. Relaxation process not \n completed")
  return(p)
}

context("distmesh2d")
test_that("distmesh2d can create a mesh on an ellipse", {
  bbox <- 2*matrix(c(-1,1,-1/2,1/2),2,2)
  ## Ellipse
  fd1 <- function(p,ra2=1/1.,rb2=1/2,xc2=0,yc2=0, ...){
    if (!is.matrix(p)) 
      p <- t(as.matrix(p))
    return(sqrt(((p[,1]-xc2)/ra2)^2+((p[,2]-yc2)/rb2)^2)-1)
  }
  ## Solve using distmesh2d()
  fh <- function(p,...)  rep(1,nrow(p))
  ## This is the original line, which throws a warning. New one does
  ## too, but runs faster p <-
  ## distmesh2d(fd=fd1,fh=fh,p=NULL,h0=0.05,bbox=bbox,maxiter=1000,
  ## plot=FALSE)
  expect_warning(p <- distmesh2d(fd=fd1,fh=fh,p=NULL,h0=0.05,bbox=bbox,maxiter=10, plot=FALSE))
})

##' Search for the enclosing Delaunay convex hull
##' 
##' For \code{t <- delaunay(cbind(x, y))}, where \code{(x, y)} is a 2D set of
##' points, \code{tsearch(x, y, t, xi, yi)} finds the index in \code{t}
##' containing the points \code{(xi, yi)}.  For points outside the convex hull
##' the index is \code{NA}.
##' 
##' @param x X-coordinates of triangulation points
##' @param y Y-coordinates of triangulation points
##' @param t Triangulation, e.g. produced by \code{t <-
##'   delaunayn(cbind(x, y))}
##' @param xi X-coordinates of points to test
##' @param yi Y-coordinates of points to test
##' @param bary If \code{TRUE} return barycentric coordinates as well
##'   as index of triangle.
##' @param method One of \code{"quadtree"} or \code{"orig"}. The
##'   Quadtree algorithm is much faster and new from version
##'   0.4.0. The \code{orig} option uses the tsearch algorithm adapted
##'   from Octave code. Its use is deprecated and it may be removed
##'   from a future version of the package.
##' @return If \code{bary} is \code{FALSE}, the index in \code{t} containing the points 
##' \code{(xi, yi)}.  For points outside the convex hull the index is \code{NA}. 
##' If \code{bary} is \code{TRUE}, a list containing: 
##'   \describe{
##'    \item{list("idx")}{the index in \code{t} containing the points \code{(xi, yi)}}
##'    \item{list("p")}{a 3-column matrix containing the barycentric coordinates with 
##'    respect to the enclosing triangle of each point \code{(xi, yi)}.}
##'   }
##' @author Jean-Romain Roussel (Quadtree algorithm), David Sterratt (Octave-based implementation)
##' @note The original Octave function is Copyright (C) 2007-2012
##'   David Bateman
##' @seealso \code{\link{tsearchn}}, \code{\link{delaunayn}}
##' @export
tsearch <- function(x, y, t, xi, yi, bary=FALSE, method="quadtree") {
  xtxt  = deparse(substitute(x))
  ytxt  = deparse(substitute(y))
  xitxt = deparse(substitute(xi))
  yitxt = deparse(substitute(yi))
  ttxt  = deparse(substitute(t))
  
  if (!is.vector(x))  {stop(paste(xtxt, "is not a vector"))}
  if (!is.vector(y))  {stop(paste(ytxt, "is not a vector"))}
  if (!is.matrix(t))  {stop(paste(ttxt, "is not a matrix"))}
  if (!is.vector(xi)) {stop(paste(xitxt, "is not a vector"))}
  if (!is.vector(yi)) {stop(paste(yitxt, "is not a vector"))}
  
  if (length(x) != length(y)) {
    stop(paste(xtxt, "is not same length as", ytxt))
  }
  if (length(xi) != length(yi)) {
    stop(paste(xitxt, "is not same length as", yitxt))
  }
  if (ncol(t) != 3) {
    stop(paste(ttxt, "does not have three columns"))
  }
  if (any(as.integer(t) != t)) {
    stop(paste(ttxt, "does not have integer elements"))
  }
  
  if (length(x) == 0) {stop(paste(xtxt, "is empty"))}
  if (length(y) == 0) {stop(paste(ytxt, "is empty"))}
  
  if (any(is.na(x)))   {stop(paste(xtxt, "contains NAs"))}
  if (any(is.na(y)))   {stop(paste(ytxt, "contains NAs"))}
  
  if (length(x) < 3 | length(y) < 3) {
    stop("A triangulation should have at least 3 points")
  }
  
  storage.mode(t) <- "integer"
  
  if (max(t) > length(x)) {
    stop(paste(ttxt, "has indexes greater than the number of points"))
  }
  
  if (min(t) <= 0) {
    stop(paste(ttxt, "has indexes which refer to non-existing points"))
  }

  if (length(xi) == 0 | length(yi) == 0) {
    if (!bary)
      return (integer(0))
    else
      return (list(idx = integer(0), p = matrix(0,0,3)))
  }
  
  if (method == "quadtree") {
    out <- C_tsearch(x, y, t, xi, yi, bary)
  } else {
    out <- .Call("C_tsearch_orig", x, y, t, xi, yi, bary, PACKAGE="geometry")
  }

  if (bary) {
    names(out) <- c("idx", "p")
  }
  return(out)
}

##' Search for the enclosing Delaunay convex hull
##' 
##' For \code{t = delaunayn(x)}, where \code{x} is a set of points in \eqn{N}
##' dimensions, \code{tsearchn(x, t, xi)} finds the index in \code{t}
##' containing the points \code{xi}. For points outside the convex hull,
##' \code{idx} is \code{NA}. \code{tsearchn} also returns the barycentric
##' coordinates \code{p} of the enclosing triangles.
##'
##' If \code{x} is \code{NA} and the \code{t} is a
##' \code{delaunayn} object produced by
##' \code{\link{delaunayn}} with the \code{full} option, then use the
##' Qhull library to perform the search. Please note that this is
##' experimental in geometry version 0.4.0 and is only partly tested
##' for 3D hulls, and does not yet work for hulls of 4 dimensions and
##' above.
##' 
##' @param x An \eqn{N}-column matrix, in which each row represents a
##'   point in \eqn{N}-dimensional space.
##' @param t A matrix with \eqn{N+1} columns. A row of \code{t}
##'   contains indices into \code{x} of the vertices of an
##'   \eqn{N}-dimensional simplex. \code{t} is usually the output of
##'   delaunayn.
##' @param xi An \eqn{M}-by-\eqn{N} matrix. The rows of \code{xi}
##'   represent \eqn{M} points in \eqn{N}-dimensional space whose
##'   positions in the mesh are being sought.
##' @param ... Additional arguments
##' @return A list containing:
##'   \describe{
##'     \item{\code{idx}}{An \eqn{M}-long vector containing the indices
##'       of the row of \code{t} in which each point in \code{xi} is found.}
##'    \item{\code{p}}{An \eqn{M}-by-\eqn{N+1} matrix containing the
##'     barycentric coordinates with respect to the enclosing simplex
##'     of each point in \code{xi}.}}
##' @author David Sterratt
##' @note Based on the Octave function Copyright (C) 2007-2012 David
##'   Bateman.
##' @seealso \code{\link{tsearch}}, \code{\link{delaunayn}}
##' @export
tsearchn <- function(x, t, xi, ...) {
  if (any(is.na(x)) && inherits(t, "delaunayn")) {
    return(tsearchn_delaunayn(t, xi))
  }
  fast <- TRUE
  if (!is.null(list(...)$fast) & is.logical(list(...)$fast))
    fast <- list(...)$fast

  ## Check input
  if (!is.matrix(x))  {stop(paste(deparse(substitute(x)), "is not a matrix"))}
  if (!is.matrix(t))  {stop(paste(deparse(substitute(t)), "is not a matrix"))}
  if (!is.matrix(xi)) {stop(paste(deparse(substitute(xi)), "is not a matrix"))}

  n <- dim(x)[2]                        # Number of dimensions
  if (n==2 && fast) {
    return(tsearch(x[,1], x[,2], t, xi[,1], xi[,2], bary=TRUE))
  }
  nt <- dim(t)[1]                       # Number of simplexes
  m <- dim(x)[1]                        # Number of points in simplex grid
  mi <- dim(xi)[1]                      # Number of points to search for
  ## If there are no points to search for, return an empty index
  ## vector and an empty coordinate matrix
  if (mi==0) {
    return(list(idx=c(), p=matrix(0, 0, n + 1)))
  }
  idx <- rep(NA, mi)
  p <- matrix(NA, mi, n + 1)

  ## Indicies of points that still need to be searched for
  ni <- 1:mi

  degenerate.simplices <- c()
  ## Go through each simplex in turn
  for (i in 1:nt) { 
    ## Only calculate the Barycentric coordinates for points that have not
    ## already been found in a simplex.
    b <- suppressWarnings(cart2bary(x[t[i,],], xi[ni,,drop=FALSE]))
    if (is.null(b)) {
      degenerate.simplices <- c(degenerate.simplices, i)
    } else {

      ## Our points xi are in the current triangle if (all(b >= 0) &&
      ## all (b <= 1)). However as we impose that sum(b,2) == 1 we only
      ## need to test all(b>=0). Note that we need to add a small margin
      ## for rounding errors
      intri <- apply(b >= -1e-12, 1, all)

      ## Set the simplex indicies  of the points that have been found to
      ## this simplex
      idx[ni[intri]] <- i

      ## Set the baryocentric coordinates of the points that have been found
      p[ni[intri],] <- b[intri,]

      ## Remove these points from the search list
      ni <- ni[!intri]

      ## If there are no more points to search for, give up
    if (length(ni) == 0) { break }
    }
  }
  if (length(degenerate.simplices) > 0) {
    warning(paste("Degenerate simplices:", toString(degenerate.simplices)))
  }
  return(list(idx=idx, p=p))
}

##' Conversion of Cartesian to Barycentric coordinates.
##' 
##' Given the Cartesian coordinates of one or more points, compute
##' the barycentric coordinates of these points with respect to a
##' simplex.
##' 
##' Given a reference simplex in \eqn{N} dimensions represented by a
##' \eqn{N+1}-by-\eqn{N} matrix an arbitrary point \eqn{P} in
##' Cartesian coordinates, represented by a 1-by-\eqn{N} row vector, can be
##' written as
##' \deqn{P = \beta X}
##' where \eqn{\beta} is an \eqn{N+1} vector of the barycentric coordinates.
##' A criterion on \eqn{\beta} is that
##' \deqn{\sum_i\beta_i = 1}
##' Now partition the simplex into its first \eqn{N} rows \eqn{X_N} and
##' its \eqn{N+1}th row \eqn{X_{N+1}}. Partition the barycentric
##' coordinates into the first \eqn{N} columns \eqn{\beta_N} and the
##' \eqn{N+1}th column \eqn{\beta_{N+1}}. This allows us to write
##' \deqn{P_{N+1} - X_{N+1} = \beta_N X_N + \beta_{N+1} X_{N+1} - X_{N+1}}
##' which can be written
##' \deqn{P_{N+1} - X_{N+1} = \beta_N(X_N - 1_N X_{N+1})}
##' where \eqn{1_N} is an \eqn{N}-by-1 matrix of ones.  We can then solve
##' for \eqn{\beta_N}:
##' \deqn{\beta_N = (P_{N+1} - X_{N+1})(X_N - 1_N X_{N+1})^{-1}}
##' and compute
##' \deqn{\beta_{N+1} = 1 - \sum_{i=1}^N\beta_i}
##' This can be generalised for multiple values of
##' \eqn{P}, one per row.
##' 
##' @param X Reference simplex in \eqn{N} dimensions represented by a
##' \eqn{N+1}-by-\eqn{N} matrix
##' @param P \eqn{M}-by-\eqn{N} matrix in which each row is the Cartesian
##' coordinates of a point.
##' @return \eqn{M}-by-\eqn{N+1} matrix in which each row is the
##' barycentric coordinates of corresponding row of \code{P}. If the
##' simplex is degenerate a warning is issued and the function returns
##' \code{NULL}.
##' @author David Sterratt
##' @note Based on the Octave function by David Bateman.
##' @examples
##' ## Define simplex in 2D (i.e. a triangle)
##' X <- rbind(c(0, 0),
##'            c(0, 1),
##'            c(1, 0))
##' ## Cartesian coordinates of points
##' P <- rbind(c(0.5, 0.5),
##'            c(0.1, 0.8))
##' ## Plot triangle and points
##' trimesh(rbind(1:3), X)
##' text(X[,1], X[,2], 1:3) # Label vertices
##' points(P)
##' cart2bary(X, P)
##' @seealso \code{\link{bary2cart}}
##' @export
cart2bary <- function(X, P) {
  M <- nrow(P)
  N <- ncol(P)
  if (ncol(X) != N) {
    stop("Simplex X must have same number of columns as point matrix P")
  }
  if (nrow(X) != (N+1)) {
    stop("Simplex X must have N columns and N+1 rows")
  }
  X1 <- X[1:N,] - (matrix(1,N,1) %*% X[N+1,,drop=FALSE])
  if (rcond(X1) < .Machine$double.eps) {
    warning("Degenerate simplex")
    return(NULL)
  }
  Beta <- (P - matrix(X[N+1,], M, N, byrow=TRUE)) %*% solve(X1)
  Beta <- cbind(Beta, 1 - apply(Beta, 1, sum))
  return(Beta)
}

##' Conversion of Barycentric to Cartesian coordinates
##' 
##' Given the barycentric coordinates of one or more points with
##' respect to a simplex, compute the Cartesian coordinates of these
##' points.
##' 
##' @param X Reference simplex in \eqn{N} dimensions represented by a
##' \eqn{N+1}-by-\eqn{N} matrix
##' @param Beta \eqn{M} points in barycentric coordinates with
##' respect to the simplex \code{X} represented by a
##' \eqn{M}-by-\eqn{N+1} matrix
##' @return \eqn{M}-by-\eqn{N} matrix in which each row is the
##' Cartesian coordinates of corresponding row of \code{Beta}
##' @examples
##' ## Define simplex in 2D (i.e. a triangle)
##' X <- rbind(c(0, 0),
##'            c(0, 1),
##'            c(1, 0))
##' ## Cartesian cooridinates of points
##' beta <- rbind(c(0, 0.5, 0.5),
##'               c(0.1, 0.8, 0.1))
##' ## Plot triangle and points
##' trimesh(rbind(1:3), X)
##' text(X[,1], X[,2], 1:3) # Label vertices
##' P <- bary2cart(X, beta)
##' points(P)
##' @seealso \code{\link{cart2bary}}
##' @author David Sterratt
##' @export
bary2cart <- function(X, Beta) {
  return(Beta %*% X)
}

tsearchn_delaunayn <- function(t, xi) {
  warning("tsearchn using the Qhull library is currently an experimental feature. It has been tested somewhat for 3D triangulations, but it does not work reliably for 4D triangulations. See https://github.com/davidcsterratt/geometry/issues/6")
  ts <- .Call("C_tsearchn", t, xi)
  p <- do.call(rbind,
               lapply(1:nrow(xi), function(i) {
                 cart2bary(ts$P[t$tri[ts$idx[i],],], xi[i,,drop=FALSE])
               }))
  ## C_tsearchn will return the *best* facet. Need to check it is
  ## actually in the triangulation
  outwith_facet_inds <- which(apply(p < 0, 1, any))
  idx <- ts$idx
  idx[outwith_facet_inds] <- NA
  p[outwith_facet_inds,] <- NA
  return(list(idx=idx, p=p, P=ts$P))
}

context("tsearch")
test_that("tsearch gives the expected output", {
  x <- c(-1, -1, 1)
  y <- c(-1, 1, -1)
  p <- cbind(x, y)
  tri <- matrix(c(1, 2, 3), 1, 3)
  ## Should be in triangle #1
  ts <- tsearch(x, y, tri, -1, -1)
  expect_equal(ts, 1)
  ## Should be in triangle #1
  ts <- tsearch(x, y, tri, 1, -1)
  expect_equal(ts, 1)
  ## Should be in triangle #1
  ts <- tsearch(x, y, tri, -1, 1)
  expect_equal(ts, 1)
  ## Centroid
  ts <- tsearch(x, y, tri, -1/3, -1/3)
  expect_equal(ts, 1)
  ## Should be outside triangle #1, so should return NA
  ts <- tsearch(x, y, tri, 1, 1)
  expect_true(is.na(ts))
})

test_that("tsearch can deal with faulty input", {
  x <- c(-1, -1, 1)
  y <- c(-1, 1, -1)
  p <- cbind(x, y)
  tri <- matrix(c(1, 2, 3), 1, 3)

  ## NULLs and NAs
  ## expect_error(tsearch(x, y, tri, NA, NA))
  expect_error(tsearch(x, y, NA, -1, 1))
  expect_error(tsearch(NA, NA, tri, -1, 1))
  expect_error(tsearch(x, y, tri, NULL, NULL))
  expect_error(tsearch(x, y, NULL, -1, 1))
  expect_error(tsearch(NULL, NULL, tri, -1, 1))

  ## Wrong number of columns
  expect_error(tsearch(p, 0, tri, -1, 1))

  ## Non-integer triangulation
  expect_error(tsearch(x, y, matrix(runif(15), 5, 3), -1, 1), regexp="does not have integer elements")

  ## Wrong number of columns in triangulation
  expect_error(tsearch(x, y, matrix(1:4, 4, 2), -1, 1))

  ## Mismatch in x and y lengths
  expect_error(tsearch(x, y[-1], tri, -1, 1))

  ## Mismatch in xi and yi lengths
  expect_error(tsearch(x, y, tri, c(-1, 1), 1))

  ## A subtle one! This gives numeric(0) as the final arguments and
  ## should give idx with no elements and a 0x3 matrix for p
  ps <- matrix(0, 0, 2)
  expect_equal(tsearch(x, y, tri, ps[,1], ps[,2], bary=TRUE),
               list(idx=integer(0), p=matrix(0, 0, 3)))

})

## See
## http://totologic.blogspot.co.uk/2014/01/accurate-point-in-triangle-test.html
## for inspiration for the test below

test_that("tsearch gives the expected output when computer precision problem arise", {

  # ==== Hand made test ====

  x1 <- 1/10
  y1 <- 1/9
  x2 <- 100/8
  y2 <- 100/3
  P <- rbind(c(x1, y1),  c(x2, y2),  c(100/4, 100/9), c(-100/8, 100/6))

  # And a single point p(x, y) lying exactly on the segment [p1, p2] :
  xi <- x1 + (3/7)*(x2 - x1)
  yi <- y1 + (3/7)*(y2 - y1)

  # Should always give triangle 2 since this is the lastest tested

  tri1 <- rbind(1:3, c(1, 2, 4))
  ts <- tsearch(P[,1], P[,2], tri1, xi, yi)
  expect_equal(ts, 2)

  tri2 <- rbind(c(1, 2, 4), 1:3)
  ts <- tsearch(P[,1], P[,2], tri2, xi, yi)
  expect_equal(ts, 2)

  # The same but with only one triangle
  P <- rbind(c(x1, y1),  c(x2, y2),  c(100/4, 100/9))
  tri <- matrix(c(1, 2, 3), 1, 3)
  ts <- tsearch(P[,1], P[,2], tri, xi, yi)
  expect_equal(ts, 1)

  tri <- matrix(c(3, 2, 1), 1, 3)
  ts <- tsearch(P[,1], P[,2], tri, xi, yi)
  expect_equal(ts, 1)

  # The same but with the other triangle
  P <- rbind(c(x2, y2),  c(100/4, 100/9), c(-100/8, 100/6))
  tri <- matrix(c(1, 2, 3), 1, 3)
  ts <- tsearch(P[,1], P[,2], tri, xi, yi)
  expect_equal(ts, 1)

  tri <- matrix(c(3, 2, 1), 1, 3)
  ts <- tsearch(P[,1], P[,2], tri, xi, yi)
  expect_equal(ts, 1)

  # Another test
  x <- c(6.89, 7.15, 7.03)
  y <- c(7.76, 7.75, 8.35)
  tri <- matrix(c(1, 2, 3), 1, 3)
  ts <- tsearch(x, y, tri, 7.125, 7.875)
  expect_equal(ts, 1)

  # ==== Test known to bug in former code ====

  x <- c(278287.03, 278286.89, 278287.15, 278287.3)
  y <- c(602248.35, 602247.76, 602247.75, 602248.35)

  xi = 278287.125
  yi = 602247.875

  # Should always give triangle 2 but here it does not work

  tri = rbind(c(3,1,4), c(3,1,2))
  ts <- tsearch(x, y, tri, xi, yi)
  expect_equal(ts, 2)

  tri = rbind(c(1,2,3), c(1,3,4))
  ts <- tsearch(x, y, tri, xi, yi)
  expect_equal(ts, 1)

  # This is because the buffer epsilon is 1.0e-12.

  x <- c(278287.03, 278287.15, 278287.3)
  y <- c(602248.35, 602247.75, 602248.35)

  tri <- matrix(c(1, 2, 3), 1, 3)
  ts <- tsearch(x, y, tri, xi, yi)
  expect_true(is.na(ts))

  #expect_equal(ts, 1)  #With epsilon = 1.0e-10 it works.
})

test_that("no regression on Issue #39", {
  ## See https://github.com/davidcsterratt/geometry/issues/39
  ## vertices
  P <- rbind(
    c(373.8112, 4673.726), #31
    c(222.9705, 4280.085), #32
    c(291.0508, 4476.996), #42
    c(553.4783, 4523.605), #43
    c(222.6388, 4023.920), #44
    c(445.2401, 4370.940), #48
    c(81.54986, 4125.393)) #61
    ## I found this error on my system with Ubuntu 16.04, R 3.4.1 and
    ## `geometry_0.4.2`. `geometry_0.4.0` gave the same error). The error
    ## persists on Windows 10 with R 3.6.1.

    ## triangulation
    T <- rbind(
      c(3,   2,   6),
      c(5,   2,   7),
      c(4,   1,   3))

    ## data
    data <- rbind(
      c(221.6, 4171.8),
      c(250.4, 4311.8),
      c(496.6, 4516.2),
      c(254.0, 4294.8),
      c(199.4, 4072.6))

    ## With bug returns NA for datapoints 3 and 5
    expect_equal(tsearch(P[,1], P[,2],T, data[,1], data[,2]), c(2, 1, 3, 1, 2))

    data2 <- rbind(
      c(221.6, 4171.8),
      c(250.4, 4311.8),
      c(496.6, 4516.2),
      c(254.0, 4294.8),
      c(199.4, 4072.0)) #note that only the Y coordinate of datapoint 5 was changed
      expect_equal(tsearch(P[,1], P[,2],T, data2[,1], data2[,2]), c(2, 1, 3, 1, 2))
})

context("Comparison of tsearch and tsearchn")
test_that("tsearch and tsearchn give the same results", {
  set.seed(1)
  X <- runif(50)
  Y <- runif(50)
  T <- delaunayn(cbind(X, Y))
  XI <- runif(1000)
  YI <- runif(1000)

  out <- tsearch(X, Y, T, XI, YI)
  outn <- tsearchn(cbind(X, Y), T, cbind(XI, YI), fast=FALSE)

  expect_equal(na.omit(out), na.omit(outn$idx))

  out <- tsearch(X, Y, T, XI, YI, TRUE)
  expect_equal(na.omit(outn$p), na.omit(out$p), tolerance=1e-12)
})

context("tsearchn")
test_that("tsearchn gives the expected output", {
  ## Simple example
  x <- c(-1, -1, 1)
  y <- c(-1, 1, -1)
  p <- cbind(x, y)
  tri <- matrix(c(1, 2, 3), 1, 3)
  ## Should be in triangle #1
  ts <- tsearchn(p, tri, cbind(-1, -1),fast=FALSE)
  expect_equal(ts$idx, 1)
  expect_equal(ts$p, cbind(1, 0, 0))
  ## Should be in triangle #1
  ts <- tsearchn(p, tri, cbind(1, -1), fast=FALSE)
  expect_equal(ts$idx, 1)
  expect_equal(ts$p, cbind(0, 0, 1))
  ## Should be in triangle #1
  ts <- tsearchn(p, tri, cbind(-1, 1), fast=FALSE)
  expect_equal(ts$idx, 1)
  expect_equal(ts$p, cbind(0, 1, 0))
  ## Centroid
  ts <- tsearchn(p, tri, cbind(-1/3, -1/3), fast=FALSE)
  expect_equal(ts$idx, 1)
  expect_equal(ts$p, cbind(1/3, 1/3, 1/3))
  ## Should be outside triangle #1, so should return NA
  ts <- tsearchn(p, tri, cbind(1, 1), fast=FALSE)
  expect_true(is.na(ts$idx))
  expect_true(all(is.na(ts$p)))

  ## Create a mesh with a zero-area element (degenerate simplex)
  p <- cbind(c(-1, -1, 0, 1, 2),
             c(-1,  1, 0, 0, 0))
  tri <- rbind(c(1, 2, 3),
               c(3, 4, 5))
  ## Look for one point in one of the simplices and a point outwith the
  ## simplices. This forces tsearchn to look in all simplices. It
  ## shouldn't fail on the degenerate simplex.
  expect_warning(ts <- tsearchn(p, tri, rbind(c(-0.5, 0), c(3, 1)), fast=FALSE))
  expect_equal(ts$idx, c(1, NA))
  ts <- tsearchn(p, tri, rbind(c(-0.5, 0), c(3, 1)), fast=TRUE)
  expect_equal(ts$idx, c(1, NA))
})



context("tsearchn_delaunayn")
test_that("tsearchn gives the expected output", {
  ## Erroneous input is caught safely. Force
  ## tsearchn_delaunayn to be called
  tfake <- matrix(1:3, 1, 3)
  class(tfake) <- "delaunayn"
  expect_error(suppressWarnings(tsearchn(NA, tfake, matrix(1:2, 1, 2))), "Delaunay triangulation has no delaunayn attribute")

  x <- cbind(c(-1, -1, 1),
             c(-1, 1, -1))

  dt <- delaunayn(x, output.options=TRUE)

  ## Should be in triangle #1
  xi <- cbind(-1, 1)
  expect_warning(ts <- tsearchn(NA, dt, xi))
  expect_equal(ts$idx, 1)
  expect_equal(bary2cart(x[dt$tri[ts$idx,],], ts$p), xi)

  ## Centroid
  xi <- cbind(-1/3, -1/3)
  expect_warning(ts <- tsearchn(NA, dt, xi))
  expect_equal(ts$idx, 1)
  expect_equal(ts$p, cbind(1/3, 1/3, 1/3))

  ## Should be outside triangle #1, so should return NA
  xi <- cbind(1, 1)
  expect_warning(ts <- tsearchn(NA, dt, xi))
  expect_true(is.na(ts$idx))
  expect_true(all(is.na(ts$p)))

  ## Check mutliple points work
  xi <- rbind(c(-1, 1),
              c(-1/3, -1/3))
  expect_warning(ts <- tsearchn(NA, dt, xi))
  expect_equal(ts$idx, c(1, 1))
  expect_equal(do.call(rbind, lapply(1:2, function(i) {
    bary2cart(x[dt$tri[ts$idx[i],],], ts$p[i,])
  })), xi)


  ## Test against original version
  p <- cbind(c(0, 0, 1, 1, 0.5),
             c(0, 1, 1, 0, 0.5))
  dt <- delaunayn(p, "FA") ## Interesting error, as default options are 'nixed
  dt <- delaunayn(p, output.options=TRUE)
  xi <- c(0.1, 0.5, 0.9, 0.5)
  yi <- c(0.5, 0.9, 0.5, 0.1)
  expect_warning(ts <- tsearchn(NA, dt, cbind(xi, yi)))
  expect_equal(ts$idx,
               tsearch(p[,1], p[,2], dt$tri,  xi, yi, method="orig"))

  ## 3D test
  x <- rbox(D=3, B=1)
  dt <- delaunayn(x, output.options=TRUE)

  xi <- rbind(c(0.5, 0.5, 0.5),
              c(-0.5, -0.5, -0.5),
              c(0.9, 0, 0))
  expect_warning(ts <- tsearchn(NA, dt, xi))
  expect_equal(do.call(rbind, lapply(1:3, function(i) {
    bary2cart(x[dt$tri[ts$idx[i],],], ts$p[i,])
  })), xi)

  ## 4D test
  ##
  ## This does not work yet. The "best" facet is not always the correct facet.
  ## x <- rbox(D=4, B=1)
  ## dt <- delaunayn(x, output.options=TRUE)

  ## xi <- rbind(c(0.5, 0.5, 0.5, 0.5),
  ##             c(-0.49, -0.49, -0.49, -0.49),
  ##             c(0.9, 0, 0, 0))
  ## ts <- tsearchn(dt, NA, xi)
  ## expect_equal(do.call(rbind, lapply(1:3, function(i) {
  ##   bary2cart(x[dt$tri[ts$idx[i],],], ts$p[i,])
  ## })), xi)
  
  ## We don't need to test when creating a mesh with a zero-area
  ## element (degenerate simplex), as these shouldn't be produced by
  ## qhull.

})

"trisplinter" <-
function(T,p,threshold=sqrt(.Machine$double.eps)){
  rownorm2 = function(x) drop(sqrt((x^2)%*%c(1,1)))
  d1 = p[T[,1],] - p[T[,2],]
  d2 = p[T[,2],] - p[T[,3],]
  d1 = d1 / rownorm2(d1)
  d2 = d2 / rownorm2(d2)
  ar = d1[,1]*d2[,2] - d1[,2]*d2[,1]
  return(abs(ar) < threshold)
}

##' Test if points lie in convex hull
##'
##' Tests if a set of points lies within a convex hull, returning a
##' boolean vector in which each element is \code{TRUE} if the
##' corresponding point lies within the hull and \code{FALSE} if it
##' lies outwith the hull or on one of its facets.
##' 
##' @param ch Convex hull produced using \code{\link{convhulln}}
##' @param p An \eqn{M}-by-\eqn{N} matrix of points to test. The rows
##'   of \code{p} represent \eqn{M} points in \eqn{N}-dimensional
##'   space.
##' @return A boolean vector with \eqn{M} elements
##' @author David Sterratt
##' @note \code{inhulln} was introduced in geometry 0.4.0, and is
##'   still under development. It is worth checking results for
##'   unexpected behaviour.
##' @seealso \code{\link{convhulln}}, \code{point.in.polygon} in \pkg{sp}
##' @export
##' @examples
##' p <- cbind(c(-1, -1, 1), c(-1, 1, -1))
##' ch <- convhulln(p)
##' ## First point should be in the hull; last two outside
##' inhulln(ch, rbind(c(-0.5, -0.5),
##'                   c( 1  ,  1),
##'                   c(10  ,  0)))
##'
##' ## Test hypercube
##' p <- rbox(D=4, B=1)
##' ch <- convhulln(p)
##' tp <-  cbind(seq(-1.9, 1.9, by=0.2), 0, 0, 0)
##' pin <- inhulln(ch, tp)
##' ## Points on x-axis should be in box only betw,een -1 and 1
##' pin == (tp[,1] < 1 & tp[,1] > -1)
inhulln <- function(ch, p) {
  return(.Call("C_inhulln", ch, p, PACKAGE="geometry"))
}
 

## Copyright (C) 1999, 2006, 2007, 2009 David M. Doolin
## bugs and limitations:  
##        Probably ought to be an optional check to make sure that
##        traversing the vertices doesn't make any sides cross 
##        (Is simple closed curve the technical definition of this?). 

##' Determines area of a polygon by triangle method.  The variables
##' \code{x} and \code{y} define the vertex pairs, and must therefore
##' have the same shape.  They can be either vectors or arrays.  If
##' they are arrays then the columns of \code{x} and \code{y} are
##' treated separately and an area returned for each.
##'
##' If the optional \code{dim} argument is given, then \code{polyarea}
##' works along this dimension of the arrays \code{x} and \code{y}.
##'
##' @title Determines area of a polygon by triangle method. 
##' @param x X coordinates of vertices.
##' @param y Y coordinates of vertices.
##' @param d Dimension of array to work along.
##' @return Area(s) of polygon(s).
##' @author David Sterratt based on the octave sources by David M. Doolin
##' @export
##' @importFrom magic shift ashift
##' @examples
##' x <- c(1, 1, 3, 3, 1)
##' y <- c(1, 3, 3, 1, 1)
##' polyarea(x, y)
##' polyarea(cbind(x, x), cbind(y, y)) ##  c(4, 4)
##' polyarea(cbind(x, x), cbind(y, y), 1) ##  c(4, 4)
##' polyarea(rbind(x, x), rbind(y, y), 2) ##  c(4, 4)
polyarea <- function(x, y, d=1) {
  if (is.vector(x) & is.vector(y)) {
    if (length(x) == length(y)) {
      a <- abs(sum(x*(shift(y, -1) - shift(y, 1))))/2
    } else {
      stop("x and y must have the same shape")
    }
  } else {
    if (is.array(x) & is.array(y)) {
      if (length(dim(x)) != 2) {
        stop("Arrays must have two dimensions.")
      }
      if (all(dim(x) == dim(y))) {
        v <- c(0, 0)
        v[d] <- 1
        a <- abs(apply(x*(ashift(y, -v) - ashift(y, v)), 3 - d, sum))/2
      } else {
        stop("x and y must have the same shape")
      }
    } else {
      stop("x and y must be of same type")
    }
  }
  names(a) <- NULL
  return(a)
}

context("polyarea")
test_that("ployarea computes the area of two identical squares", {
  x <- c(1, 1, 3, 3, 1)
  y <- c(1, 3, 3, 1, 1)

  expect_equal(polyarea(cbind(x, x), cbind(y, y)), c(4, 4))
  expect_equal(polyarea(cbind(x, x), cbind(y, y), 1), c(4, 4))
  expect_equal(polyarea(rbind(x, x), rbind(y, y), 2), c(4, 4))
})

##' Extract Unique Rows
##' 
##' \sQuote{Unique} returns a vector, data frame or array like 'x' but with
##' duplicate elements removed.
##' 
##' 
##' @param X Numerical matrix.
##' @param rows.are.sets If \sQuote{\code{TRUE}}, rows are treated as sets -
##' i.e., to define uniqueness, the order of the rows does not matter.
##' @return Matrix of the same number of columns as \code{x}, with the unique
##' rows in \code{x} sorted according to the columns of \code{x}. If
##' \code{rows.are.sets = TRUE} the rows are also sorted.
##' @note \sQuote{\code{Unique}} is (under circumstances) much quicker than the
##' more generic base function \sQuote{\code{unique}}.
##' @author Raoul Grasman
##' @keywords arith math array
##' @examples
##' 
##' # `Unique' is faster than `unique'
##' x = matrix(sample(1:(4*8),4*8),ncol=4)
##' y = x[sample(1:nrow(x),3000,TRUE), ]
##' gc(); system.time(unique(y))
##' gc(); system.time(Unique(y))
##' 
##' #
##' z = Unique(y)
##' x[matorder(x),]
##' z[matorder(z),]
##' 
##' @export
"Unique" <-
function (X, rows.are.sets = FALSE) 
{
    if (rows.are.sets) 
        X = matsort(X)
    X = X[matorder(X), ]
    dX = apply(X, 2, diff)
    uniq = c(TRUE, ((dX^2) %*% rep(1, ncol(dX))) > 0)
    X = X[uniq, ]
    return(X)
}

"matmin" <-
function (...)
{
    x = cbind(...)
    if(!is.numeric(x))
        stop("Input should by numeric.")
    if (!is.matrix(drop(x)))
        x = t(x)
    x[1:nrow(x) + nrow(x) * (max.col(-x) - 1)]
}

qhull.options <- function(options, output.options, supported_output.options, full=FALSE) {
  if (full) {
    if (!is.null(output.options)) {
      stop("full and output.options should not be specified together")
    }
    output.options = TRUE
    ## Enable message in 0.4.1
    ## Turn to warning in 0.4.7
    message("delaunayn: \"full\" option is deprecated; adding \"Fa\" and \"Fn\" to options.
      Please update your code to use \"output.options=TRUE\" or set \"output.options\" to a
      string containing desired QHull options.")
  }
  
  if (is.null(output.options)) {
    output.options <- ""
  }
  if (is.logical(output.options)) {
    if (output.options) {
      output.options <- paste(supported_output.options, collapse=" ")
    } else {
      output.options  <- ""
    }    
  }
  if (!is.character(output.options)) {
    stop("output.options must be a string, logical or NULL")
  }
  
  ## Input sanitisation
  options <- paste(options, output.options, collapse=" ")
  return(options)
}

##' Compute external- or \sQuote{cross}- product of 3D vectors.
##'
##' Computes the external product
##' \deqn{ \left(x_2 y_3 - x_3 y_2,\;
##'              x_3 y_1 - x_1 y_3,\;
##'              x_1 y_2 - x_2 y_1 \right) }{
##'             (x2 * y3 - x3 * y2,
##'              x3 * y1 - x1 * y3,
##'              x1 * y2 - x2 * y1) }
##' of the 3D vectors in \bold{x} and \bold{y}.
##'
##' @param x \code{n}-by-3 matrix. Each row is one \bold{x}-vector
##' @param y \code{n}-by-3 matrix. Each row is one \bold{y}-vector
##' @param drop logical. If \code{TRUE} and if the inputs are one row
##'   matrices or vectors, then delete the dimensions of the array
##'   returned.
##' @return If \code{n} is greater than 1 or \code{drop} is
##'   \code{FALSE}, \code{n}-by-3 matrix; if \code{n} is 1 and
##'   \code{drop} is \code{TRUE}, a vector of length 3.
##' @author Raoul Grasman
##' @keywords arith math array
##' @seealso \code{\link[base]{drop}}
##' @export
extprod3d <-function (x, y, drop=TRUE)
{
    x = matrix(x, ncol = 3)
    y = matrix(y, ncol = 3)
    z = cbind(x[, 2] * y[, 3] - x[, 3] * y[, 2],
              x[, 3] * y[, 1] - x[, 1] * y[, 3],
              x[, 1] * y[, 2] - x[, 2] * y[, 1])
    if (drop) {
      return(drop(z))
    }
    return(z)
}

## Based on Octave tests for cross.m
## http://hg.savannah.gnu.org/hgweb/octave/file/c2ef0eddf6bc/scripts/linear-algebra/cross.m

context("extprod3d")
test_that("extprod3d gives the expected output", {
  
  x <- c(1, 0, 0)
  y <- c(0, 1, 0)
  r <- c(0, 0, 1)
  expect_equal(extprod3d(x, y), r)
  expect_equal(extprod3d(x, y, drop=FALSE), t(r))
  
  x <- c(1, 2, 3)
  y <- c(4, 5, 6)
  r <- c((2*6-3*5), (3*4-1*6), (1*5-2*4))
  expect_equal(extprod3d(x, y), r)

  x <- rbind(c(1, 0, 0),
             c(0, 1, 0),
             c(0, 0, 1))
  y <- rbind(c(0, 1, 0),
             c(0, 0, 1),
             c(1, 0, 0))
  r <- rbind(c(0, 0, 1),
             c(1, 0, 0),
             c(0, 1, 0))
  expect_equal(extprod3d(x, y), r)

  ##error extprod3d (0,0)
  ##error extprod3d ()
})

##' Compute smallest convex hull that encloses a set of points
##'
##' Returns information about the smallest convex complex of a set of
##' input points in \eqn{N}-dimensional space (the convex hull of the
##' points). By default, indices to points forming the facets of the
##' hull are returned; optionally normals to the facets and the
##' generalised surface area and volume can be returned. This function
##' interfaces the \href{http://www.qhull.org}{Qhull} library.
##'
##' @param p An \eqn{M}-by-\eqn{N} matrix. The rows of \code{p}
##'   represent \eqn{M} points in \eqn{N}-dimensional space.
##'
##' @param options String containing extra options for the underlying
##'   Qhull command; see details below and Qhull documentation at
##'   \url{../doc/qhull/html/qconvex.html#synopsis}.
##'
##' @param output.options String containing Qhull options to generate
##'   extra output. Currently \code{n} (normals) and \code{FA}
##'   (generalised areas and volumes) are supported; see
##'   \sQuote{Value} for details. If \code{output.options} is
##'   \code{TRUE}, select all supported options.
##'
##' @param return.non.triangulated.facets logical defining whether the
##'   output facets should be triangulated; \code{FALSE} by default.
##'
##' @return By default (\code{return.non.triangulated.facets} is
##'   \code{FALSE}), return an \eqn{M}-by-\eqn{N} matrix in which each
##'   row contains the indices of the points in \code{p} forming an
##'   \eqn{N-1}-dimensional facet. e.g In 3 dimensions, there are 3
##'   indices in each row describing the vertices of 2-dimensional
##'   triangles.
##'
##'   If \code{return.non.triangulated.facets} is \code{TRUE} then the
##'   number of columns equals the maximum number of vertices in a
##'   facet, and each row defines a polygon corresponding to a facet
##'   of the convex hull with its vertices followed by \code{NA}s
##'   until the end of the row.
##'
##'   If the \code{output.options} or \code{options} argument contains
##'   \code{FA} or \code{n}, return a list with class \code{convhulln}
##'   comprising the named elements:
##'   \describe{
##'     \item{\code{p}}{The points passed to \code{convnhulln}}
##'     \item{\code{hull}}{The convex hull, represented as a matrix indexing \code{p}, as
##'       described above}
##'     \item{\code{area}}{If \code{FA} is specified, the generalised area of
##'       the hull. This is the surface area of a 3D hull or the length of
##'       the perimeter of a 2D hull.
##'       See \url{../doc/qhull/html/qh-optf.html#FA}.}
##'     \item{\code{vol}}{If \code{FA} is specified, the generalised volume of
##'        the hull. This is volume of a 3D hull or the area of a 2D hull.
##'        See \url{../doc/qhull/html/qh-optf.html#FA}. }
##'     \item{\code{normals}}{If \code{n} is specified, this is a matrix
##'     hyperplane normals with offsets. See \url{../doc/qhull/html/qh-opto.html#n}.}
##'   }
##'
##' @note This function was originally a port of the
##'   \href{https://octave.org/}{Octave} convhulln function written
##'   by Kai Habel.
##'
##' See further notes in \code{\link{delaunayn}}.
##'
##' @author Raoul Grasman, Robert B. Gramacy, Pavlo Mozharovskyi and
##'   David Sterratt \email{david.c.sterratt@@ed.ac.uk}
##' @seealso \code{\link{intersectn}}, \code{\link{delaunayn}},
##'   \code{\link{surf.tri}}, \code{\link[interp]{convex.hull}}
##' @references \cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
##'   \dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM
##'   Trans. on Mathematical Software,} Dec 1996.}
##'
##' \url{http://www.qhull.org}
##' @keywords math dplot graphs
##' @examples
##' ## Points in a sphere
##' ps <- matrix(rnorm(3000), ncol=3)
##' ps <- sqrt(3)*ps/drop(sqrt((ps^2) %*% rep(1, 3)))
##' ts.surf <- t(convhulln(ps))  # see the qhull documentations for the options
##' \dontrun{
##' rgl::triangles3d(ps[ts.surf,1],ps[ts.surf,2],ps[ts.surf,3],col="blue",alpha=.2)
##' for(i in 1:(8*360)) rgl::view3d(i/8)
##' }
##'
##' ## Square
##' pq <- rbox(0, C=0.5, D=2)
##' # Return indices only
##' convhulln(pq)
##' # Return convhulln object with normals, generalised area and volume
##' ch <- convhulln(pq, output.options=TRUE)
##' plot(ch)
##'
##' ## Cube
##' pc <- rbox(0, C=0.5, D=3)
##' # Return indices of triangles on surface
##' convhulln(pc)
##' # Return indices of squares on surface
##' convhulln(pc, return.non.triangulated.facets=TRUE)
##' @export
##' @useDynLib geometry
convhulln <- function (p, options = "Tv", output.options=NULL, return.non.triangulated.facets = FALSE) {
  tmp_stdout <- tempfile("Rf")
  tmp_stderr <- tempfile("Rf")
  on.exit(unlink(c(tmp_stdout, tmp_stderr)))

  ## Combine and check options
  options <- tryCatch(qhull.options(options, output.options, supported_output.options  <- c("n", "FA")), error=function(e) {stop(e)})

  ## Coerce the input to be matrix
  if (is.data.frame(p)) {
    p <- as.matrix(p)
  }

  ## Make sure we have real-valued input
  storage.mode(p) <- "double"

  ## We need to check for NAs in the input, as these will crash the C
  ## code.
  if (any(is.na(p))) {
    stop("The first argument should not contain any NAs")
  }

  if (!return.non.triangulated.facets){
    ## It is essential that delaunayn is called with either the QJ or Qt
    ## option. Otherwise it may return a non-triangulated structure, i.e
    ## one with more than dim+1 points per structure, where dim is the
    ## dimension in which the points p reside.
    if (!grepl("Qt", options) & !grepl("QJ", options)) {
      options <- paste(options, "Qt")
    }
  }
  out <- .Call("C_convhulln", p, as.character(options), as.integer(return.non.triangulated.facets), tmp_stdout, tmp_stderr, PACKAGE="geometry")

  # Remove NULL elements
  out[which(sapply(out, is.null))] <- NULL
  if (is.null(out$area) & is.null(out$vol) & is.null(out$normals)) {
    attr(out$hull, "convhulln") <- attr(out, "convhulln")
    return(out$hull)
  }
  class(out) <- "convhulln"
  out$p <- p
  return(out)
}

##' @importFrom graphics plot
##' @method plot convhulln
##' @export
plot.convhulln <- function(x, y, ...) {
  if (ncol(x$p) < 2 || ncol(x$p) > 3)
    stop("Only 2D and 3D convhullns can be plotted")
  args <- list(...)
  add <- FALSE
  if ("add" %in% names(args)) {
    add <- args$add
    args$add <- NULL
  }
  if (ncol(x$p) == 2) {
    if (!add) {
      plot(x$p[,1], x$p[,2], ...)
    }
    m <- x$hull
    p <- x$p
    do.call(segments, c(list(p[m[,1],1],p[m[,1],2],p[m[,2],1],p[m[,2],2]),
                        args))
  }
  if (ncol(x$p) == 3) {
    if(requireNamespace("rgl") == FALSE)
      stop("The rgl package is required for tetramesh")
    if (!add) rgl::clear3d()
    if (ncol(x$hull) == 3) {
      do.call(rgl::triangles3d,
              c(list(x$p[t(x$hull),1], x$p[t(x$hull),2], x$p[t(x$hull),3]),
                args))
    } else {
      stop("At present only convhullns with triangulated facets can be plotted")
    }
  }
}

##' Convert convhulln object to RGL mesh
##'
##' @param x \code{\link{convhulln}} object
##' @param ... Arguments to \code{\link[rgl]{qmesh3d}} or
##'   \code{\link[rgl]{tmesh3d}}
##' @return \code{\link[rgl]{mesh3d}} object, which can be displayed
##'   in RGL with \code{\link[rgl]{dot3d}}, \code{\link[rgl]{wire3d}}
##'   or \code{\link[rgl]{shade3d}}
##'
##' @seealso \code{\link[rgl]{as.mesh3d}}
##' @export
to.mesh3d <- function(x, ...) UseMethod("to.mesh3d")

##' @importFrom graphics plot
##' @method to.mesh3d convhulln
##' @export
to.mesh3d.convhulln <- function(x, ...) {
  if(requireNamespace("rgl") == FALSE)
    stop("The rgl package is required for as.mesh.convhulln")
  if (ncol(x$p) != 3) {
    stop("Only convex hulls of points in 3D can be turned into meshes")
  }
  if (ncol(x$hull) == 4) {
    stop("At present only convhulls with triangulated facets can be converted to mesh3d")
    ## return(rgl::qmesh3d(t(x$p), t(x$hull), homogeneous=FALSE, ...))
  }
  if (ncol(x$hull) == 3) {
    return(rgl::tmesh3d(t(x$p), t(x$hull), homogeneous=FALSE, ...))
  }
  stop("Facets of hull must be triangles or quadrilaterals")
}


##  LocalWords:  dQuote Qhull param itemize Kai Habel delaunayn Pavlo
##  LocalWords:  Grasman Gramacy Mozharovskyi Sterratt seealso tri ps
##  LocalWords:  interp distmesh intersectn Dobkin Huhdanpaa emph Tv
##  LocalWords:  Quickhull ACM dplot convhulln qhull rnorm ncol sqrt
##  LocalWords:  dontrun useDynLib tmp_stdout tmp_stderr tempdir sanitisation NAs na
##  LocalWords:  QJ grepl importFrom convhulls args requireNamespace
##  LocalWords:  rgl tetramesh eqn href sQuote hyperplane rbox
##  LocalWords:  convulln convhullns

## Copyright (C) 2000, 2001, 2002, 2004, 2005, 2006, 2007, 2009, 2017 Kai Habel
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or (at
## your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## This file has been adapted for R by David C Sterratt

##' Transform Cartesian to spherical coordinates
##'
##' If called with a single matrix argument then each row of \code{c}
##' represents the Cartesian coordinate (\code{x}, \code{y}, \code{z}).
##'
##' @param x x-coordinates or matrix with three columns
##' @param y y-coordinates (optional, if \code{x}) is a matrix 
##' @param z z-coordinates (optional, if \code{x}) is a matrix 
##' @return Matrix with columns:
##' \item{\code{theta}}{the angle relative to the positive x-axis}
##' \item{\code{phi}}{the angle relative to the xy-plane}
##' \item{\code{r}}{the distance to the origin \code{(0, 0, 0)}}
##' @seealso \code{\link{sph2cart}}, \code{\link{cart2pol}},
##'   \code{\link{pol2cart}}
##' @author Kai Habel
##' @author David Sterratt
##' @export
cart2sph <- function(x, y=NULL, z=NULL) {
  if ((is.null(y) & !is.null(z)) |
      (is.null(z) & !is.null(y))) {
    stop("There should be 3 arguments (x, y, z) or one argument (x)")
  }

  if (is.null(y) & is.null(z)) {
    if (!(is.numeric(x))) {
      stop("input must be matrix with 3 columns [x, y, z]")
    }
    if (!(is.matrix(x) & ncol(x) == 3)) {
      stop("matrix input must have 3 columns [x, y, z]")
    }
    z <- x[,3]
    y <- x[,2]    
    x <- x[,1]    
  } else {
    if (!is.numeric(x) | !is.numeric(y) | !is.numeric (z))
      stop("x, y, z must be numeric arrays of the same size, or scalar")

    if ( !(((length(x) == length(y)) | (length(x) == 1) | (length(y) == 1)) &
           ((length(x) == length(z)) | (length(x) == 1) | (length(z) == 1)) &
           ((length(y) == length(z)) | (length(y) == 1) | (length(z) == 1)))) {
      stop("x, y, z must be matrices of the same size, or scalar")
    }
  }

  theta <- atan2(y, x)
  phi <- atan2(z, sqrt(x^2 + y^2))
  r <- sqrt(x^2 + y^2 + z^2)

  return(cbind(theta, phi, r))
}

context("cart2sph")
test_that("cart2sph works correctly", {
  x <- c(0, 1, 2)
  y <- c(0, 1, 2)
  z <- c(0, 1, 2)
  Ps <- cart2sph(x, y, z)
  expect_equal(Ps[,"theta"], c(0, pi/4, pi/4))
  expect_equal(Ps[,"phi"], c(0, 1, 1)*atan(sqrt(0.5)))
  expect_equal(Ps[,"r"], c(0, 1, 2)*sqrt(3))

  x <- 0
  y <- c(0, 1, 2)
  z <- c(0, 1, 2)
  Ps <- cart2sph(x, y, z)
  expect_equal(Ps[,"theta"], c(0, 1, 1)*pi/2)
  expect_equal(Ps[,"phi"], c(0, 1, 1)*pi/4)
  expect_equal(Ps[,"r"], c(0, 1, 2)*sqrt(2))

  x <- c(0, 1, 2)
  y <- 0
  z <- c(0, 1, 2)
  Ps <- cart2sph(x, y, z)
  expect_equal(Ps[,"theta"], c(0, 0, 0))
  expect_equal(Ps[,"phi"], c(0, 1, 1)*pi/4)
  expect_equal(Ps[,"r"], c(0, 1, 2)*sqrt(2))

  x <- c(0, 1, 2)
  y <- c(0, 1, 2)
  z <- 0
  Ps <- cart2sph(x, y, z)
  expect_equal(Ps[,"theta"], c(0, 1, 1)*pi/4)
  expect_equal(Ps[,"phi"], c(0, 0, 0))
  expect_equal(Ps[,"r"], c(0, 1, 2)*sqrt(2))

  C <- rbind(c(0, 0, 0),
             c(1, 0, 1),
             c(2, 0, 2))
  S <- rbind(c(theta=0, phi=0, r=0),
             c(0, pi/4, sqrt(2)),
             c(0, pi/4, 2*sqrt(2)))
  expect_equal(cart2sph(C), S)
})


test_that("cart2sph error validation works correctly", {
  expect_error(cart2sph())
  expect_error(cart2sph(1,2))
  expect_error(cart2sph(1,2,3,4))
  expect_error(cart2sph(list(1, 2, 3)), regexp="input must be matrix with 3 columns")
  expect_error(cart2sph(array(1, c(3,3,2))), regexp="matrix input must have 3 columns")
  expect_error(cart2sph(cbind(1,2,3,4)), regexp=c("matrix input must have 3 columns"))
  expect_error(cart2sph(list(1,2,3), c(1,2,3), c(1,2,3)), regexp="numeric arrays of the same size")
  expect_error(cart2sph(c(1,2,3), list(1,2,3), c(1,2,3), regexp="numeric arrays of the same size"))
  expect_error(cart2sph(c(1,2,3), c(1,2,3), list(1,2,3)), regexp="numeric arrays of the same size")
  expect_error(cart2sph(array(1, c(3, 3, 3)), 1, array(1, c(3,3,2))), regexp="matrices of the same size")
  expect_error(cart2sph(array(1, c(3, 3, 3)), array(1, c(3,3,2)), 1), regexp="matrices of the same size")
})

##' \code{trimesh(T, p)} displays the triangles defined in the m-by-3
##' matrix \code{T} and points \code{p} as a mesh.  Each row of
##' \code{T} specifies a triangle by giving the 3 indices of its
##' points in \code{X}. 
##' 
##' @title Display triangles mesh (2D)
##' @param T T is a \code{m}-by-3 matrix. A row of \code{T} contains
##' indices into \code{X} of the vertices of a triangle. \code{T} is
##' usually the output of \code{\link{delaunayn}}.
##' @param p A vector or a matrix.
##' @param p2 if \code{p} is not a matrix \code{p} and \code{p2} are bind to a
##' matrix with \code{cbind}.
##' @param add Add to existing plot in current active device?
##' @param axis Draw axes?
##' @param boxed Plot box?
##' @param \dots Parameters to the rendering device. See the \link[rgl]{rgl}
##' package.
##' @author Raoul Grasman
##' @seealso \code{\link{tetramesh}}, \code{\link[rgl]{rgl}},
##' \code{\link{delaunayn}}, \code{\link{convhulln}},
##' \code{\link{surf.tri}}
##' @keywords hplot
##' @examples
##' #example trimesh
##' p = cbind(x=rnorm(30), y=rnorm(30))
##' tt = delaunayn(p)
##' trimesh(tt,p)
##' @export
##' @importFrom graphics box plot.new plot.window segments
trimesh <- function(T, p, p2, add=FALSE, axis=FALSE, boxed=FALSE, ...){
  if(!is.matrix(p)){
     p = cbind(p,p2) # automatically generates error if p2 not present
  }
  xlim = range(p[,1])
  ylim = range(p[,2])
  if(!add){
    plot.new()
    plot.window(xlim, ylim, ...)
  }
  if(boxed){
    box()
  }
  if(axis) {
    axis(1)
    axis(2)
  }
  m = rbind(T[,-1], T[, -2], T[, -3])
  segments(p[m[,1],1],p[m[,1],2],p[m[,2],1],p[m[,2],2], ...)
  return(invisible(list(T = T, p = p)))
}

##' Row-wise matrix functions
##' 
##' Compute maximum or minimum of each row, or sort each row of a matrix, or a
##' set of (equal length) vectors.
##' 
##' 
##' @aliases matmax matmin matsort matorder
##' @param \dots A numeric matrix or a set of numeric vectors (that are
##' column-wise bind together into a matrix with \code{cbind}).
##' @return \code{matmin} and \code{matmax} return a vector of length
##' \code{nrow(cbind(...))}.  \code{matsort} returns a matrix of dimension
##' \code{dim(cbind(...))} with in each row of \code{cbind(...)} sorted.
##' \code{matsort(x)} is a lot faster than, e.g., \code{t(apply(x,1,sort))},
##' if \code{x} is tall (i.e., \code{nrow(x)}>>\code{ncol(x)} and
##' \code{ncol(x)}<30. If \code{ncol(x)}>30 then \code{matsort} simply calls
##' `\code{t(apply(x,1,sort))}'.  \code{matorder} returns a permutation which
##' rearranges its first argument into ascending order, breaking ties by
##' further arguments.
##' @author Raoul Grasman
##' @keywords array arith
##' @examples
##' 
##' example(Unique)
##' 
##' @export matmax matmin matsort matorder
"matmax" <-
function (...)
{
    x = cbind(...)
    if(!is.numeric(x))
        stop("Input should by numeric.")
    if (!is.matrix(drop(x)))
        x = t(x)
    x[1:nrow(x) + nrow(x) * (max.col(x) - 1)]
}

##' Compute the signed distances from points \code{p} to a region
##' defined by the difference, union or intersection of regions
##' specified by the functions \code{regionA} and \code{regionB}.
##' \code{regionA} and \code{regionB} must accept a matrix \code{p}
##' with 2 columns as their first argument, and must return a vector
##' of length \code{nrow(p)} containing the signed distances of the
##' supplied points in \code{p} to their respective regions.
##'
##' @title Difference, union and intersection operation on  two regions
##' @return A vector of length \code{nrow(p)} containing the signed
##' distances to the boundary of the region.
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @param p A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
##' representing a point in the plane.
##' @param regionA vectorized function describing region A in the
##' union / intersection / difference
##' @param regionB vectorized function describing region B in the
##' union / intersection / difference
##' @param ... additional arguments passed to \code{regionA} and
##' \code{regionB}
##' @aliases mesh.diff mesh.union mesh.intersect
##' @seealso \code{\link{distmesh2d}}, \code{\link{mesh.dcircle}},
##' \code{\link{mesh.drectangle}} \code{\link{mesh.dsphere}}
##' @export mesh.diff mesh.union mesh.intersect
mesh.diff <-function (p, regionA, regionB, ...) {
  return(matmax(regionA(p, ...), -regionB(p, ...)))
}

## Copyright (C) 2000-2017 Kai Habel
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## This file has been adapted for R by David C Sterratt

##' Transform polar or cylindrical coordinates to Cartesian coordinates.
##'
##' The inputs \code{theta}, \code{r}, (and \code{z}) must be the same shape, or
##' scalar.  If called with a single matrix argument then each row of \code{P}
##' represents the polar/(cylindrical) coordinate (\code{theta}, \code{r}
##' (, \code{z})).
##'
##' @param theta describes the angle relative to the positive x-axis.
##' @param r is the distance to the z-axis (0, 0, z).
##' @param z (optional) is the z-coordinate
##' @return a matrix \code{C} where each row represents one Cartesian
##'   coordinate (\code{x}, \code{y} (, \code{z})).
##' @seealso \code{\link{cart2pol}}, \code{\link{sph2cart}},
##'   \code{\link{cart2sph}}
##' @author Kai Habel
##' @author David Sterratt
##' @export
pol2cart <- function(theta, r=NULL, z=NULL) {

  if (is.null(r) & is.null(z)) {
    if (!(is.numeric(theta))) {
      stop("input must be matrix with 2 or 3 columns")
    }
    if (!(is.numeric (theta) & is.matrix (theta) 
      & (ncol(theta) == 2 | ncol(theta) == 3))) {
      stop("matrix input must have 2 or 3 columns [THETA, R (, Z)]");
    }
    if (ncol(theta) == 3) {
      z <- theta[,3]
    }
    r <- theta[,2]
    theta <- theta[,1]
  } else {
    if  (is.null(z)) {
      if (!is.numeric (theta) | !is.numeric (r)) {
        stop("THETA, R must be numeric arrays of the same size, or scalar")
      }
      if ( !((length(theta) == length(r)) | (length(theta) == 1) | (length(r) == 1))) {
        stop("THETA, Y must be numeric arrays of the same size, or scalar")
      }
    } else {
      if (! is.numeric (theta) | ! is.numeric (r) | ! is.numeric (z)) {
        stop("THETA, R, Z must be numeric arrays of the same size, or scalar")
      }
      if ( !(((length(theta) == length(r)) | (length(theta) == 1) | (length(r) == 1)) &
             ((length(theta) == length(z)) | (length(theta) == 1) | (length(z) == 1)) &
             ((length(r) == length(z)) | (length(r) == 1) | (length(z) == 1)))) {
        stop("theta, r, z must be matrices of the same size, or scalar")
      }
    }
  }

  x <- r*cos(theta)
  y <- r*sin(theta)
  if (is.null(z)) {
    return(cbind(x=x, y=y))
  }
  return(cbind(x=x, y=y, z=z))
}
         

context("pol2cart")
test_that("pol2cart works correctly", {

 t <- c(0, 0.5, 1)*pi
 r <- 1
 C <- pol2cart(t, r)
 expect_equal(C[,"x"], c(1, 0, -1))
 expect_equal(C[,"y"], c(0, 1,  0))


 t <- c(0, 1, 1)*pi/4
 r <- sqrt(2)*c(0, 1, 2)
 C <- pol2cart(t, r)
 expect_equal(C[,"x"], c(0, 1, 2))
 expect_equal(C[,"y"], c(0, 1, 2))

 t <- c(0, 1, 1)*pi/4
 r <- sqrt(2)*c(0, 1, 2)
 z <- c(0, 1, 2)
 C <- pol2cart(t, r, z)
 expect_equal(C[,"x"], c(0, 1, 2))
 expect_equal(C[,"y"], c(0, 1, 2))
 expect_equal(C[,"z"], z)


 t <- 0
 r <- c(0, 1, 2)
 z <- c(0, 1, 2)
 C <- pol2cart (t, r, z)
 expect_equal (C[,"x"], c(0, 1, 2))
 expect_equal (C[,"y"], c(0, 0, 0))
 expect_equal (C[,"z"], z)


 t <- c(1, 1, 1)*pi/4
 r <- 1
 z <- c(0, 1, 2)
 C <- pol2cart (t, r, z)
 expect_equal(C[,"x"], c(1, 1, 1)/sqrt(2))
 expect_equal(C[,"y"], c(1, 1, 1)/sqrt(2))
 expect_equal(C[,"z"], z)

 
 t <- 0
 r <- c(1, 2, 3)
 z <- 1
 C <- pol2cart (t, r, z)
 expect_equal(C[,"x"], c(1, 2, 3))
 expect_equal(C[,"y"], c(0, 0, 0)/sqrt (2))
 expect_equal(C[,"z"], c(1, 1, 1))

 P <- rbind(c(theta=0, r=0),
            c(pi/4, sqrt(2)),
            c(pi/4, 2*sqrt(2)))
 C <- rbind(c(x=0, y=0),
            c(1, 1),
            c(2, 2))
 expect_equal(pol2cart(P), C)

## %!test
## %! P <- c(0, 0, 0 pi/4, sqrt(2), 1 pi/4, 2*sqrt(2), 2)
## %! C <- c(0, 0, 0 1, 1, 1 2, 2, 2)
## %! expect_equal (pol2cart (P), C, sqrt (eps))

## %!test
## %! r <- ones (1, 1, 1, 2)
## %! r(1, 1, 1, 2) <- 2
## %! t <- pi/2 * r
## %! c(x, y) <- pol2cart (t, r)
## %! X <- zeros (1, 1, 1, 2)
## %! X(1, 1, 1, 2) <- -2
## %! Y <- zeros (1, 1, 1, 2)
## %! Y(1, 1, 1, 1) <- 1
## %! expect_equal (C[,"x"], X, 2*eps)
## %! expect_equal (C[,"y"], Y, 2*eps)

## %!test
## %! c(t, r, Z) <- meshgrid (c(0, pi/2), c(1, 2), c(0, 1))
## %! c(x, y, z) <- pol2cart (t, r, Z)
## %! X <- zeros(2, 2, 2)
## %! X(:, 1, 1) <- c(1 2)
## %! X(:, 1, 2) <- c(1 2)
## %! Y <- zeros(2, 2, 2)
## %! Y(:, 2, 1) <- c(1 2)
## %! Y(:, 2, 2) <- c(1 2)
## %! expect_equal (C[,"x"], X, eps)
## %! expect_equal (C[,"y"], Y, eps)
## %! expect_equal (z, Z)

 ## Test input validation
 expect_error(pol2cart())
 expect_error(pol2cart(1,2,3,4))
 expect_error(pol2cart(list(1,2,3)), regexp="input must be matrix with 2 or 3 columns")
## %expect_error <matrix input must have 2 or 3 columns> pol2cart (ones (3,3,2))
## %expect_error <matrix input must have 2 or 3 columns> pol2cart (c(1))
## %expect_error <matrix input must have 2 or 3 columns> pol2cart (c(1,2,3,4))
## %expect_error <numeric arrays of the same size> pol2cart ({1,2,3}, c(1,2,3))
## %expect_error <numeric arrays of the same size> pol2cart (c(1,2,3), {1,2,3})
## %expect_error <numeric arrays of the same size> pol2cart (ones (3,3,3), ones (3,2,3))
## %expect_error <numeric arrays of the same size> pol2cart ({1,2,3}, c(1,2,3), c(1,2,3))
## %expect_error <numeric arrays of the same size> pol2cart (c(1,2,3), {1,2,3}, c(1,2,3))
## %expect_error <numeric arrays of the same size> pol2cart (c(1,2,3), c(1,2,3), {1,2,3})
## %expect_error <numeric arrays of the same size> pol2cart (ones (3,3,3), 1, ones (3,2,3))
## %expect_error <numeric arrays of the same size> pol2cart (ones (3,3,3), ones (3,2,3), 1)
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' @importFrom Rcpp sourceCpp
C_tsearch <- function(x, y, elem, xi, yi, bary = FALSE, eps = 1.0e-12) {
    .Call('_geometry_C_tsearch', PACKAGE = 'geometry', x, y, elem, xi, yi, bary, eps)
}

##' Retrieve or set a list of array element values
##' 
##' \code{entry.value} retrieves or sets the values in an array \code{a} at the
##' positions indicated by the rows of a matrix \code{idx}.
##' 
##' 
##' @aliases entry.value entry.value<-
##' @param a An array.
##' @param idx Numerical matrix with the same number of columns as the number
##' of dimensions of \code{a}.  Each row indices a cell in \code{a} of which
##' the value is to be retrieved or set.
##' @param value An array of length \code{nrow(idx)}.
##' @return \code{entry.value(a,idx)} returns a vector of values at the
##' indicated cells.  \code{entry.value(a,idx) <- val} changes the indicated
##' cells of \code{a} to \code{val}.
##' @author Raoul Grasman
##' @keywords arith math array
##' @examples
##' 
##' a = array(1:(4^4),c(4,4,4,4))
##' entry.value(a,cbind(1:4,1:4,1:4,1:4))
##' entry.value(a,cbind(1:4,1:4,1:4,1:4)) <- 0
##' 
##' entry.value(a, as.matrix(expand.grid(1:4,1:4,1:4,1:4)))
##'      # same as `c(a[1:4,1:4,1:4,1:4])' which is same as `c(a)'
##'
##' @export entry.value entry.value<-
"entry.value" <-
function (a, idx) 
{
    if (!is.array(a)) 
        stop("First argument `", deparse(substitute(a)), "' should be an array.")
    if (!is.matrix(idx)) 
        stop("Second argument `", substitute(idx), "' should be a matrix.")
    n <- length(dim(a))
    if (n != ncol(idx)) 
        stop("Number of columns in", deparse(substitute(idx)),
             "is incompatible is dimension of", deparse(substitute(a)))
    a[(idx - 1) %*% c(1, cumprod(dim(a))[-n]) + 1]
}
"entry.value<-" <-
function (a, idx, value) 
{
    if (!is.array(a)) 
        stop("First argument `", deparse(substitute(a)), "' should be an array.")
    if (!is.matrix(idx)) 
        stop("Second argument `", substitute(idx), "' should be a matrix.")
    n <- length(dim(a))
    if (n != ncol(idx)) 
        stop("Number of columns in", deparse(substitute(idx)),
            "is incompatible is dimension of", deparse(substitute(a)))
    a[(idx - 1) %*% c(1, cumprod(dim(a))[-n]) + 1] <- value
    return(a)
}

## Copyright (C) 2000-2017 Kai Habel
##
## This file is part of Octave.
##
## Octave is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## Octave is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Octave; see the file COPYING.  If not, see
## <http://www.gnu.org/licenses/>.

## This file has been adapted for R by David C Sterratt

##' Transform Cartesian coordinates to polar or cylindrical coordinates.
##'
##' The inputs \code{x}, \code{y} (, and \code{z}) must be the same shape, or
##' scalar.  If called with a single matrix argument then each row of \code{C}
##' represents the Cartesian coordinate (\code{x}, \code{y} (, \code{z})).
##'
##' @param x x-coordinates or matrix with three columns
##' @param y y-coordinates (optional, if \code{x}) is a matrix 
##' @param z z-coordinates (optional, if \code{x}) is a matrix 
##' @return A matrix \code{P} where each row represents one
##'   polar/(cylindrical) coordinate (\code{theta}, \code{r}, (,
##'   \code{z})).
##' @seealso \code{\link{pol2cart}}, \code{\link{cart2sph}},
##'   \code{\link{sph2cart}}
##' @author Kai Habel
##' @author David Sterratt
##' @export
cart2pol <- function(x, y=NULL, z=NULL) {

  if (is.null(y) & is.null(z)) {
    if (!(is.numeric(x))) {
      stop("input must be matrix with 2 or 3 columns")
    }
    if (!(is.numeric (x) & is.matrix (x) 
      & (ncol(x) == 2 | ncol(x) == 3))) {
      stop("matrix input must have 2 or 3 columns [X, Y (, Z)]");
    }
    if (ncol(x) == 3) {
      z <- x[,3]
    }
    y <- x[,2]
    x <- x[,1]
  } else {
    if  (is.null(z)) {
      if (!is.numeric (x) | !is.numeric (y)) {
        stop("X, Y must be numeric arrays of the same size, or scalar")
      }
      if ( !((length(x) == length(y)) | (length(x) == 1) | (length(y) == 1))) {
        stop("X, Y must be numeric arrays of the same size, or scalar")
      }
    } else {
      if (! is.numeric (x) | ! is.numeric (y) | ! is.numeric (z)) {
        stop("X, Y, Z must be numeric arrays of the same size, or scalar")
      }
      if ( !(((length(x) == length(y)) | (length(x) == 1) | (length(y) == 1)) &
             ((length(x) == length(z)) | (length(x) == 1) | (length(z) == 1)) &
             ((length(y) == length(z)) | (length(y) == 1) | (length(z) == 1)))) {
        stop("x, y, z must be matrices of the same size, or scalar")
      }
    }
  }
  theta <- atan2 (y, x)
  r <- sqrt(x^2 + y^2)
  if (is.null(z)) {
    return(cbind(theta=theta, r=r))
  }
  return(cbind(theta=theta, r=r, z=z))
}

context("cart2pol")
test_that("cart2pol works correctly", {
  x <- c(0, 1, 2)
  y <- 0
  P <- cart2pol (x, y)
  expect_equal (P[,"theta"], c(0, 0, 0))
  expect_equal (P[,"r"], x)

  x <- c(0, 1, 2)
  y <- c(0, 1, 2)
  P <- cart2pol(x, y)
  expect_equal (P[,"theta"], c(0, pi/4, pi/4))
  expect_equal (P[,"r"], sqrt (2)*c(0, 1, 2))

  x <- c(0, 1, 2)
  y <- c(0, 1, 2)
  z <- c(0, 1, 2)
  P <- cart2pol (x, y, z)
  expect_equal (P[,"theta"], c(0, pi/4, pi/4))
  expect_equal (P[,"r"], sqrt (2)*c(0, 1, 2))
  expect_equal (P[,"z"], z)

  x <- c(0, 1, 2)
  y <- 0
  z <- 0
  P <- cart2pol (x, y, z)
  expect_equal (P[,"theta"], c(0, 0, 0))
  expect_equal (P[,"r"], x)
  expect_equal (P[,"z"], c(0, 0, 0))

  x <- 0
  y <- c(0, 1, 2)
  z <- 0
  P <- cart2pol (x, y, z)
  expect_equal (P[,"theta"], c(0, 1, 1)*pi/2)
  expect_equal (P[,"r"], y)
  expect_equal (P[,"z"], c(0, 0, 0))

  x <- 0
  y <- 0
  z <- c(0, 1, 2)
  P <- cart2pol (x, y, z)
  expect_equal (P[,"theta"], c(0, 0, 0))
  expect_equal (P[,"r"], c(0, 0, 0))
  expect_equal (P[,"z"], z)

  C <- rbind(c(x=0, y=0), c(1, 1), c( 2, 2))
  P <- rbind(c(theta=0, r=0), c(pi/4, sqrt(2)), c(pi/4, 2*sqrt(2)))
  expect_equal(cart2pol(C), P)

## %!test
## %! C <- c(0, 0, 0 1, 1, 1 2, 2, 2)
## %! P <- c(0, 0, 0 pi/4, sqrt(2), 1 pi/4, 2*sqrt(2), 2)
## %! expect_equal (cart2pol (C), P)

## %!test
## %! x <- zeros (1, 1, 1, 2)
## %! x(1, 1, 1, 2) <- sqrt (2)
## %! y <- x
## %! c(P[,"theta"], r) <- cart2pol (x, y)
## %! T <- zeros (1, 1, 1, 2)
## %! T(1, 1, 1, 2) <- pi/4
## %! R <- zeros (1, 1, 1, 2)
## %! R(1, 1, 1, 2) <- 2
## %! expect_equal (P[,"theta"], T)
## %! expect_equal (P[,"r"], R)

## %!test
## %! c(x, y, Z) <- meshgrid (c(0, 1), c(0, 1), c(0, 1))
## %! c(t, r, z) <- cart2pol (x, y, Z)
## %! T(:, :, 1) <- c(0, 0 pi/2, pi/4)
## %! T(:, :, 2) <- T(:, :, 1)
## %! R <- sqrt (x.^2 + y.^2)
## %! expect_equal (t, T)
## %! expect_equal (P[,"r"], R)
## %! expect_equal (z, Z)

  ## Test input validation
  expect_error(cart2pol())
  expect_error(cart2pol(1,2,3,4))
  expect_error(cart2pol(list(1,2,3)), regexp="input must be matrix with 2 or 3 columns")
## expect_error <matrix input must have 2 or 3 columns> cart2pol (ones (3,3,2))
## expect_error <matrix input must have 2 or 3 columns> cart2pol (c(1))
## expect_error <matrix input must have 2 or 3 columns> cart2pol (c(1,2,3,4))
## expect_error <numeric arrays of the same size> cart2pol ({1,2,3}, c(1,2,3))
## expect_error <numeric arrays of the same size> cart2pol (c(1,2,3), {1,2,3})
## expect_error <numeric arrays of the same size> cart2pol (ones (3,3,3), ones (3,2,3))
## expect_error <numeric arrays of the same size> cart2pol ({1,2,3}, c(1,2,3), c(1,2,3))
## expect_error <numeric arrays of the same size> cart2pol (c(1,2,3), {1,2,3}, c(1,2,3))
## expect_error <numeric arrays of the same size> cart2pol (c(1,2,3), c(1,2,3), {1,2,3})
## expect_error <numeric arrays of the same size> cart2pol (ones (3,3,3), 1, ones (3,2,3))
## expect_error <numeric arrays of the same size> cart2pol (ones (3,3,3), ones (3,2,3), 1)
})

##' Compute convex hull of intersection of two sets of points
##' @param ps1 First set of points
##' @param ps2 Second set of points
##' @param tol Tolerance used to determine if a feasible point lies
##'   within the convex hulls of both points and to round off the
##'   points generated by the halfspace intersection, which sometimes
##'   produces points very close together.
##' @param return.chs If \code{TRUE} (default) return the convex hulls
##'   of the first and second sets of points, as well as the convex
##'   hull of the intersection.
##' @param options Options passed to \code{\link{halfspacen}}. By
##'   default this is \code{Tv}.
##' @param fp Coordinates of feasible point, i.e. a point known to lie
##'   in the hulls of \code{ps1} and \code{ps2}. The feasible point is
##'   required for \code{\link{halfspacen}} to find the intersection.
##'   \code{intersectn} tries to find the feasible point automatically
##'   using the linear program in \code{\link{feasible.point}}, but
##'   currently the linear program fails on some examples where there
##'   is an obvious solution. This option overrides the automatic
##'   search for a feasible point
##' @param autoscale \emph{Experimental in v0.4.2} Automatically scale
##'   the points to lie in a sensible numeric range. May help to
##'   correct some numerical issues.
##' @return List containing named elements: \code{ch1}, the convex
##'   hull of the first set of points, with volumes, areas and normals
##'   (see \code{\link{convhulln}}; \code{ch2}, the convex hull of the
##'   first set of points, with volumes, areas and normals; \code{ps},
##'   the intersection points of convex hulls \code{ch1} and
##'   \code{ch2}; and \code{ch}, the convex hull of the intersection
##'   points, with volumes, areas and normals.
##' @export
##' @examples
##' # Two overlapping boxes
##' ps1 <- rbox(0, C=0.5)
##' ps2 <- rbox(0, C=0.5) + 0.5
##' out <- intersectn(ps1, ps2)
##' message("Volume of 1st convex hull: ", out$ch1$vol)
##' message("Volume of 2nd convex hull: ", out$ch2$vol)
##' message("Volume of intersection convex hull: ", out$ch$vol)
##' @author David Sterratt
##' @note \code{intersectn} was introduced in geometry 0.4.0, and is
##'   still under development. It is worth checking results for
##'   unexpected behaviour.
##' @seealso \code{\link{convhulln}}, \code{\link{halfspacen}},
##'   \code{\link{inhulln}}, \code{\link{feasible.point}}
##' @importFrom utils packageDescription
intersectn <- function(ps1, ps2, tol=0, return.chs=TRUE, options="Tv",
                       fp=NULL, autoscale=FALSE) {
  distinct <-
    any(apply(ps1, 2, min) > apply(ps2, 2, max)) ||
    any(apply(ps1, 2, max) < apply(ps2, 2, min))
  
  if (distinct & !return.chs) {
    return(list(ch=list(vol=0)))
  }
  
  ch1 <- convhulln(ps1, "n FA")
  ch2 <- convhulln(ps2, "n FA")

  if (distinct) {
    return(list(ch1=ch1, ch2=ch2, ch=list(vol=0)))
  }

  ch1s <- ch1
  ch2s <- ch2
  if (autoscale) {
    pmean <- colMeans(rbind(ps1, ps2))
    ch1s <- convhulln(t(t(ps1) - pmean), "n FA")
    ch2s <- convhulln(t(t(ps2) - pmean), "n FA")
    if (!is.null(fp)) {
      fp <- fp - pmean
    }
  }
  
  ## Find feasible point in which points could overlap
  if (is.null(fp)) {
    fp <-tryCatch(feasible.point(ch1s, ch2s, tol=tol),
                  error=function(e){
                    stop("feasible.point() failed with error ",
                         e$message, "\n",
                         "If you can find a feasible point (i.e. point that lies in both hulls)\n",
                         "for this input, supply this with the \"fp\" option.\n",
                         "Otherwise, report bug, including inputs to intersectn() at\n",
                         utils::packageDescription("geometry", fields="BugReports"), "\n",
                         "or to ",
                         utils::packageDescription("geometry", fields="Maintainer"))
                  })
    if (all(is.na(fp))) {
      if (return.chs) {
        return(list(ch1=ch1, ch2=ch2, ch=list(vol=0)))
      }
      return(list(ch=list(vol=0)))
    }
  } else {
    ## fp supplied
    if (!is.numeric(fp)) {
      stop("fp should be numeric")
    }
    if (length(fp) != ncol(ps1)) {
      stop("fp should have same dimension as ps1 and ps2")
    }
  }
  
  ## Find intersections of halfspaces about feasible point. Catch error
  ## (code QH6023) when fixed point is not in intersection, due to
  ## precision issue.
  ps <- tryCatch(halfspacen(rbind(ch1s$normals, ch2s$normals), fp, options=options),
                 error=function(e) {
                   if (grepl("QH6023", e$message)) {
                     return(NA)
                   }
                   errmess <- e$message
                   if (!grepl("QJ", options)) {
                     errmess <- paste(errmess, "\nTry calling intersectn with options=\"Tv QJ\"")
                   }
                   stop(errmess)
                 })
  if (all(is.na(ps)) || is.null(ps)) {
    if (return.chs) {
      return(list(ch1=ch1, ch2=ch2, ch=list(vol=0)))
    }
    return(list(ch=list(vol=0)))
  }

  if (autoscale) {
    ps <- t(t(ps) + pmean)
  }
  
  ## Occasionally the halfspace creates points very close together. We
  ## can impose a tolerance to merge them
  if (tol != 0) {
    ps <- round(ps, ceiling(-log10(abs(tol/2))))
    ps <- ps[!duplicated(ps),]
  }
  ch <- convhulln(ps, "n FA")

  ## Check for gross volume errors
  if ((ch$vol > ch1$vol * (1 + 1E-4))) {
    warning("Volume of final intersection hull is bigger than first of the original hulls\n",
         "ch1 vol = ", ch1$vol, "\n",
         "ch vol = ", ch$vol, "\n",
         "Returning ch1")
    ch <- ch1
  }
  if ((ch$vol > ch2$vol * (1 + 1E-4))) {
    warning("Volume of final intersection hull is bigger than first of the original hulls\n",
         "ch2 vol = ", ch2$vol, "\n",
         "ch vol = ", ch$vol, "\n",
         "Returning ch2")
    ch <- ch2
  }
  
  
  if (return.chs) {
    out <- list(ch1=ch1, ch2=ch2, ps=ps, ch=ch)
    class(out) <- "intersectn"
    return(out)
  }
  return(list(ch=ch))
}

##' Find point in intersection of convex hulls
##'
##' Find point that lies somewhere in interesction of two convex
##' hulls. If such a point does not exist, return \code{NA}. The
##' feasible point is found using a linear program similar to the one
##' suggested at \url{../doc/qhull/html/qhalf.html#notes}
##' 
##' @param ch1 First convex hull with normals
##' @param ch2 Second convex hull with normals
##' @param tol The point must be at least this far within the facets
##'   of both convex hulls
##' @export
feasible.point <- function(ch1, ch2, tol=0) {
  N <- ncol(ch1$p)                      # Number of dimensions
  M <- nrow(ch1$normals) + nrow(ch2$normals) # Total number of normals
  
  ## Find the point that bounds all points from below. Becuase
  ## lpSolve::lp() implicitly gives solutions in the positive
  ## quadrant, we need to subtract p0 from the search point, and then
  ## add it to the optimised solution. This will ensure that solutions
  ## not in the positive quadrant are found.
  p0 <- apply(rbind(ch1$p, ch2$p), 2, min)
  objective.in <- c(rep(0, N), 1)
  const.mat <- rbind(cbind(ch1$normals[,-(N + 1)], 1),
                     cbind(ch2$normals[,-(N + 1)], 1),
                     c(rep(0, N), -1))

  ## p0 is incorporated into the matrix here
  const.rhs <- -c(const.mat[1:M, 1:N] %*% cbind(p0) +
                  c(ch1$normals[,N + 1], ch2$normals[,N + 1]),
                  tol)
  const.dir <- c(rep("<", length(const.rhs)))

  ## Scaling: The scale option of lpSolve::lp() determines options
  ## used for scaling, and is crucial to avoid errors in some edge
  ## cases. For list of options, see:
  ## http://lpsolve.sourceforge.net/5.1/set_scaling.htm
  ## http://lpsolve.sourceforge.net/5.1/scaling.htm
  ## See also https://github.com/davidcsterratt/geometry/issues/35
  ##
  ## After testing on 10,000+ examples, it appears that to get
  ## maxiumum coverage, in some cases multiple combinations of options
  ## need to be tried. This code cycles through options, starting with
  ## the combinations most likely to work.

  ## DYNUPDATE == 0 may also work, but DYNUPDATE == 1 may work better
  ## for some 4D examples
  DYNUPDATE <- 1
  ## Both options may help
  for (POWER2 in 0:1) {
    ## Both options may help
    for (QUADRATIC in 0:1) {
      ## 1 (SCALE_EXTREME) and 3 (SCALE_EXTREME) didn't work well in
      ## tests
      ##
      ## 4 (SCALE_GEOMETRIC) on one example (included in tests) caused
      ## some sort of infinite loop or race condition leading to the
      ## process up 100%
      ##
      ## 2 (SCALE_RANGE) and 7 (SCALE_CURTISREID) seem to work OK for
      ## most cases. SCALE_CURTISREID (with the 5.6.13 version of
      ## lpSolve, at least) has caused some sort of infinite loop or
      ## race condition leading to the process up 100% on 2 out of
      ## millions of intersections computed on some processors, hence
      ## it is put as the second option.
      for (MAIN in c(2, 7)) {
        ## Both options may help
        for (LOGARITHMIC in 0:1) {
          ## Equilibriate seemed to help for some cases, but caused a
          ## crash in the example of inst/extdata
          EQUILIBRIATE <- 0 
          scale <-
            MAIN +
            QUADRATIC    *   8 +
            LOGARITHMIC  *  16 +
            POWER2       *  32 +
            EQUILIBRIATE *  64 +
            DYNUPDATE    * 256
          opt <- lpSolve::lp(direction = "max",
                             objective.in,
                             const.mat,
                             const.dir,
                             const.rhs,
                             scale=scale)
          ## See http://lpsolve.sourceforge.net/5.5/solve.htm for status codes
          ## Infeasible solution
          if (opt$status == 2) return(NA)
          ## Optimal
          if (opt$status == 0) return(opt$solution[1:N] + p0)
        }
      }
    }
  }
  ## Debugging output
  if (!is.null(getOption("geometry.debug"))) {
    opt <- linprog::writeMps("feasible-point.mps",
                             cvec=objective.in,
                             bvec=const.rhs,
                             Amat=const.mat, "Feasible point")
    writeLines(gsub("_ ", "_", readLines("feasible-point-tmp.mps")), "feasible-point.mps")
    message("Debugging output saved to feasible-point.mps\n",
            "Test on command line by running\n",
            "  lp_solve -max -fmps feasible-point.mps")
  }
  stop("lpSolve::lp() returned error code ", opt$status, "\n",
       "See http://lpsolve.sourceforge.net/5.5/solve.htm for explanation of errors.\n",
       "Rescaling your problem may help")
}

##' @method plot intersectn
##' @export 
plot.intersectn <- function(x, y, ...) {
  args <- list(...)
  add <- FALSE
  if ("add" %in% names(args)) {
    add <- args$add
    args$add <- NULL
  }
  xlim <- ylim <- NULL
  if ("xlim" %in% names(args)) {
    xlim <- args$xlim
    args$xlim <- NULL
  }
  if ("ylim" %in% names(args)) {
    ylim <- args$ylim
    args$xlim <- NULL
  }
  if (ncol(x$p) == 2) {
    if (!add) {
      p <- rbind(x$ch1$p, x$ch2$p)
      if (is.null(xlim)) xlim <- range(p[,1])
      if (is.null(ylim)) ylim <- range(p[,2])
      plot.new()
      do.call(plot.window, c(list(xlim=xlim, ylim=ylim), args))
    }
    plot(x$ch1, add=TRUE, lty=2)
    plot(x$ch2, add=TRUE, lty=2)
    plot(x$ch, add=TRUE, lwd=2)
  }
}

test_that("intersectn can run on overlapping triangles", {
  ## Make star of David from isosceles triangles of length 3
  ps1 <- rbind(c(0,   sqrt(3)),
               c(3/2, -sqrt(3)/2),
               c(-3/2, -sqrt(3)/2))
  ps2 <- ps1
  ps2[,2] <- -ps2[,2]

  expect_equal(feasible.point(convhulln(ps1, output.options=TRUE),
                              convhulln(ps2, output.options=TRUE)),
               c(0, 0))
  is <-  intersectn(ps1, ps2)
  isa <-  intersectn(ps1, ps2, autoscale=TRUE)
  ## Intersecting area is same as 6 isosceles triangles of length 1, which have 
  ## area sqrt(3)/4
  ## 
  expect_equal(is$ch$vol, sqrt(3)/4*6)
  expect_equal(isa$ch$vol, sqrt(3)/4*6)

  ## Another overlapping example
  ps2 <- ps1
  ps2[,2] <- ps2[,2]+2
  is <-  intersectn(ps1, ps2)

  ## Now make one element of feasible point negative
  ps3 <- ps1
  ps4 <- ps1
  ps4[,2] <- -ps4[,2]
  ps3[,2] <- ps3[,2] - 10
  ps4[,2] <- ps4[,2] - 10

  expect_equal(feasible.point(convhulln(ps3, output.options=TRUE),
                              convhulln(ps4, output.options=TRUE)),
               c(0, -10))
  expect_equal(intersectn(ps3, ps4)$ch$vol, sqrt(3)/4*6)
})

test_that("intersectn gives zero volume on non-overlapping triangles", {
  ps1 <- rbind(c(0,   sqrt(3)),
               c(3/2, -sqrt(3)/2),
               c(-3/2, -sqrt(3)/2))
  ps2 <- ps1
  ps2[,2] <- ps2[,2] + 3
  
  expect_equal(feasible.point(convhulln(ps1, "n"), convhulln(ps2, "n")), NA)
  is <-  intersectn(ps1, ps2)
  expect_equal(is$ch$vol, 0)
  
})


test_that("intersectn gives zero volume on non-overlapping triangles", {
  ps1 <- rbind(c(0,   sqrt(3)),
               c(3/2, -sqrt(3)/2),
               c(-3/2, -sqrt(3)/2))
  ps2 <- ps1
  ps2[,2] <- ps2[,2] + 3
  
  expect_equal(feasible.point(convhulln(ps1, "n"), convhulln(ps2, "n")), NA)
  is <-  intersectn(ps1, ps2)
  expect_equal(is$ch$vol, 0)
  
})

test_that("feasible.point works on a 3D example", {
  ## These tetrahedra do not overlap
  ps1 <- rbind(c( 0.5000000, -0.5000000,  0.5000000),
               c(-0.1018942,  0.1848312, -0.1260239),
               c( 0.5000000, -0.5000000, -0.5000000),
               c(-0.5000000, -0.5000000, -0.5000000))
  ps2 <- rbind(c( 0.7581575,  0.6352585,  0.32876),
               c( 1.0000000,  0.0000000,  1.00000),
               c( 0.0000000,  0.0000000,  1.00000),
               c( 1.0000000,  0.0000000,  0.00000))
  expect_equal(feasible.point(convhulln(ps1, "n"), convhulln(ps2, "n")), NA)
})

test_that("intersectn can run on overlapping tetrahedra", {
  ## Make star of David from isocelese triangles of length 3
  ps1 <- rbind(c(0,   sqrt(3), 0),
               c(3/2, -sqrt(3)/2, 0),
               c(-3/2, -sqrt(3)/2, 0),
               c(0, 0, 3*sqrt(2/3)))
  ch1 <- convhulln(ps1, "FA")
  expect_equal(ch1$vol, sqrt(2)/12*27)
  ps2 <- ps1

  ## By shifting tetrahedron up by half of its height, we should make
  ## something with 1/8 of the volume
  ps2[,3] <- ps2[,3] + 3/2*sqrt(2/3)
  is <-  intersectn(ps1, ps2)
  expect_equal(is$ch$vol, sqrt(2)/12*27/8)
  
})

test_that("intersectn can run on  tetrahedra with a common point", {
  ps1 <- rbind(c(-0.4015654, -0.1084358, -0.3727391),
               c( 0.2384763,  0.3896078, -0.4447473),
               c( 0.5000000, -0.5000000, -0.5000000),
               c(-0.5000000, -0.5000000, -0.5000000))
  ps2 <- rbind(c(-0.1392469,  0.03303547, -0.2436112),
               c( 0.3434195, -0.20338201, -0.4638141),
               c(-0.5000000,  0.50000000, -0.5000000),
               c(-0.5000000, -0.50000000, -0.5000000))
  is <-  intersectn(ps1, ps2)
})

test_that("intersectn can compute the volume of overlapping delaunay triangulations of boxes", {
  ## Volume of overlap should be 1/8
  ps1 <- rbox(2, B=0.5, C=0.5)
  ps2 <- rbox(2, B=0.5, C=0.5) + 0.5
  dt1 <- delaunayn(ps1)
  dt2 <- delaunayn(ps2)

  vol <- 0
  for (i in 1:nrow(dt1)) {
    for (j in 1:nrow(dt2)) {
      is <- intersectn(ps1[dt1[i,],], ps2[dt2[j,],])
      vol <- vol + is$ch$vol
    }
  }
  expect_equal(vol, 0.125, tol=0.0001)
})

test_that("intersectn can deal with some input that caused errors before fixing Issue #34", {
  ## Issue 34: https://github.com/davidcsterratt/geometry/issues/34
  ps1 <- rbind(
    c(500.9656357388012111187, 843268.9656357388012111, 5.5),
    c(658.9656357388012111187, 843109.9656357388012111, 10.0),
    c(576.9656357388012111187, 843174.9656357388012111,  2.0),
    c(795.9656357388012111187, 843235.9656357388012111, 20.0))
  ps2 <- rbind(
    c(707.9656400000000076034, 843153.9656399999512359, 12.000000000000000000000),
    c(645.6795799999999871943, 843166.4228499999735504, 10.200630000000000308091),
    c(631.6632399999999734064, 843182.9680800000205636,  8.772800000000000153477),
    c(707.9656400000000076034, 843153.9656399999512359, 12.000000000000000000000),
    c(608.9447900000000117871, 843172.7368899999419227,  7.772330000000000183036),
    c(607.9656400000000076034, 843173.9656399999512359,  7.669999999999999928946))
  ## Before Issue #34 was fixed this threw an error:
  ##   Received error code 2 from qhull. Qhull error:
  ##     qhull precision warning: 
  ##     The initial hull is narrow (cosine of min. angle is 1.0000000000000002).
  ## expect_error(intersectn(ps1, ps2, tol=1E-4, return.chs=FALSE, options="Tv"), ".*The initial hull is narrow.*")
  
  ## This threw an error in Rev aab45b7311b6
  out <- intersectn(ps1, ps2, tol=1E-4, return.chs=FALSE)
})

test_that("intersectn works on rotated boxes", {
  rot <- function(theta) {return(rbind(c(cos(theta), sin(theta)), c(-sin(theta), cos(theta))))}
  ## Area of octogan created by two squares at 45 deg to each other
  sq <- rbox(C=1, D=2, n=0)
  expect_equal(intersectn(sq%*%rot(pi/4), sq)$ch$vol, 8*(sqrt(2) - 1))
  
  rot4 <- function(theta) {return(rbind(c(cos(theta), sin(theta), 0, 0), c(-sin(theta), cos(theta), 0, 0), c(0, 0, 1, 0), c(0, 0, 0 ,1)))}
  ## Area of hyperoctoid created by two hypercubes at 45 deg to each other
  hc <- rbox(C=1, D=4, n=0)
  expect_equal(intersectn(hc%*%rot4(pi/4), hc)$ch$vol, 4*8*(sqrt(2) - 1))
})

test_that("intersectn works in 4D", {
    load(file.path(system.file(package="geometry"), "extdata", "intersectn4D.RData"))
  chi <- convhulln(seti, output.options=TRUE)
  chj <- convhulln(setj, output.options=TRUE)
  chij <- intersectn(seti, setj)
  chji <- intersectn(setj, seti)
  expect_equal(chij$ch$vol, chji$ch$vol)
  expect_true(chi$vol >= chij$ch$vol)
  expect_equal(chj$vol, chij$ch$vol)
})

test_that("no regression on issue 35", {
  ## This gave an error in version 0.4.1
  ## See https://github.com/davidcsterratt/geometry/issues/35
  load(file.path(system.file(package="geometry"), "extdata", "issue35-intersectn.RData"))

  ch <- intersectn(seti, setj)
  expect_true(ch$ch$vol > 0)
  cha <- intersectn(seti, setj, autoscale=TRUE)
  expect_true(cha$ch$vol > 0)
  expect_equal(ch$ch$vol, cha$ch$vol)
})

test_that("no regression on issue 35", {
  ## This is an example that requires various combinations of flags to
  ## be provided to lpSolve::lp
  ##
  ## Also testing a scaled version, which was easier to fixed with the
  ## set of flags used originally.
  ## https://github.com/davidcsterratt/geometry/issues/35
  load(file.path(system.file(package="geometry"), "extdata", "error_15_620.RData"))
  ch <- intersectn(p1, p1)
  expect_true(ch$ch$vol > 0)
  cha <- intersectn(p1, p1, autoscale=TRUE)
  expect_true(cha$ch$vol > 0)
  expect_equal(ch$ch$vol, cha$ch$vol)
  
  zfac <- 10
  p1[,3] <- p1[,3]*zfac
  p2[,3] <- p2[,3]*zfac
  ch <- intersectn(p1, p1)
  expect_true(ch$ch$vol > 0)
  cha <- intersectn(p1, p1, autoscale=TRUE)
  expect_true(cha$ch$vol > 0)
  expect_equal(ch$ch$vol, cha$ch$vol)
})

test_that("intersectn doesn't crash on some input", {
  ## This is an example causes a crash if flag SCALE_GEOMETRIC (4) is
  ## given to lpSolve::lp in feasible.point()
  load(file.path(system.file(package="geometry"), "extdata", "overlap260-5034.RData"))
  ch <- intersectn(p1, p2)
  cha <- intersectn(p1, p2, autoscale=TRUE)
  expect_equal(ch$ch$vol, cha$ch$vol)
})

test_that("intersectn doesn't crash on input that causes a crash with scale=7 on some processors", {
  ## This is an example causes a crash on some processors if flag SCALE_CURTISREID (7) is
  ## given to lpSolve::lp in feasible.point()
  load(file.path(system.file(package="geometry"), "extdata", "save-overlap32-176.RData"))
  intersectn(p1, p2, tol=1E-3)

  load(file.path(system.file(package="geometry"), "extdata", "save-overlap68-557.RData"))
  intersectn(p1, p2, tol=1E-3)
})

test_that("intersectn doesn't crash on input that causes a crash with EQUILIBRIATE=1 on some processors", {
  ## This is an example causes a crash on some processors if flag
  ## EQUILIBRIATE is given to lpSolve::lp in feasible.point()
  load(file.path(system.file(package="geometry"), "extdata", "save-overlap149-9428.RData"))
  intersectn(p1, p2, tol=1E-3)
})

"matsort" <-
function (...) {
    x = cbind(...)
    if(!is.numeric(x))
        stop("Input should by numeric.")
    res = array(dim = c(nrow(x), 0))
    if (!is.matrix(drop(x)))
        return(x)
    else if (ncol(x) > 30)
        return(t(apply(x, 1, sort)))
    else while (is.matrix(drop(x))) {
        imc = max.col(x)
        x = t(x)
        imx = nrow(x) * (1:ncol(x) - 1) + imc
        xmax = x[imx]
        x = t(matrix(x[-imx], ncol = ncol(x)))
        res = cbind(res, xmax)
    }
    res = cbind(res, x)
    colnames(res) = NULL
    rownames(res) = NULL
    return(res)
}

##' Signed distance from points \code{p} to boundary of circle to
##' allow easy definition of regions in \code{\link{distmesh2d}}.
##'
##' @title Circle distance function
##' @param p A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
##' representing a point in the plane.
##' @param radius radius of circle
##' @param ... additional arguments (not used)
##' @return A vector of length \code{nrow(p)} containing the signed
##' distances to the circle
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @seealso \code{\link{distmesh2d}}, \code{\link{mesh.drectangle}},
##' \code{\link{mesh.diff}}, \code{\link{mesh.intersect}},
##' \code{\link{mesh.union}}
##' @references \url{http://persson.berkeley.edu/distmesh/}
##' 
##' \cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
##' Review, Volume 46 (2), pp. 329-345, June 2004}
##' @keywords arith math
##' @examples
##' 
##' example(distmesh2d)
##' @export
mesh.dcircle <- function (p, radius = 1, ...) {
  if (!is.matrix(p))
    p <- t(as.matrix(p))
  return(sqrt((p^2) %*% c(1, 1)) - radius)
}

##' \code{tetramesh(T, X, col)} uses the \link[rgl]{rgl} package to
##' display the tetrahedrons defined in the m-by-4 matrix T as mesh.
##' Each row of \code{T} specifies a tetrahedron by giving the 4
##' indices of its points in \code{X}.
##' 
##' @title Render tetrahedron mesh (3D)
##' @param T T is a \code{m}-by-3 matrix in trimesh and \code{m}-by-4 in
##' tetramesh. A row of \code{T} contains indices into \code{X} of the vertices
##' of a triangle/tetrahedron. \code{T} is usually the output of delaunayn.
##' @param X X is an n-by-2/n-by-3 matrix. The rows of X represent \code{n}
##' points in 2D/3D space.
##' @param col The tetrahedron colour. See rgl documentation for details.
##' @param clear Should the current rendering device be cleared?
##' @param \dots Parameters to the rendering device. See the \link[rgl]{rgl}
##' package.
##' @author Raoul Grasman
##' @seealso \code{\link{trimesh}}, \code{\link[rgl]{rgl}}, \code{\link{delaunayn}},
##' \code{\link{convhulln}}, \code{\link{surf.tri}}
##' @keywords hplot
##' @examples
##' \dontrun{
##' # example delaunayn
##' d = c(-1,1)
##' pc = as.matrix(rbind(expand.grid(d,d,d),0))
##' tc = delaunayn(pc)
##' 
##' # example tetramesh
##' clr = rep(1,3) %o% (1:nrow(tc)+1)
##' rgl::view3d(60,fov=20)
##' rgl::light3d(270,60)
##' tetramesh(tc,pc,alpha=0.7,col=clr)
##' }
##' @export
tetramesh <- function (T, X, col = grDevices::heat.colors(nrow(T)), clear = TRUE, ...) {
  if(requireNamespace("rgl") == FALSE)
    stop("the rgl package is required for tetramesh")
  if (!is.numeric(T) | !is.numeric(T))
    stop("`T' and `X' should both be numeric.")
  if (ncol(T) != 4)
    stop("Expect first arg `T' to have 4 columns.")
  if (ncol(X) != 3)
    stop("Expect second arg `X' to have 3 columns.")
  t = t(rbind(T[, -1], T[, -2], T[, -3], T[, -4]))
  if (clear)
    rgl::clear3d()
  rgl::triangles3d(X[t, 1], X[t, 2], X[t, 3], col = col, ...)
}

##' Signed distance from points \code{p} to boundary of rectangle to
##' allow easy definition of regions in \code{\link{distmesh2d}}.
##' 
##' @title Rectangle distance function
##' @param p A matrix with 2 columns, each row representing a point in
##' the plane.
##' @param x1 lower left corner of rectangle
##' @param y1 lower left corner of rectangle
##' @param x2 upper right corner of rectangle
##' @param y2 upper right corner of rectangle
##' @param \dots additional arguments (not used)
##' @return a vector of length \code{nrow(p)} containing the signed distances
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @seealso \code{\link{distmesh2d}}, \code{\link{mesh.drectangle}},
##' \code{\link{mesh.diff}}, \code{\link{mesh.intersect}},
##' \code{\link{mesh.union}}
##' @references \url{http://persson.berkeley.edu/distmesh/}
##' 
##' \cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
##' Review, Volume 46 (2), pp. 329-345, June 2004}
##' @keywords arith math
##' @examples
##' example(distmesh2d)
##' @export
mesh.drectangle <- function (p, x1 = -1/2, y1 = -1/2, x2 = 1/2, y2 = 1/2, ...)  {
  if (!is.matrix(p)) 
    p = t(as.matrix(p))
  d1 = y1 - p[, 2]
  d2 = -y2 + p[, 2]
  d3 = x1 - p[, 1]
  d4 = -x2 + p[, 1]
  d5 = sqrt(d1^2 + d3^2)
  d6 = sqrt(d1^2 + d4^2)
  d7 = sqrt(d2^2 + d3^2)
  d8 = sqrt(d2^2 + d4^2)
  matmin = function(...) apply(cbind(...), 1, min)
  d = -matmin(matmin(matmin(-d1, -d2), -d3), -d4)
  ix = d1 > 0 & d3 > 0
  d[ix] = d5[ix]
  ix = d1 > 0 & d4 > 0
  d[ix] = d6[ix]
  ix = d2 > 0 & d3 > 0
  d[ix] = d7[ix]
  ix = d2 > 0 & d4 > 0
  d[ix] = d8[ix]
  return(d)
}

##' A simple mesh generator for non-convex regions in n-D space
##'
##' An unstructured simplex requires a choice of mesh points (vertex nodes) and
##' a triangulation.  This is a simple and short algorithm that improves the
##' quality of a mesh by relocating the mesh points according to a relaxation
##' scheme of forces in a truss structure. The topology of the truss is reset
##' using Delaunay triangulation. A (sufficiently smooth) user supplied signed
##' distance function (\code{fd}) indicates if a given node is inside or
##' outside the region. Points outside the region are projected back to the
##' boundary.
##'
##' This is an implementation of original Matlab software of Per-Olof Persson.
##'
##' Excerpt (modified) from the reference below:
##'
##' \sQuote{The algorithm is based on a mechanical analogy between a triangular
##' mesh and a n-D truss structure. In the physical model, the edges of the
##' Delaunay triangles of a set of points correspond to bars of a truss. Each
##' bar has a force-displacement relationship \eqn{f(\ell, \ell_{0})}{F(L,L0)}
##' depending on its current length \eqn{\ell}{L} and its unextended length
##' \eqn{\ell_{0}}{L0}.}
##'
##' \sQuote{External forces on the structure come at the boundaries, on which
##' external forces have normal orientations. These external forces are just
##' large enough to prevent nodes from moving outside the boundary. The
##' position of the nodes are the unknowns, and are found by solving for a
##' static force equilibrium. The hope is that (when \code{fh = function(p)
##' return(rep(1,nrow(p)))}), the lengths of all the bars at equilibrium will
##' be nearly equal, giving a well-shaped triangular mesh.}
##'
##' See the references below for all details. Also, see the comments in the
##' source file of \code{distmesh2d}.
##'
##' @param fdist Vectorized signed distance function, for example
##' \code{\link{mesh.dsphere}}, accepting an \code{m}-by-\code{n}
##' matrix, where \code{m} is arbitrary, as the first argument.
##' @param fh Vectorized function, for example \code{\link{mesh.hunif}},
##' that returns desired edge length as a function of position.
##' Accepts an \code{m}-by-\code{n} matrix, where \code{n} is
##' arbitrary, as its first argument.
##' @param h Initial distance between mesh nodes.
##' @param box \code{2}-by-\code{n} matrix that specifies the bounding box.
##' (See \link{distmesh2d} for an example.)
##' @param pfix \code{nfix}-by-2 matrix with fixed node positions.
##' @param \dots parameters that are passed to \code{fdist} and \code{fh}
##' @param ptol Algorithm stops when all node movements are smaller than
##' \code{dptol}
##' @param ttol Controls how far the points can move (relatively) before a
##' retriangulation with \code{\link{delaunayn}}.
##' @param deltat Size of the time step in Euler's method.
##' @param geps Tolerance in the geometry evaluations.
##' @param deps Stepsize \eqn{\Delta x} in numerical derivative computation for
##' distance function.
##' @return \code{m}-by-\code{n} matrix with node positions.
##' @section Wishlist : \itemize{ \item Implement in C/Fortran \item Translate
##' other functions of the Matlab package }
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @seealso \code{\link{distmesh2d}}, \code{\link[interp]{tri.mesh}},
##' \code{\link{delaunayn}}, \code{\link{mesh.dsphere}},
##' \code{\link{mesh.hunif}},\cr \code{\link{mesh.diff}},
##' \code{\link{mesh.union}}, \code{\link{mesh.intersect}}
##' @references \url{http://persson.berkeley.edu/distmesh/}
##'
##' \cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
##' Review, Volume 46 (2), pp. 329-345, June 2004}
##' @keywords math optimize dplot graphs
##' @examples
##'
##' \dontrun{
##' # examples distmeshnd
##' require(rgl)
##'
##' fd = function(p, ...) sqrt((p^2)%*%c(1,1,1)) - 1
##'      # also predefined as `mesh.dsphere'
##' fh = function(p,...)  rep(1,nrow(p))
##'      # also predefined as `mesh.hunif'
##' bbox = matrix(c(-1,1),2,3)
##' p = distmeshnd(fd,fh,0.2,bbox, maxiter=100)
##'     # this may take a while:
##'     # press Esc to get result of current iteration
##' }
##'
##' @export
"distmeshnd"  <-
function (fdist, fh, h, box, pfix = array(dim = c(0, ncol(box))),
    ..., ptol = 0.001, ttol = 0.1, deltat = 0.1, geps = 0.1 *
        h, deps = sqrt(.Machine$double.eps) * h)
{
# %DISTMESHND N-D Mesh Generator using Distance Functions.
    dim = ncol(as.matrix(box))
    L0mult = 1 + 0.4/2^(dim - 1)
    rownorm2 = function(x) drop(sqrt((x^2) %*% rep(1, ncol(x))))

    # %1. Create initial distribution in bounding box
    if (dim == 1) {
        p = seq(box[1], box[2], by = h)
    }
    else {
        cbox = lapply(1:dim, function(ii) seq(box[1, ii], box[2,
            ii], by = h))
        p = do.call("expand.grid", cbox)
        p = as.matrix(p)
    }

    # %2. Remove points outside the region, apply the rejection method
    p = p[fdist(p, ...) < geps, ]
    r0 = fh(p, ...)
    p = rbind(pfix, p[stats::runif(nrow(p)) < min(r0)^dim/r0^dim, ])
    N = nrow(p)
    if (N <= dim + 1)
        stop("Not enough starting points inside boundary (is h0 too large?).")
    on.exit(return(invisible(p)))

    message("Press esc if the mesh seems fine but the algorithm hasn't converged.")
    utils::flush.console()
    count = 0

    p0 = 1/.Machine$double.eps

    # mimick Matlab call ``localpairs=nchoosek(1:dim+1,2)'':
    localpairs = as.matrix(expand.grid(1:(dim + 1), 1:(dim + 1)))
    localpairs = localpairs[lower.tri(matrix(TRUE, dim + 1, dim + 1)), 2:1]

    while (TRUE) {
        if (max(rownorm2(p - p0)) > ttol * h) {
            # %3. Retriangulation by Delaunay:

            p0 = p
            t = delaunayn(p)
            pmid = matrix(0, nrow(t), dim)
            for (ii in 1:(dim + 1)) pmid = pmid + p[t[, ii],
                ]/(dim + 1)
            t = t[fdist(pmid, ...) < (-geps), ]
            pair = array(dim = c(0, 2))
            for (ii in 1:nrow(localpairs)) {
                pair = rbind(pair, t[, localpairs[ii, ]])
            }

            # %4. Describe each edge by a unique pair of nodes
            pair = Unique(pair, TRUE); # base-function `unique' is way too slow
            if (dim == 2) {
                trimesh(t, p[, 1:2])
            }
            else if (dim == 3) {
                if (count%%5 == 0) {
                  tetramesh(t, p)
                }
            }
            else {
                message("Retriangulation #", 15)
                utils::flush.console()
            }
            count = count + 1
        }
        bars = p[pair[, 1], ] - p[pair[, 2], ]
        L = rownorm2(bars)
        L0 = fh((p[pair[, 1], ] + p[pair[, 2], ])/2, ...)
        L0 = L0 * L0mult * (sum(L^dim)/sum(L0^dim))^(1/dim)
        F = L0 - L
        F[F < 0] = 0
        Fbar = cbind(bars, -bars) * matrix(F/L, nrow = nrow(bars),
            ncol = 2 * dim)
        ii = pair[, t(matrix(1:2, 2, dim))]
        jj = rep(1, nrow(pair)) %o% c(1:dim, 1:dim)
        s = c(Fbar)
        ns = length(s)
        dp = matrix(0, N, dim)
        dp[1:(dim * N)] = rowsum(s, ii[1:ns] + ns * (jj[1:ns] -
            1))
        if (nrow(pfix) > 0)
            dp[1:nrow(pfix), ] = 0
        p = p + deltat * dp
        d = fdist(p, ...)
        ix = d > 0
        gradd = matrix(0, sum(ix), dim)
        for (ii in 1:dim) {
            a = rep(0, dim)
            a[ii] = deps
            d1x = fdist(p[ix, ] + rep(1, sum(ix)) %o% a, ...)
            gradd[, ii] = (d1x - d[ix])/deps
        }
        p[ix, ] = p[ix, ] - (d[ix] %o% rep(1, dim)) * gradd
        maxdp = max(deltat * rownorm2(dp[d < (-geps), ]))
        if (maxdp < ptol * h)
            break
    }
}

##' Signed distance from points \code{p} to boundary of sphere to
##' allow easy definition of regions in \code{\link{distmeshnd}}.
##'
##' @title Sphere distance function
##' @param p A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
##' representing a point in the plane.
##' @param radius radius of sphere
##' @param ... additional arguments (not used)
##' @return A vector of length \code{nrow(p)} containing the signed
##' distances to the sphere
##' @author Raoul Grasman; translated from original Matlab sources of Per-Olof
##' Persson.
##' @seealso \code{\link{distmeshnd}}
##' @references \url{http://persson.berkeley.edu/distmesh/}
##' 
##' \cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
##' Review, Volume 46 (2), pp. 329-345, June 2004}
##' @keywords arith math
##' @examples
##' 
##' example(distmeshnd)
##' @export
"mesh.dsphere" <-
function (p, radius = 1, ...) 
{
    if (!is.matrix(p)) 
        p = t(as.matrix(p))
    sqrt((p^2) %*% rep(1, ncol(p))) - radius
}

%\VignetteIndexEntry{Qhull examples}
\documentclass{article}
\usepackage{Sweave}
\SweaveOpts{echo=TRUE}
\usepackage{hyperref}
\usepackage[british]{babel}

\title{Qhull examples}
\author{David C. Sterratt}

\begin{document}

\maketitle

This document presents examples of the \texttt{geometry} package
functions which implement functions using the
\href{http://www.qhull.org}{Qhull library}.

\section{Convex hulls in 2D}
\label{qhull-eg:sec:convex-hull-2d}

\subsection{Calling \texttt{convhulln} with one argument}
\label{qhull-eg:sec:call-convh-with}

With one argument, convhulln returns the indices of the points of the
convex hull.
<<>>=
library(geometry)
ps <-matrix(rnorm(30), , 2)
ch <- convhulln(ps)
head(ch)
@ 

\subsection{Calling \texttt{convhulln} with \texttt{options}}
\label{qhull-eg:sec:call-convh-with}

We can supply Qhull options to \texttt{convhulln}; in this case it
returns an object of class \texttt{convhulln} which is also a list.
For example \texttt{FA} returns the generalised \texttt{area} and

\texttt{vol}ume. Confusingly in 2D the generalised area is the length
of the perimeter, and the generalised volume is the area.
<<>>=
ps <-matrix(rnorm(30), , 2)
ch <- convhulln(ps, options="FA")
print(ch$area)
print(ch$vol)
@ 

A \texttt{convhulln} object can also be plotted. 
<<fig=TRUE>>=
plot(ch)
@ 

We can also find the normals to the ``facets'' of the convex hull:
<<>>=
ch <- convhulln(ps, options="n")
head(ch$normals)
@ 
Here the first two columns and the $x$ and $y$ direction of the normal,
and the third column defines the position at which the face intersects
that normal.

\subsection{Testing if points are inside a convex hull with \texttt{inhulln}}
\label{qhull-eg:sec:testing-if-points}

The function \texttt{inhulln} can be used to test if points are inside
a convex hull. Here the function \texttt{rbox} is a handy way to
create points at random locations.
<<fig=TRUE>>=
tp <- rbox(n=200, D=2, B=4)
in_ch <- inhulln(ch, tp)
plot(tp[!in_ch,], col="gray")
points(tp[in_ch,], col="red")
plot(ch, add=TRUE)
@ 

\section{Delaunay triangulation in 2D}
\label{qhull-eg:sec:dela-triang-2d}

\subsection{Calling \texttt{delaunayn} with one argument}
\label{qhull-eg:sec:call-delaunayn-with}

With one argument, a set of points, \texttt{delaunayn} returns the
indices of the points at each vertex of each triangle in the triangulation.
<<fig=TRUE>>=
ps <- rbox(n=10, D=2)
dt <- delaunayn(ps)
head(dt)
trimesh(dt, ps)
points(ps)
@ 

\subsection{Calling \texttt{delaunayn} with \texttt{options}}
\label{qhull-eg:sec:call-dela-with}

We can supply Qhull options to \texttt{delaunayn}; in this case it
returns an object of class \texttt{delaunayn} which is also a list.
For example \texttt{Fa} returns the generalised \texttt{area} of each
triangle. In 2D the generalised area is the actual area; in 3D it
would be the volume.
<<>>=
dt2 <- delaunayn(ps, options="Fa")
print(dt2$areas)
dt2 <- delaunayn(ps, options="Fn")
print(dt2$neighbours)
@

\end{document}


%  LocalWords:  Qhull convhulln ps rnorm ume inhulln rbox tp gray dt
%  LocalWords:  delaunayn trimesh Fn

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convhulln.R
\name{to.mesh3d}
\alias{to.mesh3d}
\title{Convert convhulln object to RGL mesh}
\usage{
to.mesh3d(x, ...)
}
\arguments{
\item{x}{\code{\link{convhulln}} object}

\item{...}{Arguments to \code{\link[rgl]{qmesh3d}} or
\code{\link[rgl]{tmesh3d}}}
}
\value{
\code{\link[rgl]{mesh3d}} object, which can be displayed
  in RGL with \code{\link[rgl]{dot3d}}, \code{\link[rgl]{wire3d}}
  or \code{\link[rgl]{shade3d}}
}
\description{
Convert convhulln object to RGL mesh
}
\seealso{
\code{\link[rgl]{as.mesh3d}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.hunif.R
\name{mesh.hunif}
\alias{mesh.hunif}
\title{Uniform desired edge length}
\usage{
mesh.hunif(p, ...)
}
\arguments{
\item{p}{A \code{n}-by-\code{m} matrix, each row representing a
point in an \code{m}-dimensional space.}

\item{...}{additional arguments (not used)}
}
\value{
Vector of ones of length \code{n}.
}
\description{
Uniform desired edge length function of position to allow easy
definition of regions when passed as the \code{fh} argument of
\code{\link{distmesh2d}} or \code{\link{distmeshnd}}.
}
\seealso{
\code{\link{distmesh2d}} and \code{\link{distmeshnd}}.
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsearch.R
\name{tsearch}
\alias{tsearch}
\title{Search for the enclosing Delaunay convex hull}
\usage{
tsearch(x, y, t, xi, yi, bary = FALSE, method = "quadtree")
}
\arguments{
\item{x}{X-coordinates of triangulation points}

\item{y}{Y-coordinates of triangulation points}

\item{t}{Triangulation, e.g. produced by \code{t <-
delaunayn(cbind(x, y))}}

\item{xi}{X-coordinates of points to test}

\item{yi}{Y-coordinates of points to test}

\item{bary}{If \code{TRUE} return barycentric coordinates as well
as index of triangle.}

\item{method}{One of \code{"quadtree"} or \code{"orig"}. The
Quadtree algorithm is much faster and new from version
0.4.0. The \code{orig} option uses the tsearch algorithm adapted
from Octave code. Its use is deprecated and it may be removed
from a future version of the package.}
}
\value{
If \code{bary} is \code{FALSE}, the index in \code{t} containing the points 
\code{(xi, yi)}.  For points outside the convex hull the index is \code{NA}. 
If \code{bary} is \code{TRUE}, a list containing: 
  \describe{
   \item{list("idx")}{the index in \code{t} containing the points \code{(xi, yi)}}
   \item{list("p")}{a 3-column matrix containing the barycentric coordinates with 
   respect to the enclosing triangle of each point \code{(xi, yi)}.}
  }
}
\description{
For \code{t <- delaunay(cbind(x, y))}, where \code{(x, y)} is a 2D set of
points, \code{tsearch(x, y, t, xi, yi)} finds the index in \code{t}
containing the points \code{(xi, yi)}.  For points outside the convex hull
the index is \code{NA}.
}
\note{
The original Octave function is Copyright (C) 2007-2012
  David Bateman
}
\seealso{
\code{\link{tsearchn}}, \code{\link{delaunayn}}
}
\author{
Jean-Romain Roussel (Quadtree algorithm), David Sterratt (Octave-based implementation)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dotprod.R
\name{dot}
\alias{dot}
\title{Compute the dot product of two vectors}
\usage{
dot(x, y, d = NULL)
}
\arguments{
\item{x}{Matrix of vectors}

\item{y}{Matrix of vectors}

\item{d}{Dimension along which to calculate the dot product}
}
\value{
Vector with length of \code{d}th dimension
}
\description{
If \code{x} and \code{y} are matrices, calculate the dot-product
along the first non-singleton dimension.  If the optional argument
\code{d} is given, calculate the dot-product along this
dimension.
}
\author{
David Sterratt
}
\keyword{arith}
\keyword{array}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convhulln.R
\name{convhulln}
\alias{convhulln}
\title{Compute smallest convex hull that encloses a set of points}
\usage{
convhulln(
  p,
  options = "Tv",
  output.options = NULL,
  return.non.triangulated.facets = FALSE
)
}
\arguments{
\item{p}{An \eqn{M}-by-\eqn{N} matrix. The rows of \code{p}
represent \eqn{M} points in \eqn{N}-dimensional space.}

\item{options}{String containing extra options for the underlying
Qhull command; see details below and Qhull documentation at
\url{../doc/qhull/html/qconvex.html#synopsis}.}

\item{output.options}{String containing Qhull options to generate
extra output. Currently \code{n} (normals) and \code{FA}
(generalised areas and volumes) are supported; see
\sQuote{Value} for details. If \code{output.options} is
\code{TRUE}, select all supported options.}

\item{return.non.triangulated.facets}{logical defining whether the
output facets should be triangulated; \code{FALSE} by default.}
}
\value{
By default (\code{return.non.triangulated.facets} is
  \code{FALSE}), return an \eqn{M}-by-\eqn{N} matrix in which each
  row contains the indices of the points in \code{p} forming an
  \eqn{N-1}-dimensional facet. e.g In 3 dimensions, there are 3
  indices in each row describing the vertices of 2-dimensional
  triangles.

  If \code{return.non.triangulated.facets} is \code{TRUE} then the
  number of columns equals the maximum number of vertices in a
  facet, and each row defines a polygon corresponding to a facet
  of the convex hull with its vertices followed by \code{NA}s
  until the end of the row.

  If the \code{output.options} or \code{options} argument contains
  \code{FA} or \code{n}, return a list with class \code{convhulln}
  comprising the named elements:
  \describe{
    \item{\code{p}}{The points passed to \code{convnhulln}}
    \item{\code{hull}}{The convex hull, represented as a matrix indexing \code{p}, as
      described above}
    \item{\code{area}}{If \code{FA} is specified, the generalised area of
      the hull. This is the surface area of a 3D hull or the length of
      the perimeter of a 2D hull.
      See \url{../doc/qhull/html/qh-optf.html#FA}.}
    \item{\code{vol}}{If \code{FA} is specified, the generalised volume of
       the hull. This is volume of a 3D hull or the area of a 2D hull.
       See \url{../doc/qhull/html/qh-optf.html#FA}. }
    \item{\code{normals}}{If \code{n} is specified, this is a matrix
    hyperplane normals with offsets. See \url{../doc/qhull/html/qh-opto.html#n}.}
  }
}
\description{
Returns information about the smallest convex complex of a set of
input points in \eqn{N}-dimensional space (the convex hull of the
points). By default, indices to points forming the facets of the
hull are returned; optionally normals to the facets and the
generalised surface area and volume can be returned. This function
interfaces the \href{http://www.qhull.org}{Qhull} library.
}
\note{
This function was originally a port of the
  \href{https://octave.org/}{Octave} convhulln function written
  by Kai Habel.

See further notes in \code{\link{delaunayn}}.
}
\examples{
## Points in a sphere
ps <- matrix(rnorm(3000), ncol=3)
ps <- sqrt(3)*ps/drop(sqrt((ps^2) \%*\% rep(1, 3)))
ts.surf <- t(convhulln(ps))  # see the qhull documentations for the options
\dontrun{
rgl::triangles3d(ps[ts.surf,1],ps[ts.surf,2],ps[ts.surf,3],col="blue",alpha=.2)
for(i in 1:(8*360)) rgl::view3d(i/8)
}

## Square
pq <- rbox(0, C=0.5, D=2)
# Return indices only
convhulln(pq)
# Return convhulln object with normals, generalised area and volume
ch <- convhulln(pq, output.options=TRUE)
plot(ch)

## Cube
pc <- rbox(0, C=0.5, D=3)
# Return indices of triangles on surface
convhulln(pc)
# Return indices of squares on surface
convhulln(pc, return.non.triangulated.facets=TRUE)
}
\references{
\cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
  \dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM
  Trans. on Mathematical Software,} Dec 1996.}

\url{http://www.qhull.org}
}
\seealso{
\code{\link{intersectn}}, \code{\link{delaunayn}},
  \code{\link{surf.tri}}, \code{\link[interp]{convex.hull}}
}
\author{
Raoul Grasman, Robert B. Gramacy, Pavlo Mozharovskyi and
  David Sterratt \email{david.c.sterratt@ed.ac.uk}
}
\keyword{dplot}
\keyword{graphs}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.drectangle.R
\name{mesh.drectangle}
\alias{mesh.drectangle}
\title{Rectangle distance function}
\usage{
mesh.drectangle(p, x1 = -1/2, y1 = -1/2, x2 = 1/2, y2 = 1/2, ...)
}
\arguments{
\item{p}{A matrix with 2 columns, each row representing a point in
the plane.}

\item{x1}{lower left corner of rectangle}

\item{y1}{lower left corner of rectangle}

\item{x2}{upper right corner of rectangle}

\item{y2}{upper right corner of rectangle}

\item{\dots}{additional arguments (not used)}
}
\value{
a vector of length \code{nrow(p)} containing the signed distances
}
\description{
Signed distance from points \code{p} to boundary of rectangle to
allow easy definition of regions in \code{\link{distmesh2d}}.
}
\examples{
example(distmesh2d)
}
\references{
\url{http://persson.berkeley.edu/distmesh/}

\cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
Review, Volume 46 (2), pp. 329-345, June 2004}
}
\seealso{
\code{\link{distmesh2d}}, \code{\link{mesh.drectangle}},
\code{\link{mesh.diff}}, \code{\link{mesh.intersect}},
\code{\link{mesh.union}}
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}
\keyword{arith}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/entry.value.R
\name{entry.value}
\alias{entry.value}
\alias{entry.value<-}
\title{Retrieve or set a list of array element values}
\usage{
entry.value(a, idx)
}
\arguments{
\item{a}{An array.}

\item{idx}{Numerical matrix with the same number of columns as the number
of dimensions of \code{a}.  Each row indices a cell in \code{a} of which
the value is to be retrieved or set.}

\item{value}{An array of length \code{nrow(idx)}.}
}
\value{
\code{entry.value(a,idx)} returns a vector of values at the
indicated cells.  \code{entry.value(a,idx) <- val} changes the indicated
cells of \code{a} to \code{val}.
}
\description{
\code{entry.value} retrieves or sets the values in an array \code{a} at the
positions indicated by the rows of a matrix \code{idx}.
}
\examples{

a = array(1:(4^4),c(4,4,4,4))
entry.value(a,cbind(1:4,1:4,1:4,1:4))
entry.value(a,cbind(1:4,1:4,1:4,1:4)) <- 0

entry.value(a, as.matrix(expand.grid(1:4,1:4,1:4,1:4)))
     # same as `c(a[1:4,1:4,1:4,1:4])' which is same as `c(a)'

}
\author{
Raoul Grasman
}
\keyword{arith}
\keyword{array}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extprod3d.R
\name{extprod3d}
\alias{extprod3d}
\title{Compute external- or `cross'- product of 3D vectors.}
\usage{
extprod3d(x, y, drop = TRUE)
}
\arguments{
\item{x}{\code{n}-by-3 matrix. Each row is one \bold{x}-vector}

\item{y}{\code{n}-by-3 matrix. Each row is one \bold{y}-vector}

\item{drop}{logical. If \code{TRUE} and if the inputs are one row
matrices or vectors, then delete the dimensions of the array
returned.}
}
\value{
If \code{n} is greater than 1 or \code{drop} is
  \code{FALSE}, \code{n}-by-3 matrix; if \code{n} is 1 and
  \code{drop} is \code{TRUE}, a vector of length 3.
}
\description{
Computes the external product \deqn{ }{ (x2 * y3 - x3 * y2, x3 * y1 - x1 *
y3, x1 * y2 - x2 * y1) }\deqn{ \left(x_2 y_3 - x_3 y_2,\; x_3 y_1 - x_1
y_3,\; x_1 y_2 - x_2 y_1 \right) }{ (x2 * y3 - x3 * y2, x3 * y1 - x1 * y3,
x1 * y2 - x2 * y1) }\deqn{ }{ (x2 * y3 - x3 * y2, x3 * y1 - x1 * y3, x1 *
y2 - x2 * y1) } of the 3D vectors in \bold{x} and \bold{y}.
}
\seealso{
\code{\link[base]{drop}}
}
\author{
Raoul Grasman
}
\keyword{arith}
\keyword{array}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersectn.R
\name{feasible.point}
\alias{feasible.point}
\title{Find point in intersection of convex hulls}
\usage{
feasible.point(ch1, ch2, tol = 0)
}
\arguments{
\item{ch1}{First convex hull with normals}

\item{ch2}{Second convex hull with normals}

\item{tol}{The point must be at least this far within the facets
of both convex hulls}
}
\description{
Find point that lies somewhere in interesction of two convex
hulls. If such a point does not exist, return \code{NA}. The
feasible point is found using a linear program similar to the one
suggested at \url{../doc/qhull/html/qhalf.html#notes}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pol2cart.R
\name{pol2cart}
\alias{pol2cart}
\title{Transform polar or cylindrical coordinates to Cartesian coordinates.}
\usage{
pol2cart(theta, r = NULL, z = NULL)
}
\arguments{
\item{theta}{describes the angle relative to the positive x-axis.}

\item{r}{is the distance to the z-axis (0, 0, z).}

\item{z}{(optional) is the z-coordinate}
}
\value{
a matrix \code{C} where each row represents one Cartesian
  coordinate (\code{x}, \code{y} (, \code{z})).
}
\description{
The inputs \code{theta}, \code{r}, (and \code{z}) must be the same shape, or
scalar.  If called with a single matrix argument then each row of \code{P}
represents the polar/(cylindrical) coordinate (\code{theta}, \code{r}
(, \code{z})).
}
\seealso{
\code{\link{cart2pol}}, \code{\link{sph2cart}},
  \code{\link{cart2sph}}
}
\author{
Kai Habel

David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matmax.R
\name{matmax}
\alias{matmax}
\alias{matmin}
\alias{matsort}
\alias{matorder}
\title{Row-wise matrix functions}
\usage{
matmax(...)
}
\arguments{
\item{\dots}{A numeric matrix or a set of numeric vectors (that are
column-wise bind together into a matrix with \code{cbind}).}
}
\value{
\code{matmin} and \code{matmax} return a vector of length
\code{nrow(cbind(...))}.  \code{matsort} returns a matrix of dimension
\code{dim(cbind(...))} with in each row of \code{cbind(...)} sorted.
\code{matsort(x)} is a lot faster than, e.g., \code{t(apply(x,1,sort))},
if \code{x} is tall (i.e., \code{nrow(x)}>>\code{ncol(x)} and
\code{ncol(x)}<30. If \code{ncol(x)}>30 then \code{matsort} simply calls
`\code{t(apply(x,1,sort))}'.  \code{matorder} returns a permutation which
rearranges its first argument into ascending order, breaking ties by
further arguments.
}
\description{
Compute maximum or minimum of each row, or sort each row of a matrix, or a
set of (equal length) vectors.
}
\examples{

example(Unique)

}
\author{
Raoul Grasman
}
\keyword{arith}
\keyword{array}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsearch.R
\name{cart2bary}
\alias{cart2bary}
\title{Conversion of Cartesian to Barycentric coordinates.}
\usage{
cart2bary(X, P)
}
\arguments{
\item{X}{Reference simplex in \eqn{N} dimensions represented by a
\eqn{N+1}-by-\eqn{N} matrix}

\item{P}{\eqn{M}-by-\eqn{N} matrix in which each row is the Cartesian
coordinates of a point.}
}
\value{
\eqn{M}-by-\eqn{N+1} matrix in which each row is the
barycentric coordinates of corresponding row of \code{P}. If the
simplex is degenerate a warning is issued and the function returns
\code{NULL}.
}
\description{
Given the Cartesian coordinates of one or more points, compute
the barycentric coordinates of these points with respect to a
simplex.
}
\details{
Given a reference simplex in \eqn{N} dimensions represented by a
\eqn{N+1}-by-\eqn{N} matrix an arbitrary point \eqn{P} in
Cartesian coordinates, represented by a 1-by-\eqn{N} row vector, can be
written as
\deqn{P = \beta X}
where \eqn{\beta} is an \eqn{N+1} vector of the barycentric coordinates.
A criterion on \eqn{\beta} is that
\deqn{\sum_i\beta_i = 1}
Now partition the simplex into its first \eqn{N} rows \eqn{X_N} and
its \eqn{N+1}th row \eqn{X_{N+1}}. Partition the barycentric
coordinates into the first \eqn{N} columns \eqn{\beta_N} and the
\eqn{N+1}th column \eqn{\beta_{N+1}}. This allows us to write
\deqn{P_{N+1} - X_{N+1} = \beta_N X_N + \beta_{N+1} X_{N+1} - X_{N+1}}
which can be written
\deqn{P_{N+1} - X_{N+1} = \beta_N(X_N - 1_N X_{N+1})}
where \eqn{1_N} is an \eqn{N}-by-1 matrix of ones.  We can then solve
for \eqn{\beta_N}:
\deqn{\beta_N = (P_{N+1} - X_{N+1})(X_N - 1_N X_{N+1})^{-1}}
and compute
\deqn{\beta_{N+1} = 1 - \sum_{i=1}^N\beta_i}
This can be generalised for multiple values of
\eqn{P}, one per row.
}
\note{
Based on the Octave function by David Bateman.
}
\examples{
## Define simplex in 2D (i.e. a triangle)
X <- rbind(c(0, 0),
           c(0, 1),
           c(1, 0))
## Cartesian coordinates of points
P <- rbind(c(0.5, 0.5),
           c(0.1, 0.8))
## Plot triangle and points
trimesh(rbind(1:3), X)
text(X[,1], X[,2], 1:3) # Label vertices
points(P)
cart2bary(X, P)
}
\seealso{
\code{\link{bary2cart}}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distmesh2d.R
\name{distmesh2d}
\alias{distmesh2d}
\title{A simple mesh generator for non-convex regions}
\usage{
distmesh2d(fd, fh, h0, bbox, p = NULL, pfix = array(0, dim = c(0, 2)),
  ..., dptol = 0.001, ttol = 0.1, Fscale = 1.2, deltat = 0.2,
  geps = 0.001 * h0, deps = sqrt(.Machine$double.eps) * h0,
  maxiter = 1000, plot = TRUE)
}
\arguments{
\item{fd}{Vectorized signed distance function, for example
\code{\link{mesh.dcircle}} or \code{\link{mesh.diff}}, accepting
an \code{n}-by-\code{2} matrix, where \code{n} is arbitrary, as
the first argument.}

\item{fh}{Vectorized function, for example
\code{\link{mesh.hunif}}, that returns desired edge length as a
function of position.  Accepts an \code{n}-by-\code{2} matrix,
where \code{n} is arbitrary, as its first argument.}

\item{h0}{Initial distance between mesh nodes. (Ignored of
\code{p} is supplied)}

\item{bbox}{Bounding box \code{cbind(c(xmin,xmax), c(ymin,ymax))}}

\item{p}{An \code{n}-by-\code{2} matrix. The rows of \code{p}
represent locations of starting mesh nodes.}

\item{pfix}{\code{nfix}-by-2 matrix with fixed node positions.}

\item{\dots}{parameters to be passed to \code{fd} and/or \code{fh}}

\item{dptol}{Algorithm stops when all node movements are smaller
than \code{dptol}}

\item{ttol}{Controls how far the points can move (relatively)
before a retriangulation with \code{\link{delaunayn}}.}

\item{Fscale}{\dQuote{Internal pressure} in the edges.}

\item{deltat}{Size of the time step in Euler's method.}

\item{geps}{Tolerance in the geometry evaluations.}

\item{deps}{Stepsize \eqn{\Delta x} in numerical derivative
computation for distance function.}

\item{maxiter}{Maximum iterations.}

\item{plot}{logical.  If \code{TRUE} (default), the mesh is
plotted as it is generated.}
}
\value{
\code{n}-by-\code{2} matrix with node positions.
}
\description{
An unstructured simplex requires a choice of mesh points (vertex nodes) and
a triangulation.  This is a simple and short algorithm that improves the
quality of a mesh by relocating the mesh points according to a relaxation
scheme of forces in a truss structure. The topology of the truss is reset
using Delaunay triangulation. A (sufficiently smooth) user supplied signed
distance function (\code{fd}) indicates if a given node is inside or
outside the region. Points outside the region are projected back to the
boundary.
}
\details{
This is an implementation of original Matlab software of Per-Olof Persson.

Excerpt (modified) from the reference below:

\sQuote{The algorithm is based on a mechanical analogy between a triangular
mesh and a 2D truss structure. In the physical model, the edges of the
Delaunay triangles of a set of points correspond to bars of a truss. Each
bar has a force-displacement relationship \eqn{f(\ell, \ell_{0})}{F(L,L0)}
depending on its current length \eqn{\ell}{L} and its unextended length
\eqn{\ell_{0}}{L0}.}

\sQuote{External forces on the structure come at the boundaries, on which
external forces have normal orientations. These external forces are just
large enough to prevent nodes from moving outside the boundary. The
position of the nodes are the unknowns, and are found by solving for a
static force equilibrium. The hope is that (when \code{fh = function(p)
return(rep(1,nrow(p)))}), the lengths of all the bars at equilibrium will
be nearly equal, giving a well-shaped triangular mesh.}

See the references below for all details. Also, see the comments in the
source file.
}
\section{Wishlist }{
 \itemize{ \item Implement in C/Fortran
  \item Implement an \code{n}D version as provided in the Matlab
  package \item Translate other functions of the Matlab package }
}

\examples{

# examples distmesh2d
fd <- function(p, ...) sqrt((p^2)\%*\%c(1,1)) - 1
     # also predefined as `mesh.dcircle'
fh <- function(p,...)  rep(1,nrow(p))
bbox <- matrix(c(-1,1,-1,1),2,2)
p <- distmesh2d(fd,fh,0.2,bbox, maxiter=100)
    # this may take a while:
    # press Esc to get result of current iteration

# example with non-convex region
fd <- function(p, ...) mesh.diff(p , mesh.drectangle, mesh.dcircle, radius=.3)
     # fd defines difference of square and circle

p <- distmesh2d(fd,fh,0.05,bbox,radius=0.3,maxiter=4)
p <- distmesh2d(fd,fh,0.05,bbox,radius=0.3, maxiter=10)
     # continue on previous mesh
}
\references{
\url{http://persson.berkeley.edu/distmesh/}

\cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
Review, Volume 46 (2), pp. 329-345, June 2004}
}
\seealso{
\code{\link[tripack]{tri.mesh}}, \code{\link{delaunayn}},
  \code{\link{mesh.dcircle}}, \code{\link{mesh.drectangle}},
  \code{\link{mesh.diff}}, \code{\link{mesh.union}},
  \code{\link{mesh.intersect}}
}
\author{
Raoul Grasman
}
\keyword{dplot}
\keyword{graphs}
\keyword{math}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surf.tri.R
\name{surf.tri}
\alias{surf.tri}
\title{Find surface triangles from tetrahedral mesh}
\usage{
surf.tri(p, t)
}
\arguments{
\item{p}{An \code{n}-by-\code{3} matrix. The rows of \code{p} represent
\code{n} points in \code{dim}-dimensional space.}

\item{t}{Matrix with 4 columns, interpreted as output of
\code{\link{delaunayn}}.}
}
\value{
An \code{m}-by-\code{3} index matrix of which each row defines a
triangle. The indices refer to the rows in \code{p}.
}
\description{
Find surface triangles from tetrahedral mesh typically obtained
with \code{\link{delaunayn}}.
}
\details{
\code{surf.tri} and \code{\link{convhulln}} serve a similar purpose in 3D,
but \code{surf.tri} also works for non-convex meshes obtained e.g. with
\code{\link{distmeshnd}}.  It also does not produce currently unavoidable
diagnostic output on the console as \code{convhulln} does at the Rterm
console--i.e., \code{surf.tri} is silent.
}
\note{
\code{surf.tri} was based on Matlab code for mesh of Per-Olof Persson
(\url{http://persson.berkeley.edu/distmesh/}).
}
\examples{

\dontrun{
# more extensive example of surf.tri

# url's of publically available data:
data1.url = "http://neuroimage.usc.edu/USCPhantom/mesh_data.bin"
data2.url = "http://neuroimage.usc.edu/USCPhantom/CT_PCS_trans.bin"

meshdata = R.matlab::readMat(url(data1.url))
elec = R.matlab::readMat(url(data2.url))$eeg.ct2pcs/1000
brain = meshdata$mesh.brain[,c(1,3,2)]
scalp = meshdata$mesh.scalp[,c(1,3,2)]
skull = meshdata$mesh.skull[,c(1,3,2)]
tbr = t(surf.tri(brain, delaunayn(brain)))
tsk = t(surf.tri(skull, delaunayn(skull)))
tsc = t(surf.tri(scalp, delaunayn(scalp)))
rgl::triangles3d(brain[tbr,1], brain[tbr,2], brain[tbr,3],col="gray")
rgl::triangles3d(skull[tsk,1], skull[tsk,2], skull[tsk,3],col="white", alpha=0.3)
rgl::triangles3d(scalp[tsc,1], scalp[tsc,2], scalp[tsc,3],col="#a53900", alpha=0.6)
rgl::view3d(-40,30,.4,zoom=.03)
lx = c(-.025,.025); ly = -c(.02,.02);
rgl::spheres3d(elec[,1],elec[,3],elec[,2],radius=.0025,col='gray')
rgl::spheres3d( lx, ly,.11,radius=.015,col="white")
rgl::spheres3d( lx, ly,.116,radius=.015*.7,col="brown")
rgl::spheres3d( lx, ly,.124,radius=.015*.25,col="black")
}

}
\seealso{
\code{\link[interp]{tri.mesh}}, \code{\link{convhulln}},
\code{\link{surf.tri}}, \code{\link{distmesh2d}}
}
\author{
Raoul Grasman
}
\keyword{dplot}
\keyword{math}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/halfspacen.R
\name{halfspacen}
\alias{halfspacen}
\title{Compute halfspace intersection about a point}
\usage{
halfspacen(p, fp, options = "Tv")
}
\arguments{
\item{p}{An \eqn{M}-by-\eqn{N+1} matrix. Each row of \code{p}
represents a halfspace by a \eqn{N}-dimensional normal to a
hyperplane and the offset of the hyperplane.}

\item{fp}{A \dQuote{feasible} point that is within the space
contained within all the halfspaces.}

\item{options}{String containing extra options, separated by
spaces, for the underlying Qhull command; see Qhull
documentation at \url{../doc/qhull/html/qhalf.html}.}
}
\value{
A \eqn{N}-column matrix containing the intersection
  points of the hyperplanes \url{../doc/qhull/html/qhalf.html}.
}
\description{
Compute halfspace intersection about a point
}
\note{
\code{halfspacen} was introduced in geometry 0.4.0, and is
  still under development. It is worth checking results for
  unexpected behaviour.
}
\examples{
p <- rbox(0, C=0.5)  # Generate points on a unit cube centered around the origin
ch <- convhulln(p, "n") # Generate convex hull, including normals to facets, with "n" option
# Intersections of half planes
# These points should be the same as the orginal points
pn <- halfspacen(ch$normals, c(0, 0, 0)) 

}
\references{
\cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
  \dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM
  Trans. on Mathematical Software,} Dec 1996.}

\url{http://www.qhull.org}
}
\seealso{
\code{\link{convhulln}}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.diff.R
\name{mesh.diff}
\alias{mesh.diff}
\alias{mesh.union}
\alias{mesh.intersect}
\title{Difference, union and intersection operation on  two regions}
\usage{
mesh.diff(p, regionA, regionB, ...)
}
\arguments{
\item{p}{A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
representing a point in the plane.}

\item{regionA}{vectorized function describing region A in the
union / intersection / difference}

\item{regionB}{vectorized function describing region B in the
union / intersection / difference}

\item{...}{additional arguments passed to \code{regionA} and
\code{regionB}}
}
\value{
A vector of length \code{nrow(p)} containing the signed
distances to the boundary of the region.
}
\description{
Compute the signed distances from points \code{p} to a region
defined by the difference, union or intersection of regions
specified by the functions \code{regionA} and \code{regionB}.
\code{regionA} and \code{regionB} must accept a matrix \code{p}
with 2 columns as their first argument, and must return a vector
of length \code{nrow(p)} containing the signed distances of the
supplied points in \code{p} to their respective regions.
}
\seealso{
\code{\link{distmesh2d}}, \code{\link{mesh.dcircle}},
\code{\link{mesh.drectangle}} \code{\link{mesh.dsphere}}
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rbox.R
\name{rbox}
\alias{rbox}
\title{Generate various point distributions}
\usage{
rbox(n = 3000, D = 3, B = 0.5, C = NA)
}
\arguments{
\item{n}{number of random points in hypercube}

\item{D}{number of dimensions of hypercube}

\item{B}{bounding box coordinate - faces will be \code{-B} and \code{B} from origin}

\item{C}{add a unit hypercube to the output - faces will be \code{-C} and \code{C} from origin}
}
\value{
Matrix of points
}
\description{
Default is corners of a hypercube.
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cart2sph.R
\name{cart2sph}
\alias{cart2sph}
\title{Transform Cartesian to spherical coordinates}
\usage{
cart2sph(x, y = NULL, z = NULL)
}
\arguments{
\item{x}{x-coordinates or matrix with three columns}

\item{y}{y-coordinates (optional, if \code{x}) is a matrix}

\item{z}{z-coordinates (optional, if \code{x}) is a matrix}
}
\value{
Matrix with columns:
\item{\code{theta}}{the angle relative to the positive x-axis}
\item{\code{phi}}{the angle relative to the xy-plane}
\item{\code{r}}{the distance to the origin \code{(0, 0, 0)}}
}
\description{
If called with a single matrix argument then each row of \code{c}
represents the Cartesian coordinate (\code{x}, \code{y}, \code{z}).
}
\seealso{
\code{\link{sph2cart}}, \code{\link{cart2pol}},
  \code{\link{pol2cart}}
}
\author{
Kai Habel

David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/delaunayn.R
\name{delaunayn}
\alias{delaunayn}
\title{Delaunay triangulation in N dimensions}
\usage{
delaunayn(p, options = NULL, output.options = NULL, full = FALSE)
}
\arguments{
\item{p}{An \eqn{M}-by-\eqn{N} matrix whose rows represent \eqn{M}
points in \eqn{N}-dimensional space.}

\item{options}{String containing extra control options for the
  underlying Qhull command; see the Qhull documentation
  (\url{../doc/qhull/html/qdelaun.html}) for the available
  options.

  The \code{Qbb} option is always passed to Qhull. The remaining
  default options are \code{Qcc Qc Qt Qz} for \eqn{N<4} and
  \code{Qcc Qc Qt Qx} for \eqn{N>=4}. If neither of the \code{QJ}
  or \code{Qt} options are supplied, the \code{Qt} option is
  passed to Qhull. The \code{Qt} option ensures all Delaunay
  regions are simplical (e.g., triangles in 2D). See
  \url{../doc/qhull/html/qdelaun.html} for more details. Contrary
  to the Qhull documentation, no degenerate (zero area) regions
  are returned with the \code{Qt} option since the R function
  removes them from the triangulation.

  \emph{If \code{options} is specified, the default options are
  overridden.} It is recommended to use \code{output.options} for
  options controlling the outputs.}

\item{output.options}{String containing Qhull options to control
output. Currently \code{Fn} (neighbours) and \code{Fa} (areas)
are supported. Causes an object of  return value for details. If
\code{output.options} is \code{TRUE}, select all supported
options.}

\item{full}{Deprecated and will be removed in a future release.
Adds options \code{Fa} and \code{Fn}.}
}
\value{
If \code{output.options} is \code{NULL} (the default),
  return the Delaunay triangulation as a matrix with \eqn{M} rows
  and \eqn{N+1} columns in which each row contains a set of
  indices to the input points \code{p}. Thus each row describes a
  simplex of dimension \eqn{N}, e.g. a triangle in 2D or a
  tetrahedron in 3D.

  If the \code{output.options} argument is \code{TRUE} or is a
  string containing \code{Fn} or \code{Fa}, return a list with
  class \code{delaunayn} comprising the named elements:
  \describe{
    \item{\code{tri}}{The Delaunay triangulation described above}
    \item{\code{areas}}{If \code{TRUE} or if \code{Fa} is specified, an
      \eqn{M}-dimensional vector containing the generalised area of
      each simplex (e.g. in 2D the areas of triangles; in 3D the volumes
      of tetrahedra). See \url{../doc/qhull/html/qh-optf.html#Fa}.}
    \item{\code{neighbours}}{If \code{TRUE} or if \code{Fn} is specified,
      a list of neighbours of each simplex. Note that a negative number
      corresponds to "facet" (="edge" in 2D or "face" in 3D) that has no
      neighbour, as will be the case for some simplices on the boundary
      of the triangulation.
      See \url{../doc/qhull/html/qh-optf.html#Fn}}
  }
}
\description{
The Delaunay triangulation is a tessellation of the convex hull of
the points such that no \eqn{N}-sphere defined by the \eqn{N}-
triangles contains any other points from the set.
}
\note{
This function interfaces the Qhull library and is a port
  from Octave (\url{https://octave.org/}) to R. Qhull computes
  convex hulls, Delaunay triangulations, halfspace intersections
  about a point, Voronoi diagrams, furthest-site Delaunay
  triangulations, and furthest-site Voronoi diagrams. It runs in
  2D, 3D, 4D, and higher dimensions. It implements the
  Quickhull algorithm for computing the convex hull. Qhull handles
  round-off errors from floating point arithmetic. It computes
  volumes, surface areas, and approximations to the convex
  hull. See the Qhull documentation included in this distribution
  (the doc directory \url{../doc/qhull/index.html}).

Qhull does not support constrained Delaunay triangulations, triangulation
of non-convex surfaces, mesh generation of non-convex objects, or
medium-sized inputs in 9D and higher. A rudimentary algorithm for mesh
generation in non-convex regions using Delaunay triangulation is
implemented in \link{distmesh2d} (currently only 2D).
}
\examples{

# example delaunayn
d <- c(-1,1)
pc <- as.matrix(rbind(expand.grid(d,d,d),0))
tc <- delaunayn(pc)

# example tetramesh
\dontrun{
rgl::view3d(60)
rgl::light3d(120,60)
tetramesh(tc,pc, alpha=0.9)
}

tc1 <- delaunayn(pc, output.options="Fa")
## sum of generalised areas is total volume of cube
sum(tc1$areas)

}
\references{
\cite{Barber, C.B., Dobkin, D.P., and Huhdanpaa, H.T.,
\dQuote{The Quickhull algorithm for convex hulls,} \emph{ACM Trans. on
Mathematical Software,} Dec 1996.}

\url{http://www.qhull.org}
}
\seealso{
\code{\link[interp]{tri.mesh}}, \code{\link{convhulln}},
\code{\link{surf.tri}}, \code{\link{distmesh2d}}
}
\author{
Raoul Grasman and Robert B. Gramacy; based on the
corresponding Octave sources of Kai Habel.
}
\keyword{dplot}
\keyword{graphs}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersectn.R
\name{intersectn}
\alias{intersectn}
\title{Compute convex hull of intersection of two sets of points}
\usage{
intersectn(
  ps1,
  ps2,
  tol = 0,
  return.chs = TRUE,
  options = "Tv",
  fp = NULL,
  autoscale = FALSE
)
}
\arguments{
\item{ps1}{First set of points}

\item{ps2}{Second set of points}

\item{tol}{Tolerance used to determine if a feasible point lies
within the convex hulls of both points and to round off the
points generated by the halfspace intersection, which sometimes
produces points very close together.}

\item{return.chs}{If \code{TRUE} (default) return the convex hulls
of the first and second sets of points, as well as the convex
hull of the intersection.}

\item{options}{Options passed to \code{\link{halfspacen}}. By
default this is \code{Tv}.}

\item{fp}{Coordinates of feasible point, i.e. a point known to lie
in the hulls of \code{ps1} and \code{ps2}. The feasible point is
required for \code{\link{halfspacen}} to find the intersection.
\code{intersectn} tries to find the feasible point automatically
using the linear program in \code{\link{feasible.point}}, but
currently the linear program fails on some examples where there
is an obvious solution. This option overrides the automatic
search for a feasible point}

\item{autoscale}{\emph{Experimental in v0.4.2} Automatically scale
the points to lie in a sensible numeric range. May help to
correct some numerical issues.}
}
\value{
List containing named elements: \code{ch1}, the convex
  hull of the first set of points, with volumes, areas and normals
  (see \code{\link{convhulln}}; \code{ch2}, the convex hull of the
  first set of points, with volumes, areas and normals; \code{ps},
  the intersection points of convex hulls \code{ch1} and
  \code{ch2}; and \code{ch}, the convex hull of the intersection
  points, with volumes, areas and normals.
}
\description{
Compute convex hull of intersection of two sets of points
}
\note{
\code{intersectn} was introduced in geometry 0.4.0, and is
  still under development. It is worth checking results for
  unexpected behaviour.
}
\examples{
# Two overlapping boxes
ps1 <- rbox(0, C=0.5)
ps2 <- rbox(0, C=0.5) + 0.5
out <- intersectn(ps1, ps2)
message("Volume of 1st convex hull: ", out$ch1$vol)
message("Volume of 2nd convex hull: ", out$ch2$vol)
message("Volume of intersection convex hull: ", out$ch$vol)
}
\seealso{
\code{\link{convhulln}}, \code{\link{halfspacen}},
  \code{\link{inhulln}}, \code{\link{feasible.point}}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tetramesh.R
\name{tetramesh}
\alias{tetramesh}
\title{Render tetrahedron mesh (3D)}
\usage{
tetramesh(T, X, col = grDevices::heat.colors(nrow(T)), clear = TRUE, ...)
}
\arguments{
\item{T}{T is a \code{m}-by-3 matrix in trimesh and \code{m}-by-4 in
tetramesh. A row of \code{T} contains indices into \code{X} of the vertices
of a triangle/tetrahedron. \code{T} is usually the output of delaunayn.}

\item{X}{X is an n-by-2/n-by-3 matrix. The rows of X represent \code{n}
points in 2D/3D space.}

\item{col}{The tetrahedron colour. See rgl documentation for details.}

\item{clear}{Should the current rendering device be cleared?}

\item{\dots}{Parameters to the rendering device. See the \link[rgl]{rgl}
package.}
}
\description{
\code{tetramesh(T, X, col)} uses the \link[rgl]{rgl} package to
display the tetrahedrons defined in the m-by-4 matrix T as mesh.
Each row of \code{T} specifies a tetrahedron by giving the 4
indices of its points in \code{X}.
}
\examples{
\dontrun{
# example delaunayn
d = c(-1,1)
pc = as.matrix(rbind(expand.grid(d,d,d),0))
tc = delaunayn(pc)

# example tetramesh
clr = rep(1,3) \%o\% (1:nrow(tc)+1)
rgl::view3d(60,fov=20)
rgl::light3d(270,60)
tetramesh(tc,pc,alpha=0.7,col=clr)
}
}
\seealso{
\code{\link{trimesh}}, \code{\link[rgl]{rgl}}, \code{\link{delaunayn}},
\code{\link{convhulln}}, \code{\link{surf.tri}}
}
\author{
Raoul Grasman
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.dcircle.R
\name{mesh.dcircle}
\alias{mesh.dcircle}
\title{Circle distance function}
\usage{
mesh.dcircle(p, radius = 1, ...)
}
\arguments{
\item{p}{A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
representing a point in the plane.}

\item{radius}{radius of circle}

\item{...}{additional arguments (not used)}
}
\value{
A vector of length \code{nrow(p)} containing the signed
distances to the circle
}
\description{
Signed distance from points \code{p} to boundary of circle to
allow easy definition of regions in \code{\link{distmesh2d}}.
}
\examples{

example(distmesh2d)
}
\references{
\url{http://persson.berkeley.edu/distmesh/}

\cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
Review, Volume 46 (2), pp. 329-345, June 2004}
}
\seealso{
\code{\link{distmesh2d}}, \code{\link{mesh.drectangle}},
\code{\link{mesh.diff}}, \code{\link{mesh.intersect}},
\code{\link{mesh.union}}
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}
\keyword{arith}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sph2cart.R
\name{sph2cart}
\alias{sph2cart}
\title{Transform spherical coordinates to Cartesian coordinates}
\usage{
sph2cart(theta, phi = NULL, r = NULL)
}
\arguments{
\item{theta}{describes the angle relative to the positive x-axis.}

\item{phi}{is the angle relative to the xy-plane.}

\item{r}{is the distance to the origin \code{(0, 0, 0)}.

If only a single return argument is requested then return a matrix
\code{C} where each row represents one Cartesian coordinate
(\code{x}, \code{y}, \code{z}).}
}
\description{
The inputs \code{theta}, \code{phi}, and \code{r} must be the same
shape, or scalar.  If called with a single matrix argument then
each row of \code{S} represents the spherical coordinate
(\code{theta}, \code{phi}, \code{r}).
}
\seealso{
\code{\link{cart2sph}}, \code{\link{pol2cart}}, \code{\link{cart2pol}}
}
\author{
Kai Habel

David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/distmeshnd.R
\name{distmeshnd}
\alias{distmeshnd}
\title{A simple mesh generator for non-convex regions in n-D space}
\usage{
distmeshnd(
  fdist,
  fh,
  h,
  box,
  pfix = array(dim = c(0, ncol(box))),
  ...,
  ptol = 0.001,
  ttol = 0.1,
  deltat = 0.1,
  geps = 0.1 * h,
  deps = sqrt(.Machine$double.eps) * h
)
}
\arguments{
\item{fdist}{Vectorized signed distance function, for example
\code{\link{mesh.dsphere}}, accepting an \code{m}-by-\code{n}
matrix, where \code{m} is arbitrary, as the first argument.}

\item{fh}{Vectorized function, for example \code{\link{mesh.hunif}},
that returns desired edge length as a function of position.
Accepts an \code{m}-by-\code{n} matrix, where \code{n} is
arbitrary, as its first argument.}

\item{h}{Initial distance between mesh nodes.}

\item{box}{\code{2}-by-\code{n} matrix that specifies the bounding box.
(See \link{distmesh2d} for an example.)}

\item{pfix}{\code{nfix}-by-2 matrix with fixed node positions.}

\item{\dots}{parameters that are passed to \code{fdist} and \code{fh}}

\item{ptol}{Algorithm stops when all node movements are smaller than
\code{dptol}}

\item{ttol}{Controls how far the points can move (relatively) before a
retriangulation with \code{\link{delaunayn}}.}

\item{deltat}{Size of the time step in Euler's method.}

\item{geps}{Tolerance in the geometry evaluations.}

\item{deps}{Stepsize \eqn{\Delta x} in numerical derivative computation for
distance function.}
}
\value{
\code{m}-by-\code{n} matrix with node positions.
}
\description{
An unstructured simplex requires a choice of mesh points (vertex nodes) and
a triangulation.  This is a simple and short algorithm that improves the
quality of a mesh by relocating the mesh points according to a relaxation
scheme of forces in a truss structure. The topology of the truss is reset
using Delaunay triangulation. A (sufficiently smooth) user supplied signed
distance function (\code{fd}) indicates if a given node is inside or
outside the region. Points outside the region are projected back to the
boundary.
}
\details{
This is an implementation of original Matlab software of Per-Olof Persson.

Excerpt (modified) from the reference below:

\sQuote{The algorithm is based on a mechanical analogy between a triangular
mesh and a n-D truss structure. In the physical model, the edges of the
Delaunay triangles of a set of points correspond to bars of a truss. Each
bar has a force-displacement relationship \eqn{f(\ell, \ell_{0})}{F(L,L0)}
depending on its current length \eqn{\ell}{L} and its unextended length
\eqn{\ell_{0}}{L0}.}

\sQuote{External forces on the structure come at the boundaries, on which
external forces have normal orientations. These external forces are just
large enough to prevent nodes from moving outside the boundary. The
position of the nodes are the unknowns, and are found by solving for a
static force equilibrium. The hope is that (when \code{fh = function(p)
return(rep(1,nrow(p)))}), the lengths of all the bars at equilibrium will
be nearly equal, giving a well-shaped triangular mesh.}

See the references below for all details. Also, see the comments in the
source file of \code{distmesh2d}.
}
\section{Wishlist }{
 \itemize{ \item Implement in C/Fortran \item Translate
other functions of the Matlab package }
}

\examples{

\dontrun{
# examples distmeshnd
require(rgl)

fd = function(p, ...) sqrt((p^2)\%*\%c(1,1,1)) - 1
     # also predefined as `mesh.dsphere'
fh = function(p,...)  rep(1,nrow(p))
     # also predefined as `mesh.hunif'
bbox = matrix(c(-1,1),2,3)
p = distmeshnd(fd,fh,0.2,bbox, maxiter=100)
    # this may take a while:
    # press Esc to get result of current iteration
}

}
\references{
\url{http://persson.berkeley.edu/distmesh/}

\cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
Review, Volume 46 (2), pp. 329-345, June 2004}
}
\seealso{
\code{\link{distmesh2d}}, \code{\link[interp]{tri.mesh}},
\code{\link{delaunayn}}, \code{\link{mesh.dsphere}},
\code{\link{mesh.hunif}},\cr \code{\link{mesh.diff}},
\code{\link{mesh.union}}, \code{\link{mesh.intersect}}
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}
\keyword{dplot}
\keyword{graphs}
\keyword{math}
\keyword{optimize}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh.dsphere.R
\name{mesh.dsphere}
\alias{mesh.dsphere}
\title{Sphere distance function}
\usage{
mesh.dsphere(p, radius = 1, ...)
}
\arguments{
\item{p}{A matrix with 2 columns (3 in \code{mesh.dsphere}), each row
representing a point in the plane.}

\item{radius}{radius of sphere}

\item{...}{additional arguments (not used)}
}
\value{
A vector of length \code{nrow(p)} containing the signed
distances to the sphere
}
\description{
Signed distance from points \code{p} to boundary of sphere to
allow easy definition of regions in \code{\link{distmeshnd}}.
}
\examples{

example(distmeshnd)
}
\references{
\url{http://persson.berkeley.edu/distmesh/}

\cite{P.-O. Persson, G. Strang, A Simple Mesh Generator in MATLAB. SIAM
Review, Volume 46 (2), pp. 329-345, June 2004}
}
\seealso{
\code{\link{distmeshnd}}
}
\author{
Raoul Grasman; translated from original Matlab sources of Per-Olof
Persson.
}
\keyword{arith}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsearch.R
\name{tsearchn}
\alias{tsearchn}
\title{Search for the enclosing Delaunay convex hull}
\usage{
tsearchn(x, t, xi, ...)
}
\arguments{
\item{x}{An \eqn{N}-column matrix, in which each row represents a
point in \eqn{N}-dimensional space.}

\item{t}{A matrix with \eqn{N+1} columns. A row of \code{t}
contains indices into \code{x} of the vertices of an
\eqn{N}-dimensional simplex. \code{t} is usually the output of
delaunayn.}

\item{xi}{An \eqn{M}-by-\eqn{N} matrix. The rows of \code{xi}
represent \eqn{M} points in \eqn{N}-dimensional space whose
positions in the mesh are being sought.}

\item{...}{Additional arguments}
}
\value{
A list containing:
  \describe{
    \item{\code{idx}}{An \eqn{M}-long vector containing the indices
      of the row of \code{t} in which each point in \code{xi} is found.}
   \item{\code{p}}{An \eqn{M}-by-\eqn{N+1} matrix containing the
    barycentric coordinates with respect to the enclosing simplex
    of each point in \code{xi}.}}
}
\description{
For \code{t = delaunayn(x)}, where \code{x} is a set of points in \eqn{N}
dimensions, \code{tsearchn(x, t, xi)} finds the index in \code{t}
containing the points \code{xi}. For points outside the convex hull,
\code{idx} is \code{NA}. \code{tsearchn} also returns the barycentric
coordinates \code{p} of the enclosing triangles.
}
\details{
If \code{x} is \code{NA} and the \code{t} is a
\code{delaunayn} object produced by
\code{\link{delaunayn}} with the \code{full} option, then use the
Qhull library to perform the search. Please note that this is
experimental in geometry version 0.4.0 and is only partly tested
for 3D hulls, and does not yet work for hulls of 4 dimensions and
above.
}
\note{
Based on the Octave function Copyright (C) 2007-2012 David
  Bateman.
}
\seealso{
\code{\link{tsearch}}, \code{\link{delaunayn}}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cart2pol.R
\name{cart2pol}
\alias{cart2pol}
\title{Transform Cartesian coordinates to polar or cylindrical coordinates.}
\usage{
cart2pol(x, y = NULL, z = NULL)
}
\arguments{
\item{x}{x-coordinates or matrix with three columns}

\item{y}{y-coordinates (optional, if \code{x}) is a matrix}

\item{z}{z-coordinates (optional, if \code{x}) is a matrix}
}
\value{
A matrix \code{P} where each row represents one
  polar/(cylindrical) coordinate (\code{theta}, \code{r}, (,
  \code{z})).
}
\description{
The inputs \code{x}, \code{y} (, and \code{z}) must be the same shape, or
scalar.  If called with a single matrix argument then each row of \code{C}
represents the Cartesian coordinate (\code{x}, \code{y} (, \code{z})).
}
\seealso{
\code{\link{pol2cart}}, \code{\link{cart2sph}},
  \code{\link{sph2cart}}
}
\author{
Kai Habel

David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Unique.R
\name{Unique}
\alias{Unique}
\title{Extract Unique Rows}
\usage{
Unique(X, rows.are.sets = FALSE)
}
\arguments{
\item{X}{Numerical matrix.}

\item{rows.are.sets}{If \sQuote{\code{TRUE}}, rows are treated as sets -
i.e., to define uniqueness, the order of the rows does not matter.}
}
\value{
Matrix of the same number of columns as \code{x}, with the unique
rows in \code{x} sorted according to the columns of \code{x}. If
\code{rows.are.sets = TRUE} the rows are also sorted.
}
\description{
\sQuote{Unique} returns a vector, data frame or array like 'x' but with
duplicate elements removed.
}
\note{
\sQuote{\code{Unique}} is (under circumstances) much quicker than the
more generic base function \sQuote{\code{unique}}.
}
\examples{

# `Unique' is faster than `unique'
x = matrix(sample(1:(4*8),4*8),ncol=4)
y = x[sample(1:nrow(x),3000,TRUE), ]
gc(); system.time(unique(y))
gc(); system.time(Unique(y))

#
z = Unique(y)
x[matorder(x),]
z[matorder(z),]

}
\author{
Raoul Grasman
}
\keyword{arith}
\keyword{array}
\keyword{math}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tsearch.R
\name{bary2cart}
\alias{bary2cart}
\title{Conversion of Barycentric to Cartesian coordinates}
\usage{
bary2cart(X, Beta)
}
\arguments{
\item{X}{Reference simplex in \eqn{N} dimensions represented by a
\eqn{N+1}-by-\eqn{N} matrix}

\item{Beta}{\eqn{M} points in barycentric coordinates with
respect to the simplex \code{X} represented by a
\eqn{M}-by-\eqn{N+1} matrix}
}
\value{
\eqn{M}-by-\eqn{N} matrix in which each row is the
Cartesian coordinates of corresponding row of \code{Beta}
}
\description{
Given the barycentric coordinates of one or more points with
respect to a simplex, compute the Cartesian coordinates of these
points.
}
\examples{
## Define simplex in 2D (i.e. a triangle)
X <- rbind(c(0, 0),
           c(0, 1),
           c(1, 0))
## Cartesian cooridinates of points
beta <- rbind(c(0, 0.5, 0.5),
              c(0.1, 0.8, 0.1))
## Plot triangle and points
trimesh(rbind(1:3), X)
text(X[,1], X[,2], 1:3) # Label vertices
P <- bary2cart(X, beta)
points(P)
}
\seealso{
\code{\link{cart2bary}}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inhulln.R
\name{inhulln}
\alias{inhulln}
\title{Test if points lie in convex hull}
\usage{
inhulln(ch, p)
}
\arguments{
\item{ch}{Convex hull produced using \code{\link{convhulln}}}

\item{p}{An \eqn{M}-by-\eqn{N} matrix of points to test. The rows
of \code{p} represent \eqn{M} points in \eqn{N}-dimensional
space.}
}
\value{
A boolean vector with \eqn{M} elements
}
\description{
Tests if a set of points lies within a convex hull, returning a
boolean vector in which each element is \code{TRUE} if the
corresponding point lies within the hull and \code{FALSE} if it
lies outwith the hull or on one of its facets.
}
\note{
\code{inhulln} was introduced in geometry 0.4.0, and is
  still under development. It is worth checking results for
  unexpected behaviour.
}
\examples{
p <- cbind(c(-1, -1, 1), c(-1, 1, -1))
ch <- convhulln(p)
## First point should be in the hull; last two outside
inhulln(ch, rbind(c(-0.5, -0.5),
                  c( 1  ,  1),
                  c(10  ,  0)))

## Test hypercube
p <- rbox(D=4, B=1)
ch <- convhulln(p)
tp <-  cbind(seq(-1.9, 1.9, by=0.2), 0, 0, 0)
pin <- inhulln(ch, tp)
## Points on x-axis should be in box only betw,een -1 and 1
pin == (tp[,1] < 1 & tp[,1] > -1)
}
\seealso{
\code{\link{convhulln}}, \code{point.in.polygon} in \pkg{sp}
}
\author{
David Sterratt
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimesh.R
\name{trimesh}
\alias{trimesh}
\title{Display triangles mesh (2D)}
\usage{
trimesh(T, p, p2, add = FALSE, axis = FALSE, boxed = FALSE, ...)
}
\arguments{
\item{T}{T is a \code{m}-by-3 matrix. A row of \code{T} contains
indices into \code{X} of the vertices of a triangle. \code{T} is
usually the output of \code{\link{delaunayn}}.}

\item{p}{A vector or a matrix.}

\item{p2}{if \code{p} is not a matrix \code{p} and \code{p2} are bind to a
matrix with \code{cbind}.}

\item{add}{Add to existing plot in current active device?}

\item{axis}{Draw axes?}

\item{boxed}{Plot box?}

\item{\dots}{Parameters to the rendering device. See the \link[rgl]{rgl}
package.}
}
\description{
\code{trimesh(T, p)} displays the triangles defined in the m-by-3
matrix \code{T} and points \code{p} as a mesh.  Each row of
\code{T} specifies a triangle by giving the 3 indices of its
points in \code{X}.
}
\examples{
#example trimesh
p = cbind(x=rnorm(30), y=rnorm(30))
tt = delaunayn(p)
trimesh(tt,p)
}
\seealso{
\code{\link{tetramesh}}, \code{\link[rgl]{rgl}},
\code{\link{delaunayn}}, \code{\link{convhulln}},
\code{\link{surf.tri}}
}
\author{
Raoul Grasman
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polyarea.R
\name{polyarea}
\alias{polyarea}
\title{Determines area of a polygon by triangle method.}
\usage{
polyarea(x, y, d = 1)
}
\arguments{
\item{x}{X coordinates of vertices.}

\item{y}{Y coordinates of vertices.}

\item{d}{Dimension of array to work along.}
}
\value{
Area(s) of polygon(s).
}
\description{
Determines area of a polygon by triangle method.  The variables
\code{x} and \code{y} define the vertex pairs, and must therefore
have the same shape.  They can be either vectors or arrays.  If
they are arrays then the columns of \code{x} and \code{y} are
treated separately and an area returned for each.
}
\details{
If the optional \code{dim} argument is given, then \code{polyarea}
works along this dimension of the arrays \code{x} and \code{y}.
}
\examples{
x <- c(1, 1, 3, 3, 1)
y <- c(1, 3, 3, 1, 1)
polyarea(x, y)
polyarea(cbind(x, x), cbind(y, y)) ##  c(4, 4)
polyarea(cbind(x, x), cbind(y, y), 1) ##  c(4, 4)
polyarea(rbind(x, x), rbind(y, y), 2) ##  c(4, 4)
}
\author{
David Sterratt based on the octave sources by David M. Doolin
}
